// line php7/php7.y:2
package php7

import __yyfmt__ "fmt"

// line php7/php7.y:2
import (
	"strconv"
	"strings"

	"github.com/z7zmey/php-parser/freefloating"
	"github.com/z7zmey/php-parser/node"
	"github.com/z7zmey/php-parser/node/expr"
	"github.com/z7zmey/php-parser/node/expr/assign"
	"github.com/z7zmey/php-parser/node/expr/binary"
	"github.com/z7zmey/php-parser/node/expr/cast"
	"github.com/z7zmey/php-parser/node/name"
	"github.com/z7zmey/php-parser/node/scalar"
	"github.com/z7zmey/php-parser/node/stmt"
	"github.com/z7zmey/php-parser/scanner"
)

// line php7/php7.y:22
type yySymType struct {
	yys   int
	node  node.Node
	token *scanner.Token
	list  []node.Node
	str   string

	ClassExtends     *stmt.ClassExtends
	ClassImplements  *stmt.ClassImplements
	InterfaceExtends *stmt.InterfaceExtends
	ClosureUse       *expr.ClosureUse
}

const T_INCLUDE = 57346
const T_INCLUDE_ONCE = 57347
const T_EXIT = 57348
const T_IF = 57349
const T_LNUMBER = 57350
const T_DNUMBER = 57351
const T_STRING = 57352
const T_STRING_VARNAME = 57353
const T_VARIABLE = 57354
const T_NUM_STRING = 57355
const T_INLINE_HTML = 57356
const T_CHARACTER = 57357
const T_BAD_CHARACTER = 57358
const T_ENCAPSED_AND_WHITESPACE = 57359
const T_CONSTANT_ENCAPSED_STRING = 57360
const T_ECHO = 57361
const T_DO = 57362
const T_WHILE = 57363
const T_ENDWHILE = 57364
const T_FOR = 57365
const T_ENDFOR = 57366
const T_FOREACH = 57367
const T_ENDFOREACH = 57368
const T_DECLARE = 57369
const T_ENDDECLARE = 57370
const T_AS = 57371
const T_SWITCH = 57372
const T_ENDSWITCH = 57373
const T_CASE = 57374
const T_DEFAULT = 57375
const T_BREAK = 57376
const T_CONTINUE = 57377
const T_GOTO = 57378
const T_FUNCTION = 57379
const T_FN = 57380
const T_CONST = 57381
const T_RETURN = 57382
const T_TRY = 57383
const T_CATCH = 57384
const T_FINALLY = 57385
const T_THROW = 57386
const T_USE = 57387
const T_INSTEADOF = 57388
const T_GLOBAL = 57389
const T_VAR = 57390
const T_UNSET = 57391
const T_ISSET = 57392
const T_EMPTY = 57393
const T_HALT_COMPILER = 57394
const T_CLASS = 57395
const T_TRAIT = 57396
const T_INTERFACE = 57397
const T_EXTENDS = 57398
const T_IMPLEMENTS = 57399
const T_OBJECT_OPERATOR = 57400
const T_DOUBLE_ARROW = 57401
const T_LIST = 57402
const T_ARRAY = 57403
const T_CALLABLE = 57404
const T_CLASS_C = 57405
const T_TRAIT_C = 57406
const T_METHOD_C = 57407
const T_FUNC_C = 57408
const T_LINE = 57409
const T_FILE = 57410
const T_COMMENT = 57411
const T_DOC_COMMENT = 57412
const T_OPEN_TAG = 57413
const T_OPEN_TAG_WITH_ECHO = 57414
const T_CLOSE_TAG = 57415
const T_WHITESPACE = 57416
const T_START_HEREDOC = 57417
const T_END_HEREDOC = 57418
const T_DOLLAR_OPEN_CURLY_BRACES = 57419
const T_CURLY_OPEN = 57420
const T_PAAMAYIM_NEKUDOTAYIM = 57421
const T_NAMESPACE = 57422
const T_NS_C = 57423
const T_DIR = 57424
const T_NS_SEPARATOR = 57425
const T_ELLIPSIS = 57426
const T_EVAL = 57427
const T_REQUIRE = 57428
const T_REQUIRE_ONCE = 57429
const T_LOGICAL_OR = 57430
const T_LOGICAL_XOR = 57431
const T_LOGICAL_AND = 57432
const T_INSTANCEOF = 57433
const T_NEW = 57434
const T_CLONE = 57435
const T_ELSEIF = 57436
const T_ELSE = 57437
const T_ENDIF = 57438
const T_PRINT = 57439
const T_YIELD = 57440
const T_STATIC = 57441
const T_ABSTRACT = 57442
const T_FINAL = 57443
const T_PRIVATE = 57444
const T_PROTECTED = 57445
const T_PUBLIC = 57446
const T_INC = 57447
const T_DEC = 57448
const T_YIELD_FROM = 57449
const T_INT_CAST = 57450
const T_DOUBLE_CAST = 57451
const T_STRING_CAST = 57452
const T_ARRAY_CAST = 57453
const T_OBJECT_CAST = 57454
const T_BOOL_CAST = 57455
const T_UNSET_CAST = 57456
const T_COALESCE = 57457
const T_SPACESHIP = 57458
const T_NOELSE = 57459
const T_PLUS_EQUAL = 57460
const T_MINUS_EQUAL = 57461
const T_MUL_EQUAL = 57462
const T_POW_EQUAL = 57463
const T_DIV_EQUAL = 57464
const T_CONCAT_EQUAL = 57465
const T_MOD_EQUAL = 57466
const T_AND_EQUAL = 57467
const T_OR_EQUAL = 57468
const T_XOR_EQUAL = 57469
const T_SL_EQUAL = 57470
const T_SR_EQUAL = 57471
const T_COALESCE_EQUAL = 57472
const T_BOOLEAN_OR = 57473
const T_BOOLEAN_AND = 57474
const T_POW = 57475
const T_SL = 57476
const T_SR = 57477
const T_IS_IDENTICAL = 57478
const T_IS_NOT_IDENTICAL = 57479
const T_IS_EQUAL = 57480
const T_IS_NOT_EQUAL = 57481
const T_IS_SMALLER_OR_EQUAL = 57482
const T_IS_GREATER_OR_EQUAL = 57483

var yyToknames = [...]string{
	"$end",
	"error",
	"$unk",
	"T_INCLUDE",
	"T_INCLUDE_ONCE",
	"T_EXIT",
	"T_IF",
	"T_LNUMBER",
	"T_DNUMBER",
	"T_STRING",
	"T_STRING_VARNAME",
	"T_VARIABLE",
	"T_NUM_STRING",
	"T_INLINE_HTML",
	"T_CHARACTER",
	"T_BAD_CHARACTER",
	"T_ENCAPSED_AND_WHITESPACE",
	"T_CONSTANT_ENCAPSED_STRING",
	"T_ECHO",
	"T_DO",
	"T_WHILE",
	"T_ENDWHILE",
	"T_FOR",
	"T_ENDFOR",
	"T_FOREACH",
	"T_ENDFOREACH",
	"T_DECLARE",
	"T_ENDDECLARE",
	"T_AS",
	"T_SWITCH",
	"T_ENDSWITCH",
	"T_CASE",
	"T_DEFAULT",
	"T_BREAK",
	"T_CONTINUE",
	"T_GOTO",
	"T_FUNCTION",
	"T_FN",
	"T_CONST",
	"T_RETURN",
	"T_TRY",
	"T_CATCH",
	"T_FINALLY",
	"T_THROW",
	"T_USE",
	"T_INSTEADOF",
	"T_GLOBAL",
	"T_VAR",
	"T_UNSET",
	"T_ISSET",
	"T_EMPTY",
	"T_HALT_COMPILER",
	"T_CLASS",
	"T_TRAIT",
	"T_INTERFACE",
	"T_EXTENDS",
	"T_IMPLEMENTS",
	"T_OBJECT_OPERATOR",
	"T_DOUBLE_ARROW",
	"T_LIST",
	"T_ARRAY",
	"T_CALLABLE",
	"T_CLASS_C",
	"T_TRAIT_C",
	"T_METHOD_C",
	"T_FUNC_C",
	"T_LINE",
	"T_FILE",
	"T_COMMENT",
	"T_DOC_COMMENT",
	"T_OPEN_TAG",
	"T_OPEN_TAG_WITH_ECHO",
	"T_CLOSE_TAG",
	"T_WHITESPACE",
	"T_START_HEREDOC",
	"T_END_HEREDOC",
	"T_DOLLAR_OPEN_CURLY_BRACES",
	"T_CURLY_OPEN",
	"T_PAAMAYIM_NEKUDOTAYIM",
	"T_NAMESPACE",
	"T_NS_C",
	"T_DIR",
	"T_NS_SEPARATOR",
	"T_ELLIPSIS",
	"T_EVAL",
	"T_REQUIRE",
	"T_REQUIRE_ONCE",
	"T_LOGICAL_OR",
	"T_LOGICAL_XOR",
	"T_LOGICAL_AND",
	"T_INSTANCEOF",
	"T_NEW",
	"T_CLONE",
	"T_ELSEIF",
	"T_ELSE",
	"T_ENDIF",
	"T_PRINT",
	"T_YIELD",
	"T_STATIC",
	"T_ABSTRACT",
	"T_FINAL",
	"T_PRIVATE",
	"T_PROTECTED",
	"T_PUBLIC",
	"T_INC",
	"T_DEC",
	"T_YIELD_FROM",
	"T_INT_CAST",
	"T_DOUBLE_CAST",
	"T_STRING_CAST",
	"T_ARRAY_CAST",
	"T_OBJECT_CAST",
	"T_BOOL_CAST",
	"T_UNSET_CAST",
	"T_COALESCE",
	"T_SPACESHIP",
	"T_NOELSE",
	"T_PLUS_EQUAL",
	"T_MINUS_EQUAL",
	"T_MUL_EQUAL",
	"T_POW_EQUAL",
	"T_DIV_EQUAL",
	"T_CONCAT_EQUAL",
	"T_MOD_EQUAL",
	"T_AND_EQUAL",
	"T_OR_EQUAL",
	"T_XOR_EQUAL",
	"T_SL_EQUAL",
	"T_SR_EQUAL",
	"T_COALESCE_EQUAL",
	"T_BOOLEAN_OR",
	"T_BOOLEAN_AND",
	"T_POW",
	"T_SL",
	"T_SR",
	"T_IS_IDENTICAL",
	"T_IS_NOT_IDENTICAL",
	"T_IS_EQUAL",
	"T_IS_NOT_EQUAL",
	"T_IS_SMALLER_OR_EQUAL",
	"T_IS_GREATER_OR_EQUAL",
	"'\"'",
	"'`'",
	"'{'",
	"'}'",
	"';'",
	"':'",
	"'('",
	"')'",
	"'['",
	"']'",
	"'?'",
	"'&'",
	"'-'",
	"'+'",
	"'!'",
	"'~'",
	"'@'",
	"'$'",
	"','",
	"'|'",
	"'='",
	"'^'",
	"'*'",
	"'/'",
	"'%'",
	"'<'",
	"'>'",
	"'.'",
}
var yyStatenames = [...]string{}

const yyEofCode = 1
const yyErrCode = 2
const yyInitialStackSize = 16

// line php7/php7.y:5666

// line yacctab:1
var yyExca = [...]int{
	-1, 1,
	1, -1,
	-2, 0,
	-1, 2,
	1, 1,
	-2, 0,
	-1, 44,
	58, 426,
	79, 426,
	144, 426,
	150, 426,
	-2, 421,
	-1, 48,
	148, 429,
	-2, 438,
	-1, 85,
	58, 428,
	79, 428,
	144, 428,
	148, 431,
	150, 428,
	-2, 416,
	-1, 109,
	79, 389,
	-2, 418,
	-1, 233,
	58, 426,
	79, 426,
	144, 426,
	150, 426,
	-2, 315,
	-1, 236,
	148, 431,
	-2, 428,
	-1, 239,
	58, 426,
	79, 426,
	144, 426,
	150, 426,
	-2, 317,
	-1, 358,
	116, 0,
	136, 0,
	137, 0,
	138, 0,
	139, 0,
	-2, 339,
	-1, 359,
	116, 0,
	136, 0,
	137, 0,
	138, 0,
	139, 0,
	-2, 340,
	-1, 360,
	116, 0,
	136, 0,
	137, 0,
	138, 0,
	139, 0,
	-2, 341,
	-1, 361,
	116, 0,
	136, 0,
	137, 0,
	138, 0,
	139, 0,
	-2, 342,
	-1, 362,
	140, 0,
	141, 0,
	167, 0,
	168, 0,
	-2, 343,
	-1, 363,
	140, 0,
	141, 0,
	167, 0,
	168, 0,
	-2, 344,
	-1, 364,
	140, 0,
	141, 0,
	167, 0,
	168, 0,
	-2, 345,
	-1, 365,
	140, 0,
	141, 0,
	167, 0,
	168, 0,
	-2, 346,
	-1, 366,
	116, 0,
	136, 0,
	137, 0,
	138, 0,
	139, 0,
	-2, 347,
	-1, 373,
	149, 164,
	160, 164,
	-2, 426,
	-1, 418,
	149, 466,
	151, 466,
	160, 466,
	-2, 426,
	-1, 423,
	58, 427,
	79, 427,
	144, 427,
	148, 430,
	150, 427,
	-2, 349,
	-1, 437,
	148, 452,
	-2, 419,
	-1, 438,
	148, 454,
	-2, 444,
	-1, 518,
	148, 452,
	-2, 420,
	-1, 519,
	148, 454,
	-2, 445,
	-1, 536,
	149, 214,
	-2, 219,
	-1, 578,
	149, 214,
	-2, 219,
	-1, 603,
	148, 430,
	-2, 427,
	-1, 669,
	149, 184,
	-2, 426,
	-1, 677,
	149, 214,
	-2, 219,
	-1, 693,
	149, 465,
	151, 465,
	160, 465,
	-2, 426,
	-1, 730,
	149, 185,
	-2, 426,
	-1, 746,
	37, 269,
	39, 269,
	-2, 266,
	-1, 760,
	94, 209,
	95, 209,
	96, 209,
	-2, 0,
	-1, 789,
	149, 184,
	-2, 426,
	-1, 791,
	149, 187,
	-2, 400,
	-1, 806,
	94, 210,
	95, 210,
	96, 210,
	-2, 0,
	-1, 856,
	31, 200,
	32, 200,
	33, 200,
	145, 200,
	-2, 0,
	-1, 888,
	29, 77,
	-2, 81,
	-1, 893,
	31, 199,
	32, 199,
	33, 199,
	145, 199,
	-2, 0,
	-1, 921,
	149, 214,
	-2, 219,
}

const yyPrivate = 57344

const yyLast = 7766

var yyAct = [...]int{

	28, 132, 765, 631, 843, 880, 866, 585, 442, 748,
	842, 109, 785, 820, 191, 839, 635, 682, 113, 634,
	668, 128, 140, 140, 140, 712, 650, 153, 380, 572,
	117, 123, 633, 649, 224, 188, 327, 580, 540, 529,
	636, 410, 372, 521, 318, 723, 226, 5, 322, 81,
	382, 9, 321, 228, 232, 8, 85, 240, 241, 242,
	243, 244, 152, 134, 245, 246, 247, 248, 249, 250,
	251, 320, 254, 149, 7, 262, 263, 264, 319, 145,
	2, 6, 130, 127, 139, 520, 436, 268, 129, 876,
	277, 278, 873, 280, 281, 850, 691, 597, 273, 258,
	83, 860, 338, 314, 234, 234, 899, 142, 143, 874,
	870, 236, 236, 847, 411, 846, 107, 900, 549, 575,
	684, 287, 339, 875, 871, 771, 716, 336, 334, 684,
	313, 312, 307, 295, 324, 297, 340, 306, 329, 330,
	304, 337, 335, 310, 706, 313, 311, 643, 628, 573,
	326, 307, 564, 290, 292, 181, 341, 342, 343, 344,
	345, 346, 347, 348, 349, 350, 351, 352, 353, 354,
	355, 356, 357, 358, 359, 360, 361, 362, 363, 364,
	365, 366, 416, 368, 370, 192, 374, 270, 708, 376,
	44, 791, 222, 273, 119, 107, 107, 167, 700, 300,
	607, 696, 610, 608, 392, 394, 395, 396, 397, 398,
	399, 400, 401, 402, 403, 404, 405, 406, 165, 164,
	407, 140, 409, 181, 228, 384, 867, 107, 166, 168,
	169, 419, 617, 163, 234, 604, 421, 272, 613, 228,
	119, 236, 107, 614, 388, 233, 239, 592, 412, 415,
	861, 265, 932, 921, 140, 895, 430, 828, 414, 181,
	827, 431, 818, 108, 136, 167, 140, 114, 807, 367,
	795, 437, 518, 422, 80, 375, 530, 531, 221, 234,
	532, 835, 270, 237, 220, 525, 236, 738, 728, 537,
	710, 705, 541, 703, 228, 119, 166, 168, 169, 146,
	695, 167, 170, 171, 666, 408, 526, 655, 177, 179,
	136, 234, 645, 114, 605, 559, 596, 793, 236, 296,
	822, 821, 165, 164, 551, 731, 554, 311, 543, 237,
	694, 927, 166, 168, 169, 176, 178, 163, 424, 568,
	677, 153, 108, 108, 609, 5, 638, 639, 429, 9,
	435, 291, 663, 8, 517, 664, 426, 427, 527, 288,
	578, 274, 425, 910, 562, 136, 560, 868, 114, 536,
	571, 524, 7, 420, 108, 373, 389, 523, 294, 6,
	546, 387, 426, 289, 427, 427, 426, 293, 587, 108,
	588, 552, 279, 589, 590, 582, 558, 276, 586, 275,
	253, 223, 219, 569, 186, 185, 184, 138, 577, 137,
	133, 567, 115, 595, 584, 566, 766, 228, 599, 758,
	418, 228, 391, 157, 159, 158, 181, 936, 190, 935,
	909, 851, 579, 780, 781, 616, 894, 637, 857, 299,
	619, 298, 181, 780, 781, 829, 824, 817, 777, 759,
	183, 180, 433, 727, 594, 602, 274, 726, 724, 722,
	719, 563, 548, 598, 545, 390, 155, 156, 167, 170,
	171, 172, 173, 174, 175, 177, 179, 378, 333, 332,
	618, 822, 821, 331, 167, 170, 171, 182, 161, 165,
	164, 301, 816, 119, 813, 808, 160, 413, 162, 166,
	168, 169, 176, 178, 163, 165, 164, 773, 615, 544,
	915, 544, 544, 217, 218, 166, 168, 169, 864, 863,
	163, 544, 809, 798, 140, 623, 204, 205, 206, 207,
	208, 209, 210, 211, 212, 213, 214, 215, 216, 794,
	740, 438, 519, 644, 683, 167, 823, 583, 640, 187,
	198, 199, 181, 257, 122, 77, 779, 193, 627, 769,
	116, 620, 119, 659, 329, 661, 116, 624, 309, 642,
	203, 665, 554, 294, 554, 311, 913, 651, 626, 309,
	309, 148, 678, 745, 522, 267, 747, 259, 654, 681,
	640, 5, 428, 586, 167, 9, 266, 119, 692, 8,
	660, 200, 202, 201, 78, 79, 657, 119, 622, 680,
	146, 309, 119, 638, 639, 698, 667, 679, 7, 849,
	674, 729, 294, 234, 234, 6, 581, 118, 259, 530,
	236, 236, 136, 648, 557, 114, 688, 752, 753, 754,
	751, 750, 749, 714, 541, 653, 647, 383, 309, 640,
	234, 386, 260, 261, 911, 555, 797, 236, 284, 285,
	720, 119, 735, 736, 702, 550, 554, 136, 704, 713,
	114, 554, 554, 228, 717, 651, 303, 733, 709, 715,
	553, 737, 711, 889, 802, 305, 801, 718, 125, 640,
	126, 760, 761, 260, 261, 228, 756, 148, 119, 119,
	107, 755, 308, 135, 106, 46, 112, 732, 739, 912,
	725, 752, 753, 754, 751, 750, 749, 535, 640, 259,
	234, 325, 697, 329, 428, 840, 125, 236, 126, 762,
	757, 764, 228, 554, 124, 554, 713, 901, 651, 767,
	772, 812, 770, 774, 194, 150, 259, 119, 775, 111,
	788, 428, 1, 803, 259, 640, 804, 586, 799, 778,
	806, 611, 652, 800, 373, 669, 790, 38, 136, 259,
	782, 114, 784, 786, 286, 121, 131, 259, 640, 234,
	556, 819, 282, 811, 260, 261, 236, 237, 150, 826,
	554, 693, 544, 814, 810, 381, 832, 621, 833, 834,
	413, 625, 815, 746, 379, 831, 825, 780, 781, 197,
	534, 260, 261, 135, 106, 196, 836, 844, 533, 260,
	261, 743, 856, 259, 783, 780, 781, 848, 256, 195,
	189, 744, 632, 855, 260, 261, 235, 539, 238, 225,
	859, 283, 260, 261, 528, 865, 371, 108, 879, 877,
	890, 891, 886, 147, 838, 144, 892, 893, 328, 885,
	151, 730, 841, 853, 676, 897, 898, 830, 255, 872,
	796, 385, 786, 896, 903, 656, 852, 905, 934, 662,
	845, 227, 43, 42, 886, 908, 904, 902, 260, 261,
	16, 885, 15, 606, 271, 49, 48, 110, 50, 84,
	82, 72, 252, 62, 269, 61, 906, 918, 884, 883,
	882, 881, 742, 45, 734, 673, 315, 928, 926, 922,
	789, 924, 925, 586, 120, 929, 302, 3, 441, 768,
	930, 707, 919, 640, 920, 933, 0, 4, 937, 89,
	90, 70, 47, 94, 95, 36, 0, 107, 0, 27,
	0, 0, 0, 112, 26, 18, 17, 0, 19, 0,
	30, 0, 31, 0, 0, 20, 0, 0, 0, 21,
	22, 35, 37, 106, 13, 23, 33, 0, 0, 34,
	12, 0, 24, 0, 29, 87, 88, 10, 39, 40,
	41, 0, 0, 0, 0, 51, 111, 0, 103, 99,
	100, 101, 96, 97, 745, 0, 0, 747, 0, 0,
	104, 0, 0, 0, 0, 11, 102, 98, 114, 0,
	91, 92, 93, 0, 0, 0, 0, 86, 53, 0,
	0, 0, 74, 75, 25, 78, 79, 0, 0, 0,
	54, 55, 76, 63, 64, 65, 66, 67, 68, 69,
	0, 0, 0, 0, 0, 0, 0, 0, 752, 753,
	754, 751, 750, 749, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 105, 73, 14,
	646, 32, 0, 60, 0, 52, 0, 0, 0, 57,
	56, 58, 59, 71, 108, 4, 0, 89, 90, 70,
	47, 94, 95, 36, 869, 107, 0, 27, 0, 0,
	0, 112, 26, 18, 17, 0, 19, 0, 30, 0,
	31, 0, 0, 20, 0, 0, 0, 21, 22, 35,
	37, 106, 13, 23, 33, 0, 0, 34, 12, 0,
	24, 0, 29, 87, 88, 10, 39, 40, 41, 0,
	0, 0, 0, 51, 111, 0, 103, 99, 100, 101,
	96, 97, 745, 0, 0, 747, 0, 0, 104, 0,
	0, 0, 0, 11, 102, 98, 114, 0, 91, 92,
	93, 0, 0, 0, 0, 86, 53, 0, 0, 0,
	74, 75, 25, 78, 79, 0, 0, 0, 54, 55,
	76, 63, 64, 65, 66, 67, 68, 69, 0, 0,
	0, 0, 0, 0, 0, 0, 752, 753, 754, 751,
	750, 749, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 105, 73, 14, 547, 32,
	0, 60, 0, 52, 0, 0, 0, 57, 56, 58,
	59, 71, 108, 4, 0, 89, 90, 70, 47, 94,
	95, 36, 837, 107, 0, 27, 0, 0, 0, 112,
	26, 18, 17, 0, 19, 0, 30, 0, 31, 0,
	0, 20, 0, 0, 0, 21, 22, 35, 37, 106,
	13, 23, 33, 0, 0, 34, 12, 0, 24, 0,
	29, 87, 88, 10, 39, 40, 41, 0, 0, 0,
	0, 51, 111, 0, 103, 99, 100, 101, 96, 97,
	745, 0, 0, 747, 0, 0, 104, 0, 0, 0,
	0, 11, 102, 98, 114, 0, 91, 92, 93, 0,
	0, 0, 0, 86, 53, 0, 0, 0, 74, 75,
	25, 78, 79, 0, 0, 0, 54, 55, 76, 63,
	64, 65, 66, 67, 68, 69, 0, 0, 0, 0,
	0, 0, 0, 0, 752, 753, 754, 751, 750, 749,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 105, 73, 14, 0, 32, 0, 60,
	0, 52, 0, 0, 0, 57, 56, 58, 59, 71,
	108, 317, 0, 89, 90, 70, 47, 94, 95, 36,
	805, 107, 0, 27, 0, 0, 0, 112, 26, 18,
	17, 0, 19, 0, 30, 0, 31, 0, 0, 20,
	0, 0, 0, 21, 22, 35, 37, 106, 0, 23,
	33, 0, 0, 34, 0, 0, 24, 0, 29, 87,
	88, 323, 39, 40, 41, 0, 0, 0, 0, 51,
	111, 0, 103, 99, 100, 101, 96, 97, 745, 0,
	0, 747, 0, 0, 104, 0, 0, 0, 0, 136,
	102, 98, 114, 0, 91, 92, 93, 0, 0, 0,
	0, 86, 53, 0, 0, 0, 74, 75, 25, 78,
	79, 0, 0, 0, 54, 55, 76, 63, 64, 65,
	66, 67, 68, 69, 0, 0, 0, 0, 0, 0,
	0, 0, 752, 753, 754, 751, 750, 749, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 105, 73, 14, 938, 32, 0, 60, 0, 52,
	0, 0, 0, 57, 56, 58, 59, 71, 108, 317,
	0, 89, 90, 70, 47, 94, 95, 36, 741, 107,
	0, 27, 0, 0, 0, 112, 26, 18, 17, 0,
	19, 0, 30, 0, 31, 0, 0, 20, 0, 0,
	0, 21, 22, 35, 37, 106, 0, 23, 33, 0,
	0, 34, 0, 0, 24, 0, 29, 87, 88, 323,
	39, 40, 41, 0, 0, 0, 0, 51, 111, 0,
	103, 99, 100, 101, 96, 97, 0, 0, 0, 0,
	0, 0, 104, 0, 0, 0, 0, 136, 102, 98,
	114, 0, 91, 92, 93, 0, 0, 0, 0, 86,
	53, 0, 0, 0, 74, 75, 25, 78, 79, 0,
	0, 0, 54, 55, 76, 63, 64, 65, 66, 67,
	68, 69, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 105,
	73, 14, 931, 32, 0, 60, 0, 52, 0, 0,
	0, 57, 56, 58, 59, 71, 108, 317, 0, 89,
	90, 70, 47, 94, 95, 36, 0, 107, 0, 27,
	0, 0, 0, 112, 26, 18, 17, 0, 19, 0,
	30, 0, 31, 0, 0, 20, 0, 0, 0, 21,
	22, 35, 37, 106, 0, 23, 33, 0, 0, 34,
	0, 0, 24, 0, 29, 87, 88, 323, 39, 40,
	41, 0, 0, 0, 0, 51, 111, 0, 103, 99,
	100, 101, 96, 97, 0, 0, 0, 0, 0, 0,
	104, 0, 0, 0, 0, 136, 102, 98, 114, 0,
	91, 92, 93, 0, 0, 0, 0, 86, 53, 0,
	0, 0, 74, 75, 25, 78, 79, 0, 0, 0,
	54, 55, 76, 63, 64, 65, 66, 67, 68, 69,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 105, 73, 14,
	917, 32, 0, 60, 0, 52, 0, 0, 0, 57,
	56, 58, 59, 71, 108, 317, 0, 89, 90, 70,
	47, 94, 95, 36, 0, 107, 0, 27, 0, 0,
	0, 112, 26, 18, 17, 0, 19, 0, 30, 0,
	31, 0, 0, 20, 0, 0, 0, 21, 22, 35,
	37, 106, 0, 23, 33, 0, 0, 34, 0, 0,
	24, 0, 29, 87, 88, 323, 39, 40, 41, 0,
	0, 0, 0, 51, 111, 0, 103, 99, 100, 101,
	96, 97, 0, 0, 0, 0, 0, 0, 104, 0,
	0, 0, 0, 136, 102, 98, 114, 0, 91, 92,
	93, 0, 0, 0, 0, 86, 53, 0, 0, 0,
	74, 75, 25, 78, 79, 0, 0, 0, 54, 55,
	76, 63, 64, 65, 66, 67, 68, 69, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 105, 73, 14, 916, 32,
	0, 60, 0, 52, 0, 0, 0, 57, 56, 58,
	59, 71, 108, 317, 0, 89, 90, 70, 47, 94,
	95, 36, 0, 107, 0, 27, 0, 0, 0, 112,
	26, 18, 17, 0, 19, 914, 30, 0, 31, 0,
	0, 20, 0, 0, 0, 21, 22, 35, 37, 106,
	0, 23, 33, 0, 0, 34, 0, 0, 24, 0,
	29, 87, 88, 323, 39, 40, 41, 0, 0, 0,
	0, 51, 111, 0, 103, 99, 100, 101, 96, 97,
	0, 0, 0, 0, 0, 0, 104, 0, 0, 0,
	0, 136, 102, 98, 114, 0, 91, 92, 93, 0,
	0, 0, 0, 86, 53, 0, 0, 0, 74, 75,
	25, 78, 79, 0, 0, 0, 54, 55, 76, 63,
	64, 65, 66, 67, 68, 69, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 105, 73, 14, 0, 32, 0, 60,
	0, 52, 0, 0, 0, 57, 56, 58, 59, 71,
	108, 317, 0, 89, 90, 70, 47, 94, 95, 36,
	0, 107, 0, 27, 0, 0, 0, 112, 26, 18,
	17, 0, 19, 0, 30, 0, 31, 0, 0, 20,
	0, 0, 0, 21, 22, 35, 37, 106, 0, 23,
	33, 0, 0, 34, 0, 0, 24, 0, 29, 87,
	88, 323, 39, 40, 41, 0, 0, 0, 0, 51,
	111, 0, 103, 99, 100, 101, 96, 97, 0, 0,
	0, 0, 0, 0, 104, 0, 0, 0, 0, 136,
	102, 98, 114, 0, 91, 92, 93, 0, 0, 0,
	0, 86, 53, 0, 0, 0, 74, 75, 25, 78,
	79, 0, 0, 0, 54, 55, 76, 63, 64, 65,
	66, 67, 68, 69, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 105, 73, 14, 862, 32, 0, 60, 0, 52,
	0, 0, 0, 57, 56, 58, 59, 71, 108, 317,
	0, 89, 90, 70, 47, 94, 95, 36, 0, 107,
	0, 27, 0, 0, 0, 112, 26, 18, 17, 0,
	19, 0, 30, 858, 31, 0, 0, 20, 0, 0,
	0, 21, 22, 35, 37, 106, 0, 23, 33, 0,
	0, 34, 0, 0, 24, 0, 29, 87, 88, 323,
	39, 40, 41, 0, 0, 0, 0, 51, 111, 0,
	103, 99, 100, 101, 96, 97, 0, 0, 0, 0,
	0, 0, 104, 0, 0, 0, 0, 136, 102, 98,
	114, 0, 91, 92, 93, 0, 0, 0, 0, 86,
	53, 0, 0, 0, 74, 75, 25, 78, 79, 0,
	0, 0, 54, 55, 76, 63, 64, 65, 66, 67,
	68, 69, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 105,
	73, 14, 0, 32, 0, 60, 0, 52, 0, 0,
	0, 57, 56, 58, 59, 71, 108, 317, 0, 89,
	90, 70, 47, 94, 95, 36, 0, 107, 0, 27,
	0, 0, 0, 112, 26, 18, 17, 0, 19, 0,
	30, 0, 31, 792, 0, 20, 0, 0, 0, 21,
	22, 35, 37, 106, 0, 23, 33, 0, 0, 34,
	0, 0, 24, 0, 29, 87, 88, 323, 39, 40,
	41, 0, 0, 0, 0, 51, 111, 0, 103, 99,
	100, 101, 96, 97, 0, 0, 0, 0, 0, 0,
	104, 0, 0, 0, 0, 136, 102, 98, 114, 0,
	91, 92, 93, 0, 0, 0, 0, 86, 53, 0,
	0, 0, 74, 75, 25, 78, 79, 0, 0, 0,
	54, 55, 76, 63, 64, 65, 66, 67, 68, 69,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 105, 73, 14,
	0, 32, 0, 60, 0, 52, 0, 0, 0, 57,
	56, 58, 59, 71, 108, 317, 0, 89, 90, 70,
	47, 94, 95, 36, 0, 107, 0, 27, 0, 0,
	0, 112, 26, 18, 17, 776, 19, 0, 30, 0,
	31, 0, 0, 20, 0, 0, 0, 21, 22, 35,
	37, 106, 0, 23, 33, 0, 0, 34, 0, 0,
	24, 0, 29, 87, 88, 323, 39, 40, 41, 0,
	0, 0, 0, 51, 111, 0, 103, 99, 100, 101,
	96, 97, 0, 0, 0, 0, 0, 0, 104, 0,
	0, 0, 0, 136, 102, 98, 114, 0, 91, 92,
	93, 0, 0, 0, 0, 86, 53, 0, 0, 0,
	74, 75, 25, 78, 79, 0, 0, 0, 54, 55,
	76, 63, 64, 65, 66, 67, 68, 69, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 105, 73, 14, 0, 32,
	0, 60, 0, 52, 0, 0, 0, 57, 56, 58,
	59, 71, 108, 317, 0, 89, 90, 70, 47, 94,
	95, 36, 0, 107, 0, 27, 0, 0, 0, 112,
	26, 18, 17, 0, 19, 0, 30, 0, 31, 0,
	0, 20, 0, 0, 0, 21, 22, 35, 37, 106,
	0, 23, 33, 0, 0, 34, 0, 0, 24, 0,
	29, 87, 88, 323, 39, 40, 41, 0, 0, 0,
	0, 51, 111, 0, 103, 99, 100, 101, 96, 97,
	0, 0, 0, 0, 0, 0, 104, 0, 0, 0,
	0, 136, 102, 98, 114, 0, 91, 92, 93, 0,
	0, 0, 0, 86, 53, 0, 0, 687, 74, 75,
	25, 78, 79, 0, 0, 0, 54, 55, 76, 63,
	64, 65, 66, 67, 68, 69, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 105, 73, 14, 0, 32, 0, 60,
	0, 52, 0, 0, 0, 57, 56, 58, 59, 71,
	108, 317, 0, 89, 90, 70, 47, 94, 95, 36,
	0, 107, 0, 27, 0, 0, 0, 112, 26, 18,
	17, 0, 19, 0, 30, 0, 31, 0, 0, 20,
	0, 0, 0, 21, 22, 35, 37, 106, 0, 23,
	33, 0, 0, 34, 0, 0, 24, 0, 29, 87,
	88, 323, 39, 40, 41, 0, 0, 0, 0, 51,
	111, 0, 103, 99, 100, 101, 96, 97, 0, 0,
	0, 0, 0, 0, 104, 0, 0, 0, 0, 136,
	102, 98, 114, 0, 91, 92, 93, 0, 0, 0,
	0, 86, 53, 0, 0, 0, 74, 75, 25, 78,
	79, 0, 0, 0, 54, 55, 76, 63, 64, 65,
	66, 67, 68, 69, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 105, 73, 14, 576, 32, 0, 60, 0, 52,
	0, 0, 0, 57, 56, 58, 59, 71, 108, 317,
	0, 89, 90, 70, 47, 94, 95, 36, 0, 107,
	0, 27, 0, 0, 0, 112, 26, 18, 17, 0,
	19, 0, 30, 0, 31, 0, 0, 20, 0, 0,
	0, 21, 22, 35, 37, 106, 0, 23, 33, 0,
	0, 34, 0, 0, 24, 0, 29, 87, 88, 323,
	39, 40, 41, 0, 0, 0, 0, 51, 111, 0,
	103, 99, 100, 101, 96, 97, 0, 0, 0, 0,
	0, 0, 104, 0, 0, 0, 0, 136, 102, 98,
	114, 0, 91, 92, 93, 0, 0, 0, 0, 86,
	53, 0, 0, 0, 74, 75, 25, 78, 79, 0,
	0, 0, 54, 55, 76, 63, 64, 65, 66, 67,
	68, 69, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 105,
	73, 14, 316, 32, 0, 60, 0, 52, 0, 0,
	0, 57, 56, 58, 59, 71, 108, 317, 0, 89,
	90, 70, 47, 94, 95, 36, 0, 107, 0, 27,
	0, 0, 0, 112, 26, 18, 17, 0, 19, 0,
	30, 0, 31, 0, 0, 20, 0, 0, 0, 21,
	22, 35, 37, 106, 0, 23, 33, 0, 0, 34,
	0, 0, 24, 0, 29, 87, 88, 323, 39, 40,
	41, 0, 0, 0, 0, 51, 111, 0, 103, 99,
	100, 101, 96, 97, 0, 0, 0, 0, 0, 0,
	104, 0, 0, 0, 0, 136, 102, 98, 114, 0,
	91, 92, 93, 0, 0, 0, 0, 86, 53, 0,
	0, 0, 74, 75, 25, 78, 79, 0, 0, 0,
	54, 55, 76, 63, 64, 65, 66, 67, 68, 69,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 105, 73, 14,
	0, 32, 0, 60, 0, 52, 0, 0, 0, 57,
	56, 58, 59, 71, 108, 449, 450, 460, 461, 0,
	0, 440, 0, 107, 0, 0, 0, 0, 0, 0,
	465, 466, 467, 468, 469, 470, 471, 472, 473, 474,
	475, 495, 496, 497, 498, 499, 487, 488, 489, 516,
	490, 491, 476, 477, 478, 479, 480, 481, 482, 483,
	484, 485, 486, 0, 507, 505, 506, 502, 503, 0,
	0, 494, 500, 501, 508, 509, 511, 510, 512, 513,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 504, 515, 514, 0, 0, 451, 452, 453, 454,
	455, 456, 457, 458, 459, 462, 463, 464, 492, 493,
	443, 444, 445, 446, 447, 448, 89, 90, 70, 47,
	94, 95, 36, 0, 107, 0, 27, 0, 0, 0,
	112, 26, 18, 17, 0, 19, 0, 30, 0, 31,
	0, 0, 20, 0, 0, 0, 21, 22, 35, 135,
	106, 0, 23, 33, 0, 439, 34, 0, 0, 24,
	0, 29, 87, 88, 0, 0, 0, 0, 0, 0,
	108, 0, 51, 111, 0, 103, 99, 100, 101, 96,
	97, 0, 0, 0, 0, 0, 0, 104, 0, 0,
	0, 0, 136, 102, 98, 114, 0, 91, 92, 93,
	0, 0, 0, 0, 86, 53, 0, 0, 0, 74,
	75, 25, 0, 0, 0, 0, 0, 54, 55, 76,
	63, 64, 65, 66, 67, 68, 69, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 105, 73, 14, 0, 32, 787,
	60, 0, 52, 0, 0, 0, 57, 56, 58, 59,
	71, 108, 89, 90, 70, 47, 94, 95, 36, 0,
	107, 0, 27, 0, 0, 0, 112, 26, 18, 17,
	0, 19, 0, 30, 0, 31, 0, 0, 20, 0,
	0, 0, 21, 22, 35, 135, 106, 0, 23, 33,
	0, 0, 34, 0, 0, 24, 0, 29, 87, 88,
	0, 0, 0, 0, 0, 0, 0, 0, 51, 111,
	0, 103, 99, 100, 101, 96, 97, 0, 0, 0,
	0, 0, 0, 104, 0, 0, 0, 0, 136, 102,
	98, 114, 0, 91, 92, 93, 0, 0, 0, 0,
	86, 53, 0, 0, 0, 74, 75, 25, 0, 0,
	0, 0, 0, 54, 55, 76, 63, 64, 65, 66,
	67, 68, 69, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	105, 73, 14, 0, 32, 854, 60, 0, 52, 0,
	0, 0, 57, 56, 58, 59, 71, 108, 89, 90,
	70, 47, 94, 95, 36, 0, 107, 0, 27, 0,
	0, 0, 112, 26, 18, 17, 0, 19, 0, 30,
	0, 31, 0, 0, 20, 0, 0, 0, 21, 22,
	35, 135, 106, 0, 23, 33, 0, 0, 34, 0,
	0, 24, 0, 29, 87, 88, 0, 0, 0, 0,
	0, 0, 0, 0, 51, 111, 0, 103, 99, 100,
	101, 96, 97, 0, 0, 0, 0, 0, 0, 104,
	0, 0, 0, 0, 136, 102, 98, 114, 0, 91,
	92, 93, 0, 0, 0, 0, 86, 53, 0, 0,
	0, 74, 75, 25, 0, 0, 0, 0, 0, 54,
	55, 76, 63, 64, 65, 66, 67, 68, 69, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 105, 73, 14, 0,
	32, 689, 60, 0, 52, 0, 0, 0, 57, 56,
	58, 59, 71, 108, 89, 90, 70, 47, 94, 95,
	36, 0, 107, 0, 27, 0, 0, 0, 112, 26,
	18, 17, 0, 19, 0, 30, 0, 31, 0, 0,
	20, 0, 0, 0, 21, 22, 35, 135, 106, 0,
	23, 33, 0, 0, 34, 0, 0, 24, 0, 29,
	87, 88, 0, 0, 0, 0, 0, 0, 0, 0,
	51, 111, 0, 103, 99, 100, 101, 96, 97, 0,
	0, 0, 0, 0, 0, 104, 0, 0, 0, 0,
	136, 102, 98, 114, 0, 91, 92, 93, 0, 0,
	0, 0, 86, 53, 0, 0, 0, 74, 75, 25,
	0, 0, 0, 0, 0, 54, 55, 76, 63, 64,
	65, 66, 67, 68, 69, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 105, 73, 14, 0, 32, 675, 60, 0,
	52, 0, 0, 0, 57, 56, 58, 59, 71, 108,
	89, 90, 70, 47, 94, 95, 36, 0, 107, 0,
	27, 0, 0, 0, 112, 26, 18, 17, 0, 19,
	0, 30, 0, 31, 0, 0, 20, 0, 0, 0,
	21, 22, 35, 135, 106, 0, 23, 33, 0, 0,
	34, 0, 0, 24, 0, 29, 87, 88, 0, 0,
	0, 0, 0, 0, 0, 0, 51, 111, 0, 103,
	99, 100, 101, 96, 97, 0, 0, 0, 0, 0,
	0, 104, 0, 0, 0, 0, 136, 102, 98, 114,
	0, 91, 92, 93, 0, 0, 0, 0, 86, 53,
	0, 0, 0, 74, 75, 25, 0, 0, 0, 0,
	0, 54, 55, 76, 63, 64, 65, 66, 67, 68,
	69, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 105, 73,
	14, 0, 32, 658, 60, 0, 52, 0, 0, 0,
	57, 56, 58, 59, 71, 108, 89, 90, 70, 47,
	94, 95, 36, 0, 107, 0, 27, 0, 0, 0,
	112, 26, 18, 17, 0, 19, 0, 30, 0, 31,
	0, 0, 20, 0, 0, 0, 21, 22, 35, 135,
	106, 0, 23, 33, 0, 0, 34, 0, 0, 24,
	0, 29, 87, 88, 0, 0, 0, 0, 0, 0,
	0, 0, 51, 111, 0, 103, 99, 100, 101, 96,
	97, 0, 0, 0, 0, 0, 0, 104, 0, 0,
	0, 0, 136, 102, 98, 114, 0, 91, 92, 93,
	0, 0, 0, 0, 86, 53, 0, 0, 0, 74,
	75, 25, 0, 0, 0, 0, 0, 54, 55, 76,
	63, 64, 65, 66, 67, 68, 69, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 105, 73, 14, 0, 32, 0,
	60, 0, 52, 0, 0, 0, 57, 56, 58, 59,
	71, 108, 449, 450, 460, 461, 0, 0, 888, 0,
	0, 0, 0, 0, 0, 0, 0, 465, 466, 467,
	468, 469, 470, 471, 472, 473, 474, 475, 495, 496,
	497, 498, 499, 487, 488, 489, 516, 490, 491, 476,
	477, 478, 479, 480, 481, 482, 483, 484, 485, 486,
	0, 507, 505, 506, 502, 503, 0, 0, 494, 500,
	501, 508, 509, 511, 510, 512, 513, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 887, 515,
	514, 114, 0, 451, 452, 453, 454, 455, 456, 457,
	458, 459, 462, 463, 464, 492, 493, 443, 444, 445,
	446, 447, 448, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 449,
	450, 460, 461, 0, 0, 888, 0, 0, 0, 0,
	0, 0, 0, 907, 465, 466, 467, 468, 469, 470,
	471, 472, 473, 474, 475, 495, 496, 497, 498, 499,
	487, 488, 489, 516, 490, 491, 476, 477, 478, 479,
	480, 481, 482, 483, 484, 485, 486, 0, 507, 505,
	506, 502, 503, 0, 0, 494, 500, 501, 508, 509,
	511, 510, 512, 513, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 887, 515, 514, 114, 0,
	451, 452, 453, 454, 455, 456, 457, 458, 459, 462,
	463, 464, 492, 493, 443, 444, 445, 446, 447, 448,
	89, 90, 70, 0, 94, 95, 119, 0, 107, 0,
	0, 0, 0, 0, 112, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 135, 106, 0, 0, 0, 0, 0,
	878, 0, 0, 0, 0, 0, 87, 88, 0, 0,
	0, 0, 0, 0, 0, 0, 231, 111, 0, 103,
	99, 100, 101, 96, 97, 0, 0, 0, 0, 0,
	0, 104, 0, 0, 0, 0, 136, 102, 98, 114,
	230, 91, 92, 93, 0, 0, 0, 0, 86, 53,
	0, 0, 0, 74, 75, 141, 0, 0, 0, 0,
	0, 54, 55, 76, 63, 64, 65, 66, 67, 68,
	69, 0, 0, 0, 89, 90, 70, 0, 94, 95,
	119, 0, 107, 0, 0, 0, 0, 0, 112, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 105, 73,
	0, 0, 0, 0, 60, 0, 52, 135, 106, 229,
	57, 56, 58, 59, 71, 108, 0, 0, 0, 0,
	87, 88, 0, 0, 0, 0, 0, 0, 0, 0,
	51, 111, 0, 103, 99, 100, 101, 96, 97, 0,
	0, 0, 0, 0, 0, 104, 0, 0, 0, 0,
	136, 102, 98, 114, 542, 91, 92, 93, 0, 0,
	0, 0, 86, 53, 0, 0, 0, 74, 75, 141,
	0, 0, 0, 0, 0, 54, 55, 76, 63, 64,
	65, 66, 67, 68, 69, 0, 0, 0, 89, 90,
	70, 0, 94, 95, 119, 0, 107, 0, 0, 0,
	0, 0, 112, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 105, 73, 0, 0, 0, 0, 60, 538,
	52, 135, 106, 0, 57, 56, 58, 59, 71, 108,
	0, 0, 0, 0, 87, 88, 0, 0, 0, 0,
	0, 0, 0, 0, 51, 111, 0, 103, 99, 100,
	101, 96, 97, 0, 0, 0, 0, 0, 0, 104,
	0, 0, 0, 0, 136, 102, 98, 114, 542, 91,
	92, 93, 0, 0, 0, 0, 86, 53, 0, 0,
	0, 74, 75, 141, 0, 0, 0, 0, 0, 54,
	55, 76, 63, 64, 65, 66, 67, 68, 69, 0,
	0, 0, 89, 90, 70, 0, 94, 95, 119, 0,
	107, 0, 0, 0, 0, 0, 112, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 105, 73, 0, 0,
	0, 0, 60, 0, 52, 135, 106, 0, 57, 56,
	58, 59, 71, 108, 0, 0, 0, 0, 87, 88,
	0, 0, 0, 0, 0, 0, 0, 0, 601, 111,
	0, 103, 99, 100, 101, 96, 97, 0, 0, 0,
	0, 0, 0, 104, 0, 0, 0, 0, 136, 102,
	98, 114, 0, 91, 92, 93, 0, 0, 0, 0,
	86, 53, 0, 0, 0, 74, 75, 141, 0, 0,
	0, 0, 0, 54, 55, 76, 63, 64, 65, 66,
	67, 68, 69, 0, 0, 0, 89, 90, 70, 0,
	94, 95, 119, 432, 107, 0, 0, 0, 0, 0,
	112, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	105, 73, 0, 0, 0, 0, 60, 0, 52, 135,
	106, 600, 57, 56, 58, 59, 71, 108, 0, 0,
	0, 0, 87, 88, 0, 0, 0, 0, 0, 0,
	0, 0, 51, 111, 0, 103, 99, 100, 101, 96,
	97, 0, 0, 0, 0, 0, 0, 104, 0, 0,
	0, 0, 136, 102, 98, 114, 0, 91, 92, 93,
	0, 0, 0, 0, 86, 53, 0, 0, 0, 74,
	75, 141, 0, 0, 0, 0, 0, 54, 55, 76,
	63, 64, 65, 66, 67, 68, 69, 0, 0, 0,
	89, 90, 70, 0, 94, 95, 119, 0, 107, 0,
	0, 0, 0, 0, 112, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 105, 73, 0, 0, 0, 0,
	60, 0, 52, 135, 106, 0, 57, 56, 58, 59,
	71, 108, 0, 0, 0, 0, 87, 88, 0, 0,
	0, 0, 0, 0, 0, 0, 51, 111, 0, 103,
	99, 100, 101, 96, 97, 0, 0, 0, 0, 0,
	0, 104, 0, 0, 0, 0, 136, 102, 98, 114,
	0, 91, 92, 93, 0, 0, 0, 0, 86, 53,
	0, 0, 0, 74, 75, 141, 0, 0, 0, 0,
	0, 54, 55, 76, 63, 64, 65, 66, 67, 68,
	69, 0, 0, 0, 89, 90, 70, 0, 94, 95,
	119, 0, 107, 0, 0, 0, 0, 0, 112, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 105, 73,
	0, 0, 0, 0, 60, 0, 52, 135, 106, 393,
	57, 56, 58, 59, 71, 108, 0, 0, 0, 0,
	87, 88, 0, 0, 0, 0, 0, 0, 0, 0,
	51, 111, 0, 103, 99, 100, 101, 96, 97, 0,
	0, 0, 0, 0, 0, 104, 0, 0, 0, 0,
	136, 102, 98, 114, 0, 91, 92, 93, 0, 0,
	0, 0, 86, 53, 0, 0, 0, 74, 75, 141,
	0, 0, 0, 0, 0, 54, 55, 76, 63, 64,
	65, 66, 67, 68, 69, 0, 0, 0, 89, 90,
	70, 0, 94, 95, 119, 0, 107, 0, 0, 0,
	0, 0, 112, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 105, 73, 0, 0, 0, 369, 60, 0,
	52, 135, 106, 0, 57, 56, 58, 59, 71, 108,
	0, 0, 0, 0, 87, 88, 0, 0, 0, 0,
	0, 0, 0, 0, 51, 111, 0, 103, 99, 100,
	101, 96, 97, 0, 0, 0, 0, 0, 0, 104,
	0, 0, 0, 0, 136, 102, 98, 114, 0, 91,
	92, 93, 0, 0, 0, 0, 86, 53, 0, 0,
	0, 74, 75, 141, 0, 0, 0, 0, 0, 54,
	55, 76, 63, 64, 65, 66, 67, 68, 69, 0,
	0, 157, 159, 158, 181, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 105, 73, 183, 180,
	0, 0, 60, 0, 52, 0, 0, 0, 57, 56,
	58, 59, 71, 108, 155, 156, 167, 170, 171, 172,
	173, 174, 175, 177, 179, 0, 157, 159, 158, 181,
	0, 0, 0, 0, 763, 182, 161, 165, 164, 0,
	0, 0, 0, 0, 160, 0, 162, 166, 168, 169,
	176, 178, 163, 183, 180, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 155,
	156, 167, 170, 171, 172, 173, 174, 175, 177, 179,
	0, 157, 159, 158, 181, 0, 0, 721, 0, 0,
	182, 161, 165, 164, 0, 0, 0, 0, 0, 160,
	0, 162, 166, 168, 169, 176, 178, 163, 183, 180,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 155, 156, 167, 170, 171, 172,
	173, 174, 175, 177, 179, 0, 0, 0, 701, 157,
	159, 158, 181, 0, 0, 182, 161, 165, 164, 0,
	0, 0, 0, 0, 160, 0, 162, 166, 168, 169,
	176, 178, 163, 0, 0, 0, 183, 180, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 155, 156, 167, 170, 171, 172, 173, 174,
	175, 177, 179, 0, 0, 0, 699, 157, 159, 158,
	181, 0, 0, 182, 161, 165, 164, 0, 0, 0,
	0, 0, 160, 0, 162, 166, 168, 169, 176, 178,
	163, 0, 0, 0, 183, 180, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	155, 156, 167, 170, 171, 172, 173, 174, 175, 177,
	179, 0, 0, 0, 690, 157, 159, 158, 181, 0,
	0, 182, 161, 165, 164, 0, 0, 0, 0, 0,
	160, 0, 162, 166, 168, 169, 176, 178, 163, 0,
	0, 0, 183, 180, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 155, 156,
	167, 170, 171, 172, 173, 174, 175, 177, 179, 0,
	157, 159, 158, 181, 0, 0, 686, 0, 0, 182,
	161, 165, 164, 0, 0, 0, 0, 0, 160, 0,
	162, 166, 168, 169, 176, 178, 163, 183, 180, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 155, 156, 167, 170, 171, 172, 173,
	174, 175, 177, 179, 0, 157, 159, 158, 181, 0,
	0, 685, 0, 0, 182, 161, 165, 164, 0, 0,
	0, 0, 0, 160, 0, 162, 166, 168, 169, 176,
	178, 163, 183, 180, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 155, 156,
	167, 170, 171, 172, 173, 174, 175, 177, 179, 0,
	0, 0, 641, 157, 159, 158, 181, 0, 0, 182,
	161, 165, 164, 0, 0, 0, 0, 0, 160, 0,
	162, 166, 168, 169, 176, 178, 163, 0, 0, 0,
	183, 180, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 155, 156, 167, 170,
	171, 172, 173, 174, 175, 177, 179, 0, 157, 159,
	158, 181, 0, 0, 630, 0, 0, 182, 161, 165,
	164, 0, 0, 0, 0, 0, 160, 0, 162, 166,
	168, 169, 176, 178, 163, 183, 180, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 155, 156, 167, 170, 171, 172, 173, 174, 175,
	177, 179, 0, 157, 159, 158, 181, 0, 0, 629,
	0, 0, 182, 161, 165, 164, 0, 0, 0, 0,
	0, 160, 0, 162, 166, 168, 169, 176, 178, 163,
	183, 180, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 155, 156, 167, 170,
	171, 172, 173, 174, 175, 177, 179, 0, 0, 0,
	612, 157, 159, 158, 181, 0, 0, 182, 161, 165,
	164, 0, 0, 0, 0, 0, 160, 0, 162, 166,
	168, 169, 176, 178, 163, 0, 0, 0, 183, 180,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 155, 156, 167, 170, 171, 172,
	173, 174, 175, 177, 179, 0, 157, 159, 158, 181,
	0, 0, 603, 0, 0, 182, 161, 165, 164, 0,
	0, 0, 0, 0, 160, 0, 162, 166, 168, 169,
	176, 178, 163, 183, 180, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 155,
	156, 167, 170, 171, 172, 173, 174, 175, 177, 179,
	574, 0, 0, 593, 157, 159, 158, 181, 0, 0,
	182, 161, 165, 164, 0, 0, 0, 0, 0, 160,
	0, 162, 166, 168, 169, 176, 178, 163, 0, 0,
	0, 183, 180, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 155, 156, 167,
	170, 171, 172, 173, 174, 175, 177, 179, 0, 157,
	159, 158, 181, 0, 0, 591, 0, 0, 182, 161,
	165, 164, 0, 0, 0, 0, 0, 160, 0, 162,
	166, 168, 169, 176, 178, 163, 183, 180, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 155, 156, 167, 170, 171, 172, 173, 174,
	175, 177, 179, 0, 157, 159, 158, 181, 0, 0,
	0, 0, 0, 182, 161, 165, 164, 0, 0, 0,
	0, 0, 160, 0, 162, 166, 168, 169, 176, 178,
	163, 183, 180, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 155, 156, 167,
	170, 171, 172, 173, 174, 175, 177, 179, 0, 157,
	159, 158, 181, 570, 0, 0, 0, 0, 182, 161,
	165, 164, 0, 0, 0, 0, 0, 160, 0, 162,
	166, 168, 169, 176, 178, 163, 183, 180, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 155, 156, 167, 170, 171, 172, 173, 174,
	175, 177, 179, 0, 157, 159, 158, 181, 0, 0,
	565, 0, 0, 182, 161, 165, 164, 0, 0, 0,
	0, 0, 160, 0, 162, 166, 168, 169, 176, 178,
	163, 183, 180, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 155, 156, 167,
	170, 171, 172, 173, 174, 175, 177, 179, 0, 157,
	159, 158, 181, 0, 0, 561, 0, 0, 182, 161,
	165, 164, 0, 0, 0, 0, 0, 160, 0, 162,
	166, 168, 169, 176, 178, 163, 183, 180, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 417, 0,
	0, 0, 155, 156, 167, 170, 171, 172, 173, 174,
	175, 177, 179, 0, 0, 0, 0, 0, 0, 0,
	423, 0, 0, 182, 161, 165, 164, 157, 159, 158,
	181, 0, 160, 0, 162, 166, 168, 169, 176, 178,
	163, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 183, 180, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	155, 156, 167, 170, 171, 172, 173, 174, 175, 177,
	179, 0, 157, 159, 158, 181, 0, 0, 0, 0,
	0, 182, 161, 165, 164, 0, 0, 0, 0, 0,
	160, 0, 162, 166, 168, 169, 176, 178, 163, 183,
	180, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 155, 156, 167, 170, 171,
	172, 173, 174, 175, 177, 179, 0, 0, 0, 0,
	377, 157, 159, 158, 181, 0, 182, 161, 165, 164,
	0, 0, 0, 0, 0, 160, 0, 162, 166, 168,
	169, 176, 178, 163, 0, 0, 0, 0, 183, 180,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 155, 156, 167, 170, 171, 172,
	173, 174, 175, 177, 179, 0, 0, 0, 0, 154,
	157, 159, 158, 181, 0, 182, 161, 165, 164, 0,
	0, 0, 0, 0, 160, 0, 162, 166, 168, 169,
	176, 178, 163, 0, 0, 0, 0, 183, 180, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 155, 156, 167, 170, 171, 172, 173,
	174, 175, 177, 179, 0, 0, 159, 158, 181, 0,
	0, 0, 0, 0, 182, 161, 165, 164, 0, 0,
	0, 0, 0, 160, 0, 162, 166, 168, 169, 176,
	178, 163, 183, 180, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 155, 156,
	167, 170, 171, 172, 173, 174, 175, 177, 179, 0,
	0, 0, 158, 181, 0, 0, 0, 0, 0, 182,
	161, 165, 164, 0, 0, 0, 0, 0, 160, 0,
	162, 166, 168, 169, 176, 178, 163, 183, 180, 434,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 155, 156, 167, 170, 171, 172, 173,
	174, 175, 177, 179, 0, 0, 0, 0, 0, 0,
	0, 181, 0, 0, 182, 161, 165, 164, 0, 0,
	0, 0, 0, 160, 0, 162, 166, 168, 169, 176,
	178, 163, 0, 0, 0, 183, 180, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 155, 156, 167, 170, 171, 172, 173, 174, 175,
	177, 179, 0, 0, 0, 0, 181, 0, 0, 0,
	0, 0, 182, 161, 165, 164, 0, 0, 0, 0,
	0, 160, 0, 162, 166, 168, 169, 176, 178, 163,
	183, 180, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 155, 156, 167, 170,
	171, 172, 173, 174, 175, 177, 179, 0, 0, 0,
	0, 181, 0, 0, 0, 0, 0, 182, 161, 165,
	164, 0, 0, 0, 0, 0, 160, 0, 162, 166,
	168, 169, 176, 178, 163, 183, 180, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 155, 156, 167, 170, 171, 172, 173, 174, 175,
	177, 179, 0, 0, 0, 181, 0, 0, 0, 0,
	0, 0, 0, 161, 165, 164, 0, 0, 0, 0,
	0, 160, 0, 162, 166, 168, 169, 176, 178, 163,
	180, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 181, 156, 167, 170, 171,
	172, 173, 174, 175, 177, 179, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 161, 165, 164,
	180, 0, 0, 0, 0, 160, 0, 162, 166, 168,
	169, 176, 178, 163, 0, 181, 0, 167, 170, 171,
	172, 173, 174, 175, 177, 179, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 161, 165, 164,
	180, 0, 0, 0, 0, 160, 0, 162, 166, 168,
	169, 176, 178, 163, 0, 181, 0, 167, 170, 171,
	172, 173, 174, 175, 177, 179, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 161, 165, 164,
	180, 0, 0, 0, 0, 0, 0, 162, 166, 168,
	169, 176, 178, 163, 0, 181, 0, 167, 170, 171,
	172, 173, 174, 175, 177, 179, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 161, 165, 164,
	180, 0, 0, 0, 0, 0, 0, 0, 166, 168,
	169, 176, 178, 163, 0, 0, 0, 167, 170, 171,
	172, 173, 174, 175, 177, 179, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 165, 164,
	449, 450, 460, 461, 0, 0, 440, 0, 166, 168,
	169, 176, 178, 163, 0, 465, 466, 467, 468, 469,
	470, 471, 472, 473, 474, 475, 495, 496, 497, 498,
	499, 487, 488, 489, 516, 490, 491, 476, 477, 478,
	479, 480, 481, 482, 483, 484, 485, 486, 0, 507,
	505, 506, 502, 503, 0, 0, 494, 500, 501, 508,
	509, 511, 510, 512, 513, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 504, 515, 514, 0,
	0, 451, 452, 453, 454, 455, 456, 457, 458, 459,
	462, 463, 464, 492, 493, 443, 444, 445, 446, 447,
	448, 449, 450, 460, 461, 0, 0, 923, 0, 0,
	0, 0, 0, 0, 0, 0, 465, 466, 467, 468,
	469, 470, 471, 472, 473, 474, 475, 495, 496, 497,
	498, 499, 487, 488, 489, 516, 490, 491, 476, 477,
	478, 479, 480, 481, 482, 483, 484, 485, 486, 0,
	507, 505, 506, 502, 503, 0, 0, 494, 500, 501,
	508, 509, 511, 510, 512, 513, 119, 0, 107, 0,
	0, 0, 0, 0, 112, 0, 0, 504, 515, 514,
	0, 0, 451, 452, 453, 454, 455, 456, 457, 458,
	459, 462, 463, 464, 492, 493, 752, 753, 754, 751,
	750, 749, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 671, 111, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 136, 0, 0, 114,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 237, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 235, 0, 672, 0, 0, 670,
	0, 0, 0, 0, 0, 108,
}
var yyPact = [...]int{

	-1000, -1000, 1251, -1000, -1000, -1000, -1000, -1000, -1000, -1000,
	264, 483, 651, 766, -1000, -1000, -1000, 262, 4342, 261,
	259, 5524, 5524, 5524, 183, 776, 5524, -1000, 6743, 258,
	257, 256, -1000, 405, 5524, 820, 281, 32, 504, 819,
	805, 799, 456, 507, 408, -1000, -1000, 254, -1000, -1000,
	134, 253, 4726, 5524, 688, 688, 5524, 5524, 5524, 5524,
	5524, -1000, -1000, 5524, 5524, 5524, 5524, 5524, 5524, 5524,
	252, 5524, -1000, 811, 5524, 5524, 5524, -1000, -1000, -1000,
	-1000, 101, -1000, 517, 506, -1000, 184, 251, 249, 5524,
	5524, 244, 5524, 5524, -1000, -1000, -1000, -1000, -1000, -1000,
	-1000, -1000, -1000, -1000, 765, 757, 32, -1000, 215, 203,
	203, 239, -1000, 490, 737, 170, 737, 295, -1000, -1000,
	345, 602, -9, 619, 737, -1000, -1000, -1000, -1000, -15,
	-1000, -59, 3147, 5524, 700, 32, 477, 5524, 5524, 337,
	6802, 666, 333, 332, -18, -1000, -1000, -19, -1000, -1000,
	-60, -24, -1000, 6802, -1000, 5524, 5524, 5524, 5524, 5524,
	5524, 5524, 5524, 5524, 5524, 5524, 5524, 5524, 5524, 5524,
	5524, 5524, 5524, 5524, 5524, 5524, 5524, 5524, 5524, 5524,
	5524, 230, 5410, 5524, 688, 5524, 766, -1000, 6684, 331,
	-1000, 794, -1000, 785, -1000, 591, -1000, 595, 233, 4342,
	228, 319, 275, 5296, 5524, 5524, 5524, 5524, 5524, 5524,
	5524, 5524, 5524, 5524, 5524, 5524, 5524, -1000, -1000, 5524,
	5524, 5524, 104, 4726, 98, 22, -1000, -1000, 6629, 688,
	5524, 225, -1000, -1000, 101, 5524, -1000, -1000, 4726, -1000,
	412, 412, 461, 412, 6561, 412, 412, 412, 412, 412,
	412, 412, -1000, 5524, 412, 219, 616, 707, -1000, 198,
	5182, 688, 7025, 6970, 7025, 5524, 3461, 3461, 203, -1000,
	505, 227, 203, -1000, -1000, 5524, 5524, 6802, 6802, 5524,
	6802, 6802, 742, -1000, 734, 575, 616, 221, 5524, -1000,
	-1000, 4840, -1000, 4726, 782, 490, 318, 490, -1000, -1000,
	1093, -1000, 316, -28, 582, 737, -1000, 597, 511, 770,
	551, -1000, -1000, 766, 5524, -1000, -1000, -1000, -1000, -1000,
	-1000, -1000, -1000, 218, 6506, 216, -1000, 315, -8, 6802,
	6451, -1000, -1000, -1000, -1000, 183, -1000, 733, 5524, -1000,
	5524, 7134, 7174, 6857, 7025, 6912, 7214, 7294, 7254, 132,
	132, 132, 461, 412, 461, 461, 64, 64, 168, 168,
	168, 168, 351, 351, 351, 351, 168, -1000, 6396, 5524,
	7080, -11, -1000, -1000, 6341, -30, 2989, -1000, -1000, -1000,
	212, 591, 569, 587, 403, -1000, 587, 5524, -1000, 5524,
	-1000, -1000, 7025, 5524, 7025, 7025, 7025, 7025, 7025, 7025,
	7025, 7025, 7025, 7025, 7025, 7025, 7025, 6286, 96, 6228,
	203, -1000, 5524, -1000, 167, -65, 4726, 5068, -1000, 6802,
	4726, 6173, 84, -1000, 165, -1000, -1000, -1000, -1000, 190,
	751, 6115, 93, 363, 5524, 81, 203, -1000, -1000, 5524,
	-1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000,
	-1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000,
	-1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000,
	-1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000,
	-1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000,
	-1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000,
	-1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000,
	-1000, -1000, -1000, -1000, -1000, -1000, -1000, 203, -1000, -1000,
	-1000, -1000, 183, 5524, 5524, 104, 183, 591, -12, -1000,
	6802, 6060, 6005, -1000, -1000, -1000, 285, 5947, -1000, -13,
	-1000, 6802, 5524, 163, -1000, -1000, 935, -1000, -1000, -1000,
	502, 550, -1000, 737, 539, 689, -1000, 501, -1000, 6802,
	158, 4186, 5524, 5524, 5524, 208, -1000, -1000, 6802, -1000,
	5524, 7080, 155, 688, 7606, 4030, -1000, 192, 285, 569,
	-1000, 587, -1000, -1000, 400, -40, -1000, 5892, 5837, 2831,
	7294, 3874, -1000, -1000, -1000, 5779, -66, 5524, -1000, 6802,
	688, 182, 151, -1000, -1000, -1000, 50, -1000, -1000, 709,
	-1000, -1000, -1000, -1000, 5524, -1000, 7025, -1000, -1000, 5721,
	-1000, -1000, 47, 5663, -1000, -1000, 569, 144, 5524, -1000,
	-1000, 142, -16, -1000, 35, -1000, -1000, 552, -1000, -1000,
	-1000, -1000, 141, 4954, 6802, -1000, -1000, 737, 499, -34,
	-1000, -1000, 737, 689, -1000, 314, -1000, -1000, -1000, 5608,
	313, 6802, -1000, 312, 311, 7080, 307, -1000, 139, 562,
	688, 177, 4726, -1000, -1000, -1000, 620, 285, 138, -1000,
	396, -40, 1433, -1000, 587, 4342, 272, 303, -1000, -1000,
	-1000, 5524, 7025, -1000, 4726, -66, -1000, -1000, 5553, -1000,
	-1000, -1000, -1000, -1000, -1000, 269, 285, 475, -1000, -1000,
	-1000, -1000, -35, -1000, 737, 362, 689, -1000, -34, -1000,
	2673, 302, 5524, 411, -1000, 793, -1000, -1000, 3562, 7606,
	-1000, 4726, 40, 2515, -1000, 169, 395, 121, 611, 379,
	-1000, -1000, -1000, 285, 647, 587, 612, -1000, -1000, -1000,
	-1000, -1000, -1000, -1000, -1000, 1275, -1000, -1000, -1000, -1000,
	3305, 7025, 119, 350, 378, -1000, 285, -1000, 729, -1000,
	349, 737, -35, -1000, -1000, 347, 301, -1000, 113, -1000,
	5524, 174, 401, 300, 775, -1000, -1000, -1000, 111, -1000,
	108, -1000, 299, 587, -1000, 269, 269, 133, -1000, 1117,
	713, 7446, 32, -31, -1000, -1000, 3305, -66, -1000, -1000,
	560, -1000, -67, -1000, -1000, 286, -1000, -1000, 3718, 335,
	-1000, -1000, -1000, -1000, -1000, 292, 2357, 3562, -1000, -1000,
	89, -1000, 2199, 375, 374, 214, 959, -1000, -36, -1000,
	-70, -37, -1000, -73, 7446, -1000, -1000, 4625, 538, 5524,
	5524, -1000, -1000, -1000, -1000, -1000, 3305, -1000, 290, -1000,
	106, 587, -1000, -1000, -1000, -43, -1000, -1000, 725, -1000,
	-1000, 713, -1000, 5524, -1000, 7446, 5524, -1000, -1000, 4498,
	-1000, 284, 217, 608, 680, 497, -1000, 477, -1000, -1000,
	7025, 6802, 2041, 3305, -1000, 366, -1000, 1883, 1725, -1000,
	214, -1000, -1000, 6802, -1000, 6802, 105, -1000, -1000, -1000,
	-1000, 587, 7547, 7446, 185, -1000, -1000, -1000, -1000, -1000,
	-1000, 285, -40, -1000, -1000, 7446, -1000, -1000, 1567, 103,
	-1000, -1000, 269, 283, -1000, -1000, -1000, 1409, -1000,
}
var yyPgo = [...]int{

	0, 931, 929, 14, 8, 928, 4, 29, 13, 927,
	11, 44, 78, 71, 52, 48, 926, 26, 924, 83,
	21, 82, 916, 0, 84, 915, 914, 42, 190, 32,
	19, 38, 913, 79, 73, 912, 5, 911, 910, 909,
	908, 15, 62, 905, 904, 100, 87, 274, 903, 902,
	901, 6, 900, 86, 41, 899, 56, 49, 898, 897,
	896, 895, 894, 99, 893, 892, 890, 883, 10, 882,
	881, 46, 39, 40, 2, 16, 705, 43, 85, 880,
	879, 878, 12, 876, 875, 555, 50, 37, 871, 870,
	9, 762, 20, 553, 868, 18, 867, 864, 862, 88,
	860, 36, 858, 855, 25, 33, 854, 853, 45, 848,
	846, 554, 845, 844, 839, 34, 837, 80, 1, 3,
	832, 17, 831, 821, 803, 7, 767, 28, 752,
}
var yyR1 = [...]int{

	0, 128, 4, 4, 4, 4, 4, 4, 4, 4,
	4, 4, 4, 4, 4, 4, 4, 4, 4, 4,
	4, 4, 4, 4, 4, 4, 4, 4, 4, 4,
	4, 4, 4, 4, 4, 4, 4, 4, 4, 4,
	4, 4, 4, 4, 4, 4, 4, 4, 4, 4,
	4, 4, 4, 4, 4, 4, 4, 4, 4, 4,
	4, 4, 4, 4, 4, 4, 4, 4, 4, 4,
	5, 5, 5, 5, 5, 5, 5, 6, 6, 117,
	117, 95, 95, 10, 10, 10, 9, 9, 9, 9,
	9, 9, 9, 9, 9, 9, 9, 9, 9, 9,
	9, 91, 91, 16, 16, 18, 18, 7, 7, 105,
	105, 104, 104, 111, 111, 17, 17, 20, 20, 19,
	19, 99, 99, 118, 118, 22, 22, 22, 22, 22,
	22, 22, 11, 11, 11, 11, 11, 11, 11, 11,
	11, 11, 11, 11, 11, 11, 11, 11, 11, 11,
	11, 11, 11, 11, 11, 11, 97, 97, 96, 96,
	26, 26, 110, 110, 27, 12, 1, 1, 2, 2,
	13, 13, 126, 126, 76, 76, 14, 15, 86, 86,
	88, 88, 87, 87, 92, 92, 92, 92, 83, 83,
	82, 82, 25, 25, 80, 80, 80, 80, 108, 108,
	108, 8, 8, 84, 84, 67, 67, 65, 65, 69,
	69, 66, 66, 119, 119, 120, 120, 29, 29, 30,
	30, 75, 75, 73, 73, 73, 74, 74, 77, 77,
	116, 116, 31, 31, 103, 103, 33, 107, 107, 34,
	34, 121, 121, 35, 35, 35, 35, 125, 125, 79,
	79, 79, 109, 109, 36, 36, 37, 38, 38, 38,
	38, 40, 40, 39, 81, 81, 123, 123, 122, 122,
	124, 124, 90, 90, 90, 90, 90, 90, 106, 106,
	41, 41, 98, 98, 68, 21, 100, 100, 42, 101,
	101, 102, 102, 44, 43, 43, 32, 32, 32, 32,
	32, 32, 32, 32, 32, 32, 32, 32, 32, 32,
	32, 32, 32, 32, 32, 32, 32, 32, 32, 32,
	32, 32, 32, 32, 32, 32, 32, 32, 32, 32,
	32, 32, 32, 32, 32, 32, 32, 32, 32, 32,
	32, 32, 32, 32, 32, 32, 32, 32, 32, 32,
	32, 32, 32, 32, 32, 32, 32, 32, 32, 32,
	32, 32, 32, 32, 32, 32, 32, 32, 32, 32,
	32, 32, 32, 85, 85, 127, 3, 3, 89, 89,
	112, 112, 51, 51, 52, 52, 52, 52, 45, 45,
	46, 46, 49, 49, 94, 94, 94, 78, 78, 56,
	56, 56, 50, 50, 50, 50, 50, 50, 50, 50,
	50, 50, 50, 50, 50, 50, 50, 50, 57, 57,
	57, 23, 23, 24, 24, 55, 58, 58, 58, 59,
	59, 59, 60, 60, 60, 60, 60, 60, 28, 28,
	28, 47, 47, 47, 61, 61, 62, 62, 62, 62,
	62, 62, 53, 53, 53, 54, 54, 54, 115, 71,
	71, 114, 114, 70, 70, 70, 70, 70, 70, 70,
	93, 93, 93, 93, 63, 63, 63, 63, 63, 63,
	63, 64, 64, 64, 64, 48, 48, 48, 48, 48,
	48, 48, 113, 113, 72,
}
var yyR2 = [...]int{

	0, 1, 1, 1, 1, 1, 1, 1, 1, 1,
	1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
	1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
	1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
	1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
	1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
	1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
	1, 1, 1, 1, 1, 1, 1, 1, 1, 2,
	0, 1, 3, 1, 3, 2, 1, 1, 1, 1,
	1, 1, 4, 3, 5, 4, 3, 4, 3, 4,
	3, 1, 1, 6, 7, 6, 7, 0, 1, 3,
	1, 3, 1, 3, 1, 1, 2, 1, 3, 1,
	2, 3, 1, 2, 0, 1, 1, 1, 1, 1,
	1, 4, 3, 1, 1, 5, 7, 9, 5, 3,
	3, 3, 3, 3, 3, 1, 2, 6, 7, 9,
	5, 1, 6, 3, 3, 2, 0, 9, 1, 3,
	0, 4, 1, 3, 1, 11, 0, 1, 0, 1,
	9, 8, 1, 2, 1, 1, 6, 7, 0, 2,
	0, 2, 0, 2, 1, 2, 4, 3, 1, 4,
	1, 4, 1, 4, 3, 4, 4, 5, 0, 5,
	4, 1, 1, 1, 4, 5, 6, 1, 3, 6,
	7, 3, 6, 1, 0, 1, 3, 4, 6, 0,
	1, 1, 2, 1, 1, 1, 0, 2, 2, 4,
	1, 3, 1, 2, 3, 1, 1, 3, 1, 1,
	3, 2, 0, 4, 4, 3, 10, 1, 3, 1,
	2, 3, 1, 2, 2, 2, 3, 3, 3, 4,
	3, 1, 1, 3, 1, 3, 1, 1, 0, 1,
	1, 2, 1, 1, 1, 1, 1, 1, 3, 1,
	2, 4, 3, 1, 4, 4, 3, 1, 1, 0,
	1, 3, 1, 8, 3, 2, 6, 5, 3, 4,
	2, 3, 3, 3, 3, 3, 3, 3, 3, 3,
	3, 3, 3, 3, 2, 2, 2, 2, 3, 3,
	3, 3, 3, 3, 3, 3, 3, 3, 3, 3,
	3, 3, 3, 3, 3, 2, 2, 2, 2, 3,
	3, 3, 3, 3, 3, 3, 3, 3, 3, 3,
	1, 5, 4, 3, 1, 2, 2, 2, 2, 2,
	2, 2, 2, 2, 1, 3, 2, 1, 2, 4,
	2, 1, 2, 11, 9, 0, 0, 1, 0, 4,
	3, 1, 1, 2, 2, 4, 4, 2, 1, 1,
	1, 1, 0, 3, 0, 1, 1, 0, 1, 4,
	3, 1, 1, 1, 1, 1, 1, 1, 1, 1,
	1, 1, 3, 2, 3, 3, 1, 1, 1, 3,
	3, 1, 1, 0, 1, 1, 1, 3, 1, 1,
	3, 1, 1, 4, 4, 4, 4, 1, 1, 1,
	3, 1, 4, 2, 3, 3, 1, 4, 4, 3,
	3, 3, 1, 3, 1, 1, 3, 1, 1, 0,
	1, 3, 1, 3, 1, 4, 2, 2, 6, 4,
	2, 2, 1, 2, 1, 4, 3, 3, 3, 6,
	3, 1, 1, 2, 1, 5, 4, 2, 2, 4,
	2, 2, 1, 3, 1,
}
var yyChk = [...]int{

	-1000, -128, -117, -9, 2, -11, -12, -13, -14, -15,
	52, 80, 45, 39, 144, -65, -66, 21, 20, 23,
	30, 34, 35, 40, 47, 99, 19, 14, -23, 49,
	25, 27, 146, 41, 44, 36, 10, 37, -126, 53,
	54, 55, -67, -69, -28, -32, -76, 7, -60, -61,
	-58, 60, 150, 93, 105, 106, 155, 154, 156, 157,
	148, -43, -48, 108, 109, 110, 111, 112, 113, 114,
	6, 158, -50, 143, 97, 98, 107, -85, 100, 101,
	-47, -57, -52, -45, -55, -56, 92, 50, 51, 4,
	5, 85, 86, 87, 8, 9, 67, 68, 82, 64,
	65, 66, 81, 63, 75, 142, 38, 12, 159, -10,
	-59, 61, 18, -95, 83, 148, 83, -95, 144, 10,
	-18, -91, -111, -95, 83, 37, 39, -19, -20, -99,
	-21, 10, -118, 148, -11, 37, 80, 148, 148, -24,
	-23, 99, -24, -24, -103, -33, -47, -107, -85, -34,
	12, -100, -42, -23, 146, 131, 132, 88, 90, 89,
	161, 153, 163, 169, 155, 154, 164, 133, 165, 166,
	134, 135, 136, 137, 138, 139, 167, 140, 168, 141,
	116, 91, 152, 115, 148, 148, 148, 144, -23, 10,
	147, -3, 153, 53, -76, 10, 10, 10, 94, 95,
	94, 96, 95, 162, 118, 119, 120, 121, 122, 123,
	124, 125, 126, 127, 128, 129, 130, 105, 106, 148,
	150, 144, 58, 148, -115, -114, -71, -70, -23, 153,
	84, 60, -23, -28, -57, 148, -56, 99, 150, -28,
	-23, -23, -23, -23, -23, -23, -23, -23, -23, -23,
	-23, -23, -49, 148, -23, -94, 17, -93, -63, 12,
	77, 78, -23, -23, -23, 150, 79, 79, -46, -44,
	-45, -62, 53, -10, -47, 148, 148, -23, -23, 148,
	-23, -23, 17, 76, -93, -93, 17, -3, 144, -47,
	-77, 148, -77, 148, 83, -95, 149, -95, 146, 144,
	-117, 146, -16, -111, -95, 83, 146, 160, 83, 29,
	-95, -20, 146, 160, 162, -22, 145, 2, -11, -12,
	-13, -14, -15, 52, -23, 21, -3, -101, -102, -23,
	-23, 146, 146, 146, 146, 160, 146, 160, 162, 146,
	160, -23, -23, -23, -23, -23, -23, -23, -23, -23,
	-23, -23, -23, -23, -23, -23, -23, -23, -23, -23,
	-23, -23, -23, -23, -23, -23, -23, -46, -23, 147,
	-23, -110, -27, -28, -23, -99, -118, 146, 146, 10,
	-127, 10, -86, 56, -127, -88, 56, 148, -11, 148,
	146, 147, -23, 153, -23, -23, -23, -23, -23, -23,
	-23, -23, -23, -23, -23, -23, -23, -23, -24, -23,
	-54, 10, 144, -47, -115, 151, 160, 59, -28, -23,
	148, -23, -115, 149, -24, 143, -63, -63, 17, 150,
	58, -23, 11, -28, 59, -24, -53, -6, -47, 144,
	10, -5, -4, 99, 100, 101, 102, 103, 104, 4,
	5, 85, 86, 87, 88, 89, 90, 91, 92, 93,
	6, 7, 94, 95, 96, 19, 20, 21, 22, 23,
	24, 25, 26, 27, 28, 29, 41, 42, 43, 44,
	45, 46, 47, 48, 49, 50, 51, 35, 36, 37,
	39, 40, 97, 98, 60, 30, 31, 32, 33, 34,
	61, 62, 56, 57, 80, 54, 55, 53, 63, 64,
	66, 65, 67, 68, 82, 81, 38, -53, -6, -47,
	-78, -77, 79, 150, 144, 58, 79, -78, -113, -72,
	-23, -23, -23, 76, 76, 142, 148, -23, 149, -116,
	-31, -23, 84, -115, 10, 146, -117, 145, 146, 146,
	83, -95, -19, 83, -95, 144, 10, 83, -21, -23,
	148, 149, 148, 146, 160, 149, -33, -34, -23, -42,
	147, -23, -7, 160, 29, 149, 145, -127, 148, -86,
	-87, 57, -10, 144, -127, -125, -10, -23, -23, -118,
	-23, 149, 151, 145, -77, -23, 149, 162, -71, -23,
	153, 60, -115, 149, 151, 149, -64, 10, 13, 154,
	12, 10, 145, 145, 150, 145, -23, 151, -77, -23,
	-77, -47, -24, -23, -54, -47, -86, -7, 160, 149,
	149, -119, -120, -29, -30, -75, -73, 152, 61, 62,
	-10, 145, -7, 160, -23, 149, 145, 144, 83, -105,
	-17, -20, -91, 144, -127, 149, -84, -11, 147, -23,
	-101, -23, -80, 144, 147, -23, 149, -27, -92, -28,
	153, 60, 150, -25, -11, 147, -97, 148, -119, -87,
	-127, -125, -121, 144, 160, 149, 149, 96, -11, 147,
	145, 162, -23, -28, 148, 149, 151, 13, -23, 145,
	151, 145, -87, 149, -72, 149, 160, -1, 153, -73,
	149, -31, -104, -20, 144, -7, 160, -20, -105, 146,
	-118, 149, 146, -108, 146, -108, 146, 146, 149, 59,
	-28, 148, -115, -118, -26, 42, 43, -119, 149, -127,
	144, 145, -35, -123, -122, 45, -124, 48, -90, 104,
	103, 102, 99, 100, 101, -121, -10, -11, 147, 146,
	-118, -23, -115, 151, -127, -74, 147, -29, -2, 84,
	-7, 160, -104, 145, -17, -7, 22, 146, -101, 145,
	32, 33, -108, 31, -108, -82, -11, 147, -92, -28,
	-115, 151, 28, 148, 144, 149, -89, 45, 144, -121,
	-30, 39, 37, -125, -90, 145, -118, 149, 145, 144,
	-127, -75, 12, 145, -20, -7, 145, 146, 149, -23,
	-8, 147, 146, 145, 146, 31, -118, 149, 149, 146,
	-96, -10, -118, -74, -74, 148, -121, 145, -106, -41,
	12, -98, -68, -6, -3, -79, 146, 144, -121, 59,
	162, 145, -83, -11, 147, -8, -118, 146, 26, -82,
	12, 161, 145, 144, 144, -112, -51, 12, 153, 145,
	146, 160, -127, 162, 146, 160, 162, -6, 145, -109,
	-36, -37, -38, -39, -40, -10, -6, 80, 10, 145,
	-23, -23, -118, -118, 146, 149, -10, -118, -118, 149,
	160, 12, -41, -23, -68, -23, -127, 145, -36, 146,
	146, 46, 29, 79, 24, 144, 145, 145, -51, -127,
	-127, 148, -125, 10, -4, -90, -6, 146, -118, -119,
	-6, 145, 149, -74, -81, 146, 144, -118, 145,
}
var yyDef = [...]int{

	80, -2, -2, 79, 86, 87, 88, 89, 90, 91,
	0, 0, 0, 0, 124, 133, 134, 0, 0, 0,
	0, 423, 423, 423, 0, 388, 0, 145, 0, 0,
	0, 0, 151, 0, 0, 0, 81, 376, 0, 0,
	0, 0, 207, 0, -2, 422, 172, 0, -2, 439,
	425, 0, 459, 0, 0, 0, 0, 0, 0, 0,
	0, 350, 354, 0, 0, 0, 0, 0, 0, 0,
	392, 0, 364, 394, 0, 367, 0, 371, 174, 175,
	432, 417, 437, 0, 0, -2, 0, 0, 0, 0,
	0, 0, 0, 0, 402, 403, 404, 405, 406, 407,
	408, 409, 410, 411, 0, 0, 376, 441, 0, -2,
	0, 0, 401, 83, 0, 0, 0, 0, 80, 81,
	0, 0, 0, 117, 0, 101, 102, 114, 119, 0,
	122, 0, 0, 0, 0, 376, 0, 289, 0, 0,
	424, 388, 0, 0, 0, 235, 236, 0, 372, 238,
	239, 0, 287, 288, 146, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 124, 0, 0,
	155, 375, 377, 0, 173, 178, 375, 180, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 314, 316, 0,
	423, 0, 0, 459, 0, 458, 462, 460, 464, 0,
	0, 0, 300, -2, 0, 0, -2, 388, 459, -2,
	335, 336, 337, 338, 0, 355, 356, 357, 358, 359,
	360, 361, 362, 423, 363, 0, 395, 396, 472, 474,
	0, 0, 366, 368, 370, 423, 0, 0, 397, 295,
	390, 391, 397, 389, 446, 0, 0, 487, 488, 0,
	490, 491, 0, 413, 0, 0, 0, 0, 0, 443,
	384, 0, 387, 459, 0, 85, 0, 84, 93, 80,
	0, 96, 0, 0, 117, 0, 98, 0, 0, 0,
	117, 120, 100, 0, 0, 123, 132, 125, 126, 127,
	128, 129, 130, 0, 0, 0, 375, 0, 290, 292,
	0, 139, 140, 141, 142, 0, 143, 0, 0, 144,
	0, 318, 319, 320, 321, 322, 323, 324, 325, 326,
	327, 328, 329, 330, 331, 332, 333, 334, -2, -2,
	-2, -2, -2, -2, -2, -2, -2, 348, 0, 0,
	353, 107, 162, -2, 0, 0, 0, 153, 154, 375,
	0, 178, 182, 0, 0, 375, 0, 0, 208, 0,
	211, 124, 298, 0, 301, 302, 303, 304, 305, 306,
	307, 308, 309, 310, 311, 312, 313, 0, 0, 0,
	440, 455, 0, 457, 0, 400, 459, 0, -2, 467,
	459, 0, 0, -2, 0, 365, 473, 470, 471, 0,
	0, 0, 0, 426, 0, 0, 0, -2, -2, 0,
	77, 78, 70, 71, 72, 73, 74, 75, 76, 2,
	3, 4, 5, 6, 7, 8, 9, 10, 11, 12,
	13, 14, 15, 16, 17, 18, 19, 20, 21, 22,
	23, 24, 25, 26, 27, 28, 29, 30, 31, 32,
	33, 34, 35, 36, 37, 38, 39, 40, 41, 42,
	43, 44, 45, 46, 47, 48, 49, 50, 51, 52,
	53, 54, 55, 56, 57, 58, 59, 60, 61, 62,
	63, 64, 65, 66, 67, 68, 69, 0, -2, -2,
	294, 398, 0, 423, 0, 0, 0, 178, 107, 492,
	494, 0, 0, 412, 415, 414, -2, 0, 228, 107,
	230, 232, 0, 0, 82, 92, 0, 95, 97, 99,
	0, 117, 113, 0, 117, 0, 118, 0, 121, 375,
	0, 0, 0, 289, 0, 0, 234, 237, 240, 286,
	0, 352, 0, 108, 0, 0, 156, 0, -2, 182,
	375, 0, 179, 242, 0, 181, 247, 0, 0, 0,
	299, 0, 433, 435, 436, 0, 0, 0, 461, 463,
	0, 0, 0, -2, 400, 393, 0, 481, 482, 0,
	484, 476, 477, 478, 0, 480, 369, 434, 385, 0,
	386, 450, 0, 0, 449, 451, 182, 0, 108, 486,
	489, 0, 213, 215, 166, 220, 221, 0, 223, 224,
	225, 442, 0, 108, 233, 399, 94, 0, 0, 107,
	110, 115, 0, 0, 285, 0, 135, 203, 124, 0,
	0, 291, 138, 198, 198, 351, 0, 163, 0, -2,
	0, 0, 459, 150, 192, 124, 160, -2, 0, 375,
	0, 183, 268, 242, 0, 0, 0, 0, 205, 124,
	456, 0, 297, -2, 459, 469, 475, 483, 0, 453,
	447, 448, 375, 485, 493, 226, 219, 168, 167, 222,
	229, 231, 107, 112, 0, 0, 108, 116, 107, 131,
	0, 0, 289, 0, 198, 0, 198, 147, 0, 0,
	-2, 459, 0, 0, 152, 0, 0, 0, 378, 0,
	242, 176, 241, 219, 0, 0, -2, 267, 270, 272,
	273, 274, 275, 276, 277, 268, 248, 206, 124, 212,
	-2, 296, 0, 0, 0, 375, 0, 216, 0, 169,
	0, 108, 107, 105, 109, 0, 0, 136, 0, 194,
	0, 0, 0, 0, 0, 148, 190, 124, 0, -2,
	0, -2, 0, 0, 124, 226, 226, 0, 242, 268,
	0, 0, 376, 0, 271, 177, -2, 468, 479, 242,
	0, 227, 217, 103, 111, 0, 106, 204, 0, 0,
	124, 201, 202, 195, 196, 0, 0, 0, 186, 193,
	0, 158, 0, 0, 0, 0, 268, 171, 0, 279,
	375, 0, 283, 0, 0, 245, 249, 0, 268, 0,
	0, 104, 137, 188, 124, 124, -2, 197, 0, 149,
	0, 0, 161, 124, 124, 0, 381, 382, 0, 170,
	243, 0, 280, 0, 244, 0, 0, 375, 250, 0,
	252, 0, 0, 262, 0, 0, 261, 57, -2, 293,
	374, 218, 0, -2, 191, 0, 159, 0, 0, 379,
	0, 383, 278, 375, 282, 375, 0, 251, 253, 254,
	255, 0, 0, 0, 0, 124, 165, 373, 380, 281,
	284, -2, 256, 257, 258, 260, 263, 189, 0, 0,
	259, 157, 226, 0, 246, 264, 124, 0, 265,
}
var yyTok1 = [...]int{

	1, 3, 3, 3, 3, 3, 3, 3, 3, 3,
	3, 3, 3, 3, 3, 3, 3, 3, 3, 3,
	3, 3, 3, 3, 3, 3, 3, 3, 3, 3,
	3, 3, 3, 156, 142, 3, 159, 166, 153, 3,
	148, 149, 164, 155, 160, 154, 169, 165, 3, 3,
	3, 3, 3, 3, 3, 3, 3, 3, 147, 146,
	167, 162, 168, 152, 158, 3, 3, 3, 3, 3,
	3, 3, 3, 3, 3, 3, 3, 3, 3, 3,
	3, 3, 3, 3, 3, 3, 3, 3, 3, 3,
	3, 150, 3, 151, 163, 3, 143, 3, 3, 3,
	3, 3, 3, 3, 3, 3, 3, 3, 3, 3,
	3, 3, 3, 3, 3, 3, 3, 3, 3, 3,
	3, 3, 3, 144, 161, 145, 157,
}
var yyTok2 = [...]int{

	2, 3, 4, 5, 6, 7, 8, 9, 10, 11,
	12, 13, 14, 15, 16, 17, 18, 19, 20, 21,
	22, 23, 24, 25, 26, 27, 28, 29, 30, 31,
	32, 33, 34, 35, 36, 37, 38, 39, 40, 41,
	42, 43, 44, 45, 46, 47, 48, 49, 50, 51,
	52, 53, 54, 55, 56, 57, 58, 59, 60, 61,
	62, 63, 64, 65, 66, 67, 68, 69, 70, 71,
	72, 73, 74, 75, 76, 77, 78, 79, 80, 81,
	82, 83, 84, 85, 86, 87, 88, 89, 90, 91,
	92, 93, 94, 95, 96, 97, 98, 99, 100, 101,
	102, 103, 104, 105, 106, 107, 108, 109, 110, 111,
	112, 113, 114, 115, 116, 117, 118, 119, 120, 121,
	122, 123, 124, 125, 126, 127, 128, 129, 130, 131,
	132, 133, 134, 135, 136, 137, 138, 139, 140, 141,
}
var yyTok3 = [...]int{
	0,
}

var yyErrorMessages = [...]struct {
	state int
	token int
	msg   string
}{}

// line yaccpar:1

/*	parser for yacc output	*/

var (
	yyDebug        = 0
	yyErrorVerbose = true
)

type yyLexer interface {
	Lex(lval *yySymType) int
	Error(s string)
}

type yyParser interface {
	Parse(yyLexer) int
	Lookahead() int
}

type yyParserImpl struct {
	lval  yySymType
	stack [yyInitialStackSize]yySymType
	char  int
}

func (p *yyParserImpl) Lookahead() int {
	return p.char
}

func yyNewParser() yyParser {
	return &yyParserImpl{}
}

const yyFlag = -1000

func yyTokname(c int) string {
	if c >= 1 && c-1 < len(yyToknames) {
		if yyToknames[c-1] != "" {
			return yyToknames[c-1]
		}
	}
	return __yyfmt__.Sprintf("tok-%v", c)
}

func yyStatname(s int) string {
	if s >= 0 && s < len(yyStatenames) {
		if yyStatenames[s] != "" {
			return yyStatenames[s]
		}
	}
	return __yyfmt__.Sprintf("state-%v", s)
}

func yyErrorMessage(state, lookAhead int) string {
	const TOKSTART = 4

	if !yyErrorVerbose {
		return "syntax error"
	}

	for _, e := range yyErrorMessages {
		if e.state == state && e.token == lookAhead {
			return "syntax error: " + e.msg
		}
	}

	res := "syntax error: unexpected " + yyTokname(lookAhead)

	// To match Bison, suggest at most four expected tokens.
	expected := make([]int, 0, 4)

	// Look for shiftable tokens.
	base := yyPact[state]
	for tok := TOKSTART; tok-1 < len(yyToknames); tok++ {
		if n := base + tok; n >= 0 && n < yyLast && yyChk[yyAct[n]] == tok {
			if len(expected) == cap(expected) {
				return res
			}
			expected = append(expected, tok)
		}
	}

	if yyDef[state] == -2 {
		i := 0
		for yyExca[i] != -1 || yyExca[i+1] != state {
			i += 2
		}

		// Look for tokens that we accept or reduce.
		for i += 2; yyExca[i] >= 0; i += 2 {
			tok := yyExca[i]
			if tok < TOKSTART || yyExca[i+1] == 0 {
				continue
			}
			if len(expected) == cap(expected) {
				return res
			}
			expected = append(expected, tok)
		}

		// If the default action is to accept or reduce, give up.
		if yyExca[i+1] != 0 {
			return res
		}
	}

	for i, tok := range expected {
		if i == 0 {
			res += ", expecting "
		} else {
			res += " or "
		}
		res += yyTokname(tok)
	}
	return res
}

func yylex1(lex yyLexer, lval *yySymType) (char, token int) {
	token = 0
	char = lex.Lex(lval)
	if char <= 0 {
		token = yyTok1[0]
		goto out
	}
	if char < len(yyTok1) {
		token = yyTok1[char]
		goto out
	}
	if char >= yyPrivate {
		if char < yyPrivate+len(yyTok2) {
			token = yyTok2[char-yyPrivate]
			goto out
		}
	}
	for i := 0; i < len(yyTok3); i += 2 {
		token = yyTok3[i+0]
		if token == char {
			token = yyTok3[i+1]
			goto out
		}
	}

out:
	if token == 0 {
		token = yyTok2[1] /* unknown char */
	}
	if yyDebug >= 3 {
		__yyfmt__.Printf("lex %s(%d)\n", yyTokname(token), uint(char))
	}
	return char, token
}

func yyParse(yylex yyLexer) int {
	return yyNewParser().Parse(yylex)
}

func (yyrcvr *yyParserImpl) Parse(yylex yyLexer) int {
	var yyn int
	var yyVAL yySymType
	var yyDollar []yySymType
	_ = yyDollar // silence set and not used
	yyS := yyrcvr.stack[:]

	Nerrs := 0   /* number of errors */
	Errflag := 0 /* error recovery flag */
	yystate := 0
	yyrcvr.char = -1
	yytoken := -1 // yyrcvr.char translated into internal numbering
	defer func() {
		// Make sure we report no lookahead when not parsing.
		yystate = -1
		yyrcvr.char = -1
		yytoken = -1
	}()
	yyp := -1
	goto yystack

ret0:
	return 0

ret1:
	return 1

yystack:
	/* put a state and value onto the stack */
	if yyDebug >= 4 {
		__yyfmt__.Printf("char %v in %v\n", yyTokname(yytoken), yyStatname(yystate))
	}

	yyp++
	if yyp >= len(yyS) {
		nyys := make([]yySymType, len(yyS)*2)
		copy(nyys, yyS)
		yyS = nyys
	}
	yyS[yyp] = yyVAL
	yyS[yyp].yys = yystate

yynewstate:
	yyn = yyPact[yystate]
	if yyn <= yyFlag {
		goto yydefault /* simple state */
	}
	if yyrcvr.char < 0 {
		yyrcvr.char, yytoken = yylex1(yylex, &yyrcvr.lval)
	}
	yyn += yytoken
	if yyn < 0 || yyn >= yyLast {
		goto yydefault
	}
	yyn = yyAct[yyn]
	if yyChk[yyn] == yytoken { /* valid shift */
		yyrcvr.char = -1
		yytoken = -1
		yyVAL = yyrcvr.lval
		yystate = yyn
		if Errflag > 0 {
			Errflag--
		}
		goto yystack
	}

yydefault:
	/* default state action */
	yyn = yyDef[yystate]
	if yyn == -2 {
		if yyrcvr.char < 0 {
			yyrcvr.char, yytoken = yylex1(yylex, &yyrcvr.lval)
		}

		/* look through exception table */
		xi := 0
		for {
			if yyExca[xi+0] == -1 && yyExca[xi+1] == yystate {
				break
			}
			xi += 2
		}
		for xi += 2; ; xi += 2 {
			yyn = yyExca[xi+0]
			if yyn < 0 || yyn == yytoken {
				break
			}
		}
		yyn = yyExca[xi+1]
		if yyn < 0 {
			goto ret0
		}
	}
	if yyn == 0 {
		/* error ... attempt to resume parsing */
		switch Errflag {
		case 0: /* brand new error */
			yylex.Error(yyErrorMessage(yystate, yytoken))
			Nerrs++
			if yyDebug >= 1 {
				__yyfmt__.Printf("%s", yyStatname(yystate))
				__yyfmt__.Printf(" saw %s\n", yyTokname(yytoken))
			}
			fallthrough

		case 1, 2: /* incompletely recovered error ... try again */
			Errflag = 3

			/* find a state where "error" is a legal shift action */
			for yyp >= 0 {
				yyn = yyPact[yyS[yyp].yys] + yyErrCode
				if yyn >= 0 && yyn < yyLast {
					yystate = yyAct[yyn] /* simulate a shift of "error" */
					if yyChk[yystate] == yyErrCode {
						goto yystack
					}
				}

				/* the current p has no shift on "error", pop stack */
				if yyDebug >= 2 {
					__yyfmt__.Printf("error recovery pops state %d\n", yyS[yyp].yys)
				}
				yyp--
			}
			/* there is no state on the stack with an error shift ... abort */
			goto ret1

		case 3: /* no shift yet; clobber input char */
			if yyDebug >= 2 {
				__yyfmt__.Printf("error recovery discards %s\n", yyTokname(yytoken))
			}
			if yytoken == yyEofCode {
				goto ret1
			}
			yyrcvr.char = -1
			yytoken = -1
			goto yynewstate /* try again in the same state */
		}
	}

	/* reduction by production yyn */
	if yyDebug >= 2 {
		__yyfmt__.Printf("reduce %v in:\n\t%v\n", yyn, yyStatname(yystate))
	}

	yynt := yyn
	yypt := yyp
	_ = yypt // guard against "declared and not used"

	yyp -= yyR2[yyn]
	// yyp is now the index of $0. Perform the default action. Iff the
	// reduced production is ε, $1 is possibly out of range.
	if yyp+1 >= len(yyS) {
		nyys := make([]yySymType, len(yyS)*2)
		copy(nyys, yyS)
		yyS = nyys
	}
	yyVAL = yyS[yyp+1]

	/* consult goto table to find next state */
	yyn = yyR1[yyn]
	yyg := yyPgo[yyn]
	yyj := yyg + yyS[yyp].yys + 1

	if yyj >= yyLast {
		yystate = yyAct[yyg]
	} else {
		yystate = yyAct[yyj]
		if yyChk[yystate] != -yyn {
			yystate = yyAct[yyg]
		}
	}
	// dummy call; replaced with literal code
	switch yynt {

	case 1:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:303
		{
			yylex.(*Parser).rootNode = node.NewRoot(yyDollar[1].list)

			// save position
			yylex.(*Parser).rootNode.SetPosition(yylex.(*Parser).positionBuilder.NewNodeListPosition(yyDollar[1].list))

			yylex.(*Parser).setFreeFloating(yylex.(*Parser).rootNode, freefloating.End, yylex.(*Parser).currentToken.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 2:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:316
		{
			yyVAL.token = yyDollar[1].token
		}
	case 3:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:316
		{
			yyVAL.token = yyDollar[1].token
		}
	case 4:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:316
		{
			yyVAL.token = yyDollar[1].token
		}
	case 5:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:316
		{
			yyVAL.token = yyDollar[1].token
		}
	case 6:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:316
		{
			yyVAL.token = yyDollar[1].token
		}
	case 7:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:316
		{
			yyVAL.token = yyDollar[1].token
		}
	case 8:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:316
		{
			yyVAL.token = yyDollar[1].token
		}
	case 9:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:316
		{
			yyVAL.token = yyDollar[1].token
		}
	case 10:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:317
		{
			yyVAL.token = yyDollar[1].token
		}
	case 11:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:317
		{
			yyVAL.token = yyDollar[1].token
		}
	case 12:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:317
		{
			yyVAL.token = yyDollar[1].token
		}
	case 13:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:317
		{
			yyVAL.token = yyDollar[1].token
		}
	case 14:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:317
		{
			yyVAL.token = yyDollar[1].token
		}
	case 15:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:317
		{
			yyVAL.token = yyDollar[1].token
		}
	case 16:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:317
		{
			yyVAL.token = yyDollar[1].token
		}
	case 17:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:317
		{
			yyVAL.token = yyDollar[1].token
		}
	case 18:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:317
		{
			yyVAL.token = yyDollar[1].token
		}
	case 19:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:317
		{
			yyVAL.token = yyDollar[1].token
		}
	case 20:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:317
		{
			yyVAL.token = yyDollar[1].token
		}
	case 21:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:317
		{
			yyVAL.token = yyDollar[1].token
		}
	case 22:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:318
		{
			yyVAL.token = yyDollar[1].token
		}
	case 23:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:318
		{
			yyVAL.token = yyDollar[1].token
		}
	case 24:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:318
		{
			yyVAL.token = yyDollar[1].token
		}
	case 25:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:318
		{
			yyVAL.token = yyDollar[1].token
		}
	case 26:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:318
		{
			yyVAL.token = yyDollar[1].token
		}
	case 27:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:318
		{
			yyVAL.token = yyDollar[1].token
		}
	case 28:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:318
		{
			yyVAL.token = yyDollar[1].token
		}
	case 29:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:318
		{
			yyVAL.token = yyDollar[1].token
		}
	case 30:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:318
		{
			yyVAL.token = yyDollar[1].token
		}
	case 31:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:318
		{
			yyVAL.token = yyDollar[1].token
		}
	case 32:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:319
		{
			yyVAL.token = yyDollar[1].token
		}
	case 33:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:319
		{
			yyVAL.token = yyDollar[1].token
		}
	case 34:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:319
		{
			yyVAL.token = yyDollar[1].token
		}
	case 35:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:319
		{
			yyVAL.token = yyDollar[1].token
		}
	case 36:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:319
		{
			yyVAL.token = yyDollar[1].token
		}
	case 37:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:319
		{
			yyVAL.token = yyDollar[1].token
		}
	case 38:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:319
		{
			yyVAL.token = yyDollar[1].token
		}
	case 39:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:319
		{
			yyVAL.token = yyDollar[1].token
		}
	case 40:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:319
		{
			yyVAL.token = yyDollar[1].token
		}
	case 41:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:319
		{
			yyVAL.token = yyDollar[1].token
		}
	case 42:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:320
		{
			yyVAL.token = yyDollar[1].token
		}
	case 43:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:320
		{
			yyVAL.token = yyDollar[1].token
		}
	case 44:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:320
		{
			yyVAL.token = yyDollar[1].token
		}
	case 45:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:320
		{
			yyVAL.token = yyDollar[1].token
		}
	case 46:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:320
		{
			yyVAL.token = yyDollar[1].token
		}
	case 47:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:320
		{
			yyVAL.token = yyDollar[1].token
		}
	case 48:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:320
		{
			yyVAL.token = yyDollar[1].token
		}
	case 49:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:320
		{
			yyVAL.token = yyDollar[1].token
		}
	case 50:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:320
		{
			yyVAL.token = yyDollar[1].token
		}
	case 51:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:320
		{
			yyVAL.token = yyDollar[1].token
		}
	case 52:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:320
		{
			yyVAL.token = yyDollar[1].token
		}
	case 53:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:321
		{
			yyVAL.token = yyDollar[1].token
		}
	case 54:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:321
		{
			yyVAL.token = yyDollar[1].token
		}
	case 55:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:321
		{
			yyVAL.token = yyDollar[1].token
		}
	case 56:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:321
		{
			yyVAL.token = yyDollar[1].token
		}
	case 57:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:321
		{
			yyVAL.token = yyDollar[1].token
		}
	case 58:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:321
		{
			yyVAL.token = yyDollar[1].token
		}
	case 59:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:321
		{
			yyVAL.token = yyDollar[1].token
		}
	case 60:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:321
		{
			yyVAL.token = yyDollar[1].token
		}
	case 61:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:322
		{
			yyVAL.token = yyDollar[1].token
		}
	case 62:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:322
		{
			yyVAL.token = yyDollar[1].token
		}
	case 63:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:322
		{
			yyVAL.token = yyDollar[1].token
		}
	case 64:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:322
		{
			yyVAL.token = yyDollar[1].token
		}
	case 65:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:322
		{
			yyVAL.token = yyDollar[1].token
		}
	case 66:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:322
		{
			yyVAL.token = yyDollar[1].token
		}
	case 67:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:322
		{
			yyVAL.token = yyDollar[1].token
		}
	case 68:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:322
		{
			yyVAL.token = yyDollar[1].token
		}
	case 69:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:322
		{
			yyVAL.token = yyDollar[1].token
		}
	case 70:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:327
		{
			yyVAL.token = yyDollar[1].token
		}
	case 71:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:330
		{
			yyVAL.token = yyDollar[1].token
		}
	case 72:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:330
		{
			yyVAL.token = yyDollar[1].token
		}
	case 73:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:330
		{
			yyVAL.token = yyDollar[1].token
		}
	case 74:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:330
		{
			yyVAL.token = yyDollar[1].token
		}
	case 75:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:330
		{
			yyVAL.token = yyDollar[1].token
		}
	case 76:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:330
		{
			yyVAL.token = yyDollar[1].token
		}
	case 77:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:335
		{
			yyVAL.token = yyDollar[1].token
		}
	case 78:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:339
		{
			yyVAL.token = yyDollar[1].token
		}
	case 79:
		yyDollar = yyS[yypt-2 : yypt+1]
		// line php7/php7.y:346
		{
			if inlineHtmlNode, ok := yyDollar[2].node.(*stmt.InlineHtml); ok && len(yyDollar[1].list) > 0 {
				prevNode := lastNode(yyDollar[1].list)
				yylex.(*Parser).splitSemiColonAndPhpCloseTag(inlineHtmlNode, prevNode)
			}

			if yyDollar[2].node != nil {
				yyVAL.list = append(yyDollar[1].list, yyDollar[2].node)
			}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 80:
		yyDollar = yyS[yypt-0 : yypt+1]
		// line php7/php7.y:359
		{
			yyVAL.list = []node.Node{}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 81:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:368
		{
			namePart := name.NewNamePart(yyDollar[1].token.Value)
			yyVAL.list = []node.Node{namePart}

			// save position
			namePart.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).setFreeFloating(namePart, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 82:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:381
		{
			namePart := name.NewNamePart(yyDollar[3].token.Value)
			yyVAL.list = append(yyDollar[1].list, namePart)

			// save position
			namePart.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[3].token))

			// save comments
			yylex.(*Parser).setFreeFloating(lastNode(yyDollar[1].list), freefloating.End, yyDollar[2].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(namePart, freefloating.Start, yyDollar[3].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 83:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:398
		{
			yyVAL.node = name.NewName(yyDollar[1].list)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodeListPosition(yyDollar[1].list))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].list[0], yyVAL.node)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 84:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:410
		{
			yyVAL.node = name.NewRelative(yyDollar[3].list)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodeListPosition(yyDollar[1].token, yyDollar[3].list))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Namespace, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 85:
		yyDollar = yyS[yypt-2 : yypt+1]
		// line php7/php7.y:423
		{
			yyVAL.node = name.NewFullyQualified(yyDollar[2].list)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodeListPosition(yyDollar[1].token, yyDollar[2].list))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 86:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:438
		{
			// error
			yyVAL.node = nil

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 87:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:445
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 88:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:451
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 89:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:457
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 90:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:463
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 91:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:469
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 92:
		yyDollar = yyS[yypt-4 : yypt+1]
		// line php7/php7.y:475
		{
			yyVAL.node = stmt.NewHaltCompiler()

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.HaltCompiller, yyDollar[2].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.OpenParenthesisToken, yyDollar[3].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.CloseParenthesisToken, yyDollar[4].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.SemiColon, yylex.(*Parser).GetFreeFloatingToken(yyDollar[4].token))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 93:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:491
		{
			name := name.NewName(yyDollar[2].list)
			yyVAL.node = stmt.NewNamespace(name, nil)

			// save position
			name.SetPosition(yylex.(*Parser).positionBuilder.NewNodeListPosition(yyDollar[2].list))
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).MoveFreeFloating(yyDollar[2].list[0], name)
			yylex.(*Parser).setFreeFloating(name, freefloating.End, yyDollar[3].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.SemiColon, yylex.(*Parser).GetFreeFloatingToken(yyDollar[3].token))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 94:
		yyDollar = yyS[yypt-5 : yypt+1]
		// line php7/php7.y:508
		{
			name := name.NewName(yyDollar[2].list)
			yyVAL.node = stmt.NewNamespace(name, yyDollar[4].list)

			// save position
			name.SetPosition(yylex.(*Parser).positionBuilder.NewNodeListPosition(yyDollar[2].list))
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[5].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).MoveFreeFloating(yyDollar[2].list[0], name)
			yylex.(*Parser).setFreeFloating(name, freefloating.End, yyDollar[3].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Stmts, yyDollar[5].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 95:
		yyDollar = yyS[yypt-4 : yypt+1]
		// line php7/php7.y:525
		{
			yyVAL.node = stmt.NewNamespace(nil, yyDollar[3].list)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Namespace, yyDollar[2].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Stmts, yyDollar[4].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 96:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:539
		{
			yyVAL.node = yyDollar[2].node

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.UseDeclarationList, yyDollar[3].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.SemiColon, yylex.(*Parser).GetFreeFloatingToken(yyDollar[3].token))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 97:
		yyDollar = yyS[yypt-4 : yypt+1]
		// line php7/php7.y:553
		{
			yyVAL.node = yyDollar[3].node.(*stmt.GroupUse).SetUseType(yyDollar[2].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.UseDeclarationList, yyDollar[4].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.SemiColon, yylex.(*Parser).GetFreeFloatingToken(yyDollar[4].token))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 98:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:567
		{
			yyVAL.node = stmt.NewUseList(nil, yyDollar[2].list)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.UseDeclarationList, yyDollar[3].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.SemiColon, yylex.(*Parser).GetFreeFloatingToken(yyDollar[3].token))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 99:
		yyDollar = yyS[yypt-4 : yypt+1]
		// line php7/php7.y:581
		{
			yyVAL.node = stmt.NewUseList(yyDollar[2].node, yyDollar[3].list)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.UseDeclarationList, yyDollar[4].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.SemiColon, yylex.(*Parser).GetFreeFloatingToken(yyDollar[4].token))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 100:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:595
		{
			yyVAL.node = stmt.NewConstList(yyDollar[2].list)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Stmts, yyDollar[3].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.SemiColon, yylex.(*Parser).GetFreeFloatingToken(yyDollar[3].token))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 101:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:612
		{
			yyVAL.node = node.NewIdentifier(yyDollar[1].token.Value)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 102:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:624
		{
			yyVAL.node = node.NewIdentifier(yyDollar[1].token.Value)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 103:
		yyDollar = yyS[yypt-6 : yypt+1]
		// line php7/php7.y:639
		{
			name := name.NewName(yyDollar[1].list)
			yyVAL.node = stmt.NewGroupUse(nil, name, yyDollar[4].list)

			// save position
			name.SetPosition(yylex.(*Parser).positionBuilder.NewNodeListPosition(yyDollar[1].list))
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodeListTokenPosition(yyDollar[1].list, yyDollar[6].token))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].list[0], name)
			yylex.(*Parser).setFreeFloating(name, freefloating.End, yyDollar[2].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Slash, yyDollar[3].token.FreeFloating)
			if yyDollar[5].token != nil {
				yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Stmts, append(yyDollar[5].token.FreeFloating, append(yylex.(*Parser).GetFreeFloatingToken(yyDollar[5].token), yyDollar[6].token.FreeFloating...)...))
			} else {
				yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Stmts, yyDollar[6].token.FreeFloating)
			}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 104:
		yyDollar = yyS[yypt-7 : yypt+1]
		// line php7/php7.y:660
		{
			name := name.NewName(yyDollar[2].list)
			yyVAL.node = stmt.NewGroupUse(nil, name, yyDollar[5].list)

			// save position
			name.SetPosition(yylex.(*Parser).positionBuilder.NewNodeListPosition(yyDollar[2].list))
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[7].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.UseType, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).MoveFreeFloating(yyDollar[2].list[0], name)
			yylex.(*Parser).setFreeFloating(name, freefloating.End, yyDollar[3].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Slash, yyDollar[4].token.FreeFloating)
			if yyDollar[6].token != nil {
				yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Stmts, append(yyDollar[6].token.FreeFloating, append(yylex.(*Parser).GetFreeFloatingToken(yyDollar[6].token), yyDollar[7].token.FreeFloating...)...))
			} else {
				yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Stmts, yyDollar[7].token.FreeFloating)
			}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 105:
		yyDollar = yyS[yypt-6 : yypt+1]
		// line php7/php7.y:685
		{
			name := name.NewName(yyDollar[1].list)
			yyVAL.node = stmt.NewGroupUse(nil, name, yyDollar[4].list)

			// save position
			name.SetPosition(yylex.(*Parser).positionBuilder.NewNodeListPosition(yyDollar[1].list))
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodeListTokenPosition(yyDollar[1].list, yyDollar[6].token))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].list[0], name)
			yylex.(*Parser).setFreeFloating(name, freefloating.End, yyDollar[2].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Slash, yyDollar[3].token.FreeFloating)
			if yyDollar[5].token != nil {
				yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Stmts, append(yyDollar[5].token.FreeFloating, append(yylex.(*Parser).GetFreeFloatingToken(yyDollar[5].token), yyDollar[6].token.FreeFloating...)...))
			} else {
				yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Stmts, yyDollar[6].token.FreeFloating)
			}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 106:
		yyDollar = yyS[yypt-7 : yypt+1]
		// line php7/php7.y:706
		{
			name := name.NewName(yyDollar[2].list)
			yyVAL.node = stmt.NewGroupUse(nil, name, yyDollar[5].list)

			// save position
			name.SetPosition(yylex.(*Parser).positionBuilder.NewNodeListPosition(yyDollar[2].list))
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[7].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Use, append(yyDollar[1].token.FreeFloating, yylex.(*Parser).GetFreeFloatingToken(yyDollar[1].token)...))
			yylex.(*Parser).MoveFreeFloating(yyDollar[2].list[0], name)
			yylex.(*Parser).setFreeFloating(name, freefloating.End, yyDollar[3].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Slash, yyDollar[4].token.FreeFloating)
			if yyDollar[6].token != nil {
				yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Stmts, append(yyDollar[6].token.FreeFloating, append(yylex.(*Parser).GetFreeFloatingToken(yyDollar[6].token), yyDollar[7].token.FreeFloating...)...))
			} else {
				yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Stmts, yyDollar[7].token.FreeFloating)
			}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 107:
		yyDollar = yyS[yypt-0 : yypt+1]
		// line php7/php7.y:731
		{
			yyVAL.token = nil
		}
	case 108:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:735
		{
			yyVAL.token = yyDollar[1].token
		}
	case 109:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:742
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)

			// save comments
			yylex.(*Parser).setFreeFloating(lastNode(yyDollar[1].list), freefloating.End, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 110:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:751
		{
			yyVAL.list = []node.Node{yyDollar[1].node}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 111:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:760
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)

			// save comments
			yylex.(*Parser).setFreeFloating(lastNode(yyDollar[1].list), freefloating.End, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 112:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:769
		{
			yyVAL.list = []node.Node{yyDollar[1].node}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 113:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:778
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)

			// save comments
			yylex.(*Parser).setFreeFloating(lastNode(yyDollar[1].list), freefloating.End, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 114:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:787
		{
			yyVAL.list = []node.Node{yyDollar[1].node}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 115:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:796
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 116:
		yyDollar = yyS[yypt-2 : yypt+1]
		// line php7/php7.y:802
		{
			yyVAL.node = yyDollar[2].node.(*stmt.Use).SetUseType(yyDollar[1].node)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 117:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:811
		{
			name := name.NewName(yyDollar[1].list)
			yyVAL.node = stmt.NewUse(nil, name, nil)

			// save position
			name.SetPosition(yylex.(*Parser).positionBuilder.NewNodeListPosition(yyDollar[1].list))
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodeListPosition(yyDollar[1].list))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].list[0], name)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 118:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:825
		{
			name := name.NewName(yyDollar[1].list)
			alias := node.NewIdentifier(yyDollar[3].token.Value)
			yyVAL.node = stmt.NewUse(nil, name, alias)

			// save position
			name.SetPosition(yylex.(*Parser).positionBuilder.NewNodeListPosition(yyDollar[1].list))
			alias.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[3].token))
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodeListTokenPosition(yyDollar[1].list, yyDollar[3].token))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].list[0], name)
			yylex.(*Parser).setFreeFloating(name, freefloating.End, yyDollar[2].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(alias, freefloating.Start, yyDollar[3].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 119:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:846
		{
			yyVAL.node = yyDollar[1].node

			// save coments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node.(*stmt.Use).Use, yyVAL.node)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 120:
		yyDollar = yyS[yypt-2 : yypt+1]
		// line php7/php7.y:855
		{
			yyVAL.node = yyDollar[2].node

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Slash, yylex.(*Parser).GetFreeFloatingToken(yyDollar[1].token))

			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Slash, yylex.(*Parser).GetFreeFloatingToken(yyDollar[1].token))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 121:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:871
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)

			// save comments
			yylex.(*Parser).setFreeFloating(lastNode(yyDollar[1].list), freefloating.End, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 122:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:880
		{
			yyVAL.list = []node.Node{yyDollar[1].node}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 123:
		yyDollar = yyS[yypt-2 : yypt+1]
		// line php7/php7.y:889
		{
			if inlineHtmlNode, ok := yyDollar[2].node.(*stmt.InlineHtml); ok && len(yyDollar[1].list) > 0 {
				prevNode := lastNode(yyDollar[1].list)
				yylex.(*Parser).splitSemiColonAndPhpCloseTag(inlineHtmlNode, prevNode)
			}

			if yyDollar[2].node != nil {
				yyVAL.list = append(yyDollar[1].list, yyDollar[2].node)
			}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 124:
		yyDollar = yyS[yypt-0 : yypt+1]
		// line php7/php7.y:902
		{
			yyVAL.list = []node.Node{}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 125:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:911
		{
			// error
			yyVAL.node = nil

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 126:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:918
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 127:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:924
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 128:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:930
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 129:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:936
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 130:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:942
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 131:
		yyDollar = yyS[yypt-4 : yypt+1]
		// line php7/php7.y:948
		{
			yyVAL.node = stmt.NewHaltCompiler()

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.HaltCompiller, yyDollar[2].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.OpenParenthesisToken, yyDollar[3].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.CloseParenthesisToken, yyDollar[4].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.SemiColon, yylex.(*Parser).GetFreeFloatingToken(yyDollar[4].token))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 132:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:966
		{
			yyVAL.node = stmt.NewStmtList(yyDollar[2].list)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Stmts, yyDollar[3].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 133:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:979
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 134:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:985
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 135:
		yyDollar = yyS[yypt-5 : yypt+1]
		// line php7/php7.y:991
		{
			switch n := yyDollar[5].node.(type) {
			case *stmt.While:
				n.Cond = yyDollar[3].node
			case *stmt.AltWhile:
				n.Cond = yyDollar[3].node
			}

			yyVAL.node = yyDollar[5].node

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[5].node))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.While, yyDollar[2].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Expr, yyDollar[4].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 136:
		yyDollar = yyS[yypt-7 : yypt+1]
		// line php7/php7.y:1012
		{
			yyVAL.node = stmt.NewDo(yyDollar[2].node, yyDollar[5].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[7].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Stmts, yyDollar[3].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.While, yyDollar[4].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Expr, yyDollar[6].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Cond, yyDollar[7].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.SemiColon, yylex.(*Parser).GetFreeFloatingToken(yyDollar[7].token))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 137:
		yyDollar = yyS[yypt-9 : yypt+1]
		// line php7/php7.y:1029
		{
			switch n := yyDollar[9].node.(type) {
			case *stmt.For:
				n.Init = yyDollar[3].list
				n.Cond = yyDollar[5].list
				n.Loop = yyDollar[7].list
			case *stmt.AltFor:
				n.Init = yyDollar[3].list
				n.Cond = yyDollar[5].list
				n.Loop = yyDollar[7].list
			}

			yyVAL.node = yyDollar[9].node

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[9].node))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.For, yyDollar[2].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.InitExpr, yyDollar[4].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.CondExpr, yyDollar[6].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.IncExpr, yyDollar[8].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 138:
		yyDollar = yyS[yypt-5 : yypt+1]
		// line php7/php7.y:1056
		{
			switch n := yyDollar[5].node.(type) {
			case *stmt.Switch:
				n.Cond = yyDollar[3].node
			case *stmt.AltSwitch:
				n.Cond = yyDollar[3].node
			default:
				panic("unexpected node type")
			}

			yyVAL.node = yyDollar[5].node

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[5].node))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Switch, yyDollar[2].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Expr, yyDollar[4].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 139:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:1079
		{
			yyVAL.node = stmt.NewBreak(yyDollar[2].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Expr, yyDollar[3].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.SemiColon, yylex.(*Parser).GetFreeFloatingToken(yyDollar[3].token))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 140:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:1093
		{
			yyVAL.node = stmt.NewContinue(yyDollar[2].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Expr, yyDollar[3].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.SemiColon, yylex.(*Parser).GetFreeFloatingToken(yyDollar[3].token))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 141:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:1107
		{
			yyVAL.node = stmt.NewReturn(yyDollar[2].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Expr, yyDollar[3].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.SemiColon, yylex.(*Parser).GetFreeFloatingToken(yyDollar[3].token))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 142:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:1121
		{
			yyVAL.node = stmt.NewGlobal(yyDollar[2].list)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.VarList, yyDollar[3].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.SemiColon, yylex.(*Parser).GetFreeFloatingToken(yyDollar[3].token))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 143:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:1135
		{
			yyVAL.node = stmt.NewStatic(yyDollar[2].list)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.VarList, yyDollar[3].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.SemiColon, yylex.(*Parser).GetFreeFloatingToken(yyDollar[3].token))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 144:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:1149
		{
			yyVAL.node = stmt.NewEcho(yyDollar[2].list)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Echo, yylex.(*Parser).GetFreeFloatingToken(yyDollar[1].token))
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Expr, yyDollar[3].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.SemiColon, yylex.(*Parser).GetFreeFloatingToken(yyDollar[3].token))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 145:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:1164
		{
			yyVAL.node = stmt.NewInlineHtml(yyDollar[1].token.Value)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 146:
		yyDollar = yyS[yypt-2 : yypt+1]
		// line php7/php7.y:1176
		{
			yyVAL.node = stmt.NewExpression(yyDollar[1].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodeTokenPosition(yyDollar[1].node, yyDollar[2].token))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Expr, yyDollar[2].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.SemiColon, yylex.(*Parser).GetFreeFloatingToken(yyDollar[2].token))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 147:
		yyDollar = yyS[yypt-6 : yypt+1]
		// line php7/php7.y:1190
		{
			yyVAL.node = stmt.NewUnset(yyDollar[3].list)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[6].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Unset, yyDollar[2].token.FreeFloating)
			if yyDollar[4].token != nil {
				yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.VarList, append(yyDollar[4].token.FreeFloating, append(yylex.(*Parser).GetFreeFloatingToken(yyDollar[4].token), yyDollar[5].token.FreeFloating...)...))
			} else {
				yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.VarList, yyDollar[5].token.FreeFloating)
			}
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.CloseParenthesisToken, yyDollar[6].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.SemiColon, yylex.(*Parser).GetFreeFloatingToken(yyDollar[6].token))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 148:
		yyDollar = yyS[yypt-7 : yypt+1]
		// line php7/php7.y:1210
		{
			switch n := yyDollar[7].node.(type) {
			case *stmt.Foreach:
				n.Expr = yyDollar[3].node
				n.Variable = yyDollar[5].node
			case *stmt.AltForeach:
				n.Expr = yyDollar[3].node
				n.Variable = yyDollar[5].node
			}

			yyVAL.node = yyDollar[7].node

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[7].node))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Foreach, yyDollar[2].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Expr, yyDollar[4].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Var, yyDollar[6].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 149:
		yyDollar = yyS[yypt-9 : yypt+1]
		// line php7/php7.y:1235
		{
			switch n := yyDollar[9].node.(type) {
			case *stmt.Foreach:
				n.Expr = yyDollar[3].node
				n.Key = yyDollar[5].node
				n.Variable = yyDollar[7].node
			case *stmt.AltForeach:
				n.Expr = yyDollar[3].node
				n.Key = yyDollar[5].node
				n.Variable = yyDollar[7].node
			}

			yyVAL.node = yyDollar[9].node

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[9].node))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Foreach, yyDollar[2].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Expr, yyDollar[4].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Key, yyDollar[6].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Var, yyDollar[8].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 150:
		yyDollar = yyS[yypt-5 : yypt+1]
		// line php7/php7.y:1262
		{
			yyVAL.node = yyDollar[5].node
			yyVAL.node.(*stmt.Declare).Consts = yyDollar[3].list

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[5].node))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Declare, yyDollar[2].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.ConstList, yyDollar[4].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 151:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:1277
		{
			yyVAL.node = stmt.NewNop()

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.SemiColon, yylex.(*Parser).GetFreeFloatingToken(yyDollar[1].token))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 152:
		yyDollar = yyS[yypt-6 : yypt+1]
		// line php7/php7.y:1290
		{
			if yyDollar[6].node == nil {
				yyVAL.node = stmt.NewTry(yyDollar[3].list, yyDollar[5].list, yyDollar[6].node)
				yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodeListPosition(yyDollar[1].token, yyDollar[5].list))
			} else {
				yyVAL.node = stmt.NewTry(yyDollar[3].list, yyDollar[5].list, yyDollar[6].node)
				yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[6].node))
			}

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Try, yyDollar[2].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Stmts, yyDollar[4].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 153:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:1307
		{
			yyVAL.node = stmt.NewThrow(yyDollar[2].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Expr, yyDollar[3].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.SemiColon, yylex.(*Parser).GetFreeFloatingToken(yyDollar[3].token))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 154:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:1321
		{
			label := node.NewIdentifier(yyDollar[2].token.Value)
			yyVAL.node = stmt.NewGoto(label)

			// save position
			label.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[2].token))
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(label, freefloating.Start, yyDollar[2].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Label, yyDollar[3].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.SemiColon, yylex.(*Parser).GetFreeFloatingToken(yyDollar[3].token))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 155:
		yyDollar = yyS[yypt-2 : yypt+1]
		// line php7/php7.y:1338
		{
			label := node.NewIdentifier(yyDollar[1].token.Value)
			yyVAL.node = stmt.NewLabel(label)

			// save position
			label.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[2].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Label, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 156:
		yyDollar = yyS[yypt-0 : yypt+1]
		// line php7/php7.y:1355
		{
			yyVAL.list = []node.Node{}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 157:
		yyDollar = yyS[yypt-9 : yypt+1]
		// line php7/php7.y:1361
		{
			identifier := node.NewIdentifier(strings.TrimLeftFunc(yyDollar[5].token.Value, isDollar))
			variable := expr.NewVariable(identifier)
			catch := stmt.NewCatch(yyDollar[4].list, variable, yyDollar[8].list)
			yyVAL.list = append(yyDollar[1].list, catch)

			// save position
			identifier.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[5].token))
			variable.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[5].token))
			catch.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[2].token, yyDollar[9].token))

			// save comments
			yylex.(*Parser).setFreeFloating(catch, freefloating.Start, yyDollar[2].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(catch, freefloating.Catch, yyDollar[3].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(variable, freefloating.Start, yyDollar[5].token.FreeFloating)
			yylex.(*Parser).addDollarToken(variable)
			yylex.(*Parser).setFreeFloating(catch, freefloating.Var, yyDollar[6].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(catch, freefloating.Cond, yyDollar[7].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(catch, freefloating.Stmts, yyDollar[9].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 158:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:1386
		{
			yyVAL.list = []node.Node{yyDollar[1].node}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 159:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:1392
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)

			// save comments
			yylex.(*Parser).setFreeFloating(lastNode(yyDollar[1].list), freefloating.End, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 160:
		yyDollar = yyS[yypt-0 : yypt+1]
		// line php7/php7.y:1404
		{
			yyVAL.node = nil

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 161:
		yyDollar = yyS[yypt-4 : yypt+1]
		// line php7/php7.y:1410
		{
			yyVAL.node = stmt.NewFinally(yyDollar[3].list)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Finally, yyDollar[2].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Stmts, yyDollar[4].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 162:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:1427
		{
			yyVAL.list = []node.Node{yyDollar[1].node}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 163:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:1433
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)

			// save comments
			yylex.(*Parser).setFreeFloating(lastNode(yyDollar[1].list), freefloating.End, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 164:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:1445
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 165:
		yyDollar = yyS[yypt-11 : yypt+1]
		// line php7/php7.y:1454
		{
			name := node.NewIdentifier(yyDollar[3].token.Value)
			yyVAL.node = stmt.NewFunction(name, yyDollar[2].token != nil, yyDollar[6].list, yyDollar[8].node, yyDollar[10].list, yyDollar[4].str)

			// save position
			name.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[3].token))
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[11].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			if yyDollar[2].token != nil {
				yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Function, yyDollar[2].token.FreeFloating)
				yylex.(*Parser).setFreeFloating(name, freefloating.Start, yyDollar[3].token.FreeFloating)
			} else {
				yylex.(*Parser).setFreeFloating(name, freefloating.Start, yyDollar[3].token.FreeFloating)
			}
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Name, yyDollar[5].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.ParamList, yyDollar[7].token.FreeFloating)
			if yyDollar[8].node != nil {
				yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Params, (*yyDollar[8].node.GetFreeFloating())[freefloating.Colon])
				delete((*yyDollar[8].node.GetFreeFloating()), freefloating.Colon)
			}
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.ReturnType, yyDollar[9].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Stmts, yyDollar[11].token.FreeFloating)

			// normalize
			if yyDollar[8].node == nil {
				yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Params, (*yyVAL.node.GetFreeFloating())[freefloating.ReturnType])
				delete((*yyVAL.node.GetFreeFloating()), freefloating.ReturnType)
			}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 166:
		yyDollar = yyS[yypt-0 : yypt+1]
		// line php7/php7.y:1490
		{
			yyVAL.token = nil
		}
	case 167:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:1494
		{
			yyVAL.token = yyDollar[1].token
		}
	case 168:
		yyDollar = yyS[yypt-0 : yypt+1]
		// line php7/php7.y:1501
		{
			yyVAL.token = nil
		}
	case 169:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:1505
		{
			yyVAL.token = yyDollar[1].token
		}
	case 170:
		yyDollar = yyS[yypt-9 : yypt+1]
		// line php7/php7.y:1512
		{
			name := node.NewIdentifier(yyDollar[3].token.Value)
			yyVAL.node = stmt.NewClass(name, yyDollar[1].list, nil, yyDollar[4].ClassExtends, yyDollar[5].ClassImplements, yyDollar[8].list, yyDollar[6].str)

			// save position
			name.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[3].token))
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewOptionalListTokensPosition(yyDollar[1].list, yyDollar[2].token, yyDollar[9].token))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].list[0], yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.ModifierList, yyDollar[2].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(name, freefloating.Start, yyDollar[3].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Name, yyDollar[7].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Stmts, yyDollar[9].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 171:
		yyDollar = yyS[yypt-8 : yypt+1]
		// line php7/php7.y:1530
		{
			name := node.NewIdentifier(yyDollar[2].token.Value)
			yyVAL.node = stmt.NewClass(name, nil, nil, yyDollar[3].ClassExtends, yyDollar[4].ClassImplements, yyDollar[7].list, yyDollar[5].str)

			// save position
			name.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[2].token))
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[8].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(name, freefloating.Start, yyDollar[2].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Name, yyDollar[6].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Stmts, yyDollar[8].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 172:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:1550
		{
			yyVAL.list = []node.Node{yyDollar[1].node}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 173:
		yyDollar = yyS[yypt-2 : yypt+1]
		// line php7/php7.y:1556
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[2].node)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 174:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:1565
		{
			yyVAL.node = node.NewIdentifier(yyDollar[1].token.Value)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 175:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:1577
		{
			yyVAL.node = node.NewIdentifier(yyDollar[1].token.Value)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 176:
		yyDollar = yyS[yypt-6 : yypt+1]
		// line php7/php7.y:1592
		{
			name := node.NewIdentifier(yyDollar[2].token.Value)
			yyVAL.node = stmt.NewTrait(name, yyDollar[5].list, yyDollar[3].str)

			// save position
			name.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[2].token))
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[6].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(name, freefloating.Start, yyDollar[2].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Name, yyDollar[4].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Stmts, yyDollar[6].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 177:
		yyDollar = yyS[yypt-7 : yypt+1]
		// line php7/php7.y:1612
		{
			name := node.NewIdentifier(yyDollar[2].token.Value)
			yyVAL.node = stmt.NewInterface(name, yyDollar[3].InterfaceExtends, yyDollar[6].list, yyDollar[4].str)

			// save position
			name.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[2].token))
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[7].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(name, freefloating.Start, yyDollar[2].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Name, yyDollar[5].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Stmts, yyDollar[7].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 178:
		yyDollar = yyS[yypt-0 : yypt+1]
		// line php7/php7.y:1632
		{
			yyVAL.ClassExtends = nil

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 179:
		yyDollar = yyS[yypt-2 : yypt+1]
		// line php7/php7.y:1638
		{
			yyVAL.ClassExtends = stmt.NewClassExtends(yyDollar[2].node)

			// save position
			yyVAL.ClassExtends.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.ClassExtends, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 180:
		yyDollar = yyS[yypt-0 : yypt+1]
		// line php7/php7.y:1653
		{
			yyVAL.InterfaceExtends = nil

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 181:
		yyDollar = yyS[yypt-2 : yypt+1]
		// line php7/php7.y:1659
		{
			yyVAL.InterfaceExtends = stmt.NewInterfaceExtends(yyDollar[2].list)

			// save position
			yyVAL.InterfaceExtends.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodeListPosition(yyDollar[1].token, yyDollar[2].list))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.InterfaceExtends, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 182:
		yyDollar = yyS[yypt-0 : yypt+1]
		// line php7/php7.y:1674
		{
			yyVAL.ClassImplements = nil

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 183:
		yyDollar = yyS[yypt-2 : yypt+1]
		// line php7/php7.y:1680
		{
			yyVAL.ClassImplements = stmt.NewClassImplements(yyDollar[2].list)

			// save position
			yyVAL.ClassImplements.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodeListPosition(yyDollar[1].token, yyDollar[2].list))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.ClassImplements, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 184:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:1695
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 185:
		yyDollar = yyS[yypt-2 : yypt+1]
		// line php7/php7.y:1701
		{
			yyVAL.node = expr.NewReference(yyDollar[2].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 186:
		yyDollar = yyS[yypt-4 : yypt+1]
		// line php7/php7.y:1713
		{
			yyVAL.node = expr.NewList(yyDollar[3].list)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.List, yyDollar[2].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.ArrayPairList, yyDollar[4].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 187:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:1727
		{
			yyVAL.node = expr.NewShortList(yyDollar[2].list)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))

			// save commentsc
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.ArrayPairList, yyDollar[3].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 188:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:1743
		{
			yyVAL.node = stmt.NewFor(nil, nil, nil, yyDollar[1].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodePosition(yyDollar[1].node))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 189:
		yyDollar = yyS[yypt-4 : yypt+1]
		// line php7/php7.y:1752
		{
			stmtList := stmt.NewStmtList(yyDollar[2].list)
			yyVAL.node = stmt.NewAltFor(nil, nil, nil, stmtList)

			// save position
			stmtList.SetPosition(yylex.(*Parser).positionBuilder.NewNodeListPosition(yyDollar[2].list))
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Cond, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Stmts, yyDollar[3].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.AltEnd, yyDollar[4].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.SemiColon, yylex.(*Parser).GetFreeFloatingToken(yyDollar[4].token))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 190:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:1772
		{
			yyVAL.node = stmt.NewForeach(nil, nil, nil, yyDollar[1].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodePosition(yyDollar[1].node))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 191:
		yyDollar = yyS[yypt-4 : yypt+1]
		// line php7/php7.y:1781
		{
			stmtList := stmt.NewStmtList(yyDollar[2].list)
			yyVAL.node = stmt.NewAltForeach(nil, nil, nil, stmtList)

			// save position
			stmtList.SetPosition(yylex.(*Parser).positionBuilder.NewNodeListPosition(yyDollar[2].list))
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Cond, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Stmts, yyDollar[3].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.AltEnd, yyDollar[4].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.SemiColon, yylex.(*Parser).GetFreeFloatingToken(yyDollar[4].token))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 192:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:1801
		{
			yyVAL.node = stmt.NewDeclare(nil, yyDollar[1].node, false)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodePosition(yyDollar[1].node))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 193:
		yyDollar = yyS[yypt-4 : yypt+1]
		// line php7/php7.y:1810
		{
			stmtList := stmt.NewStmtList(yyDollar[2].list)
			yyVAL.node = stmt.NewDeclare(nil, stmtList, true)

			// save position
			stmtList.SetPosition(yylex.(*Parser).positionBuilder.NewNodeListPosition(yyDollar[2].list))
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Cond, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Stmts, yyDollar[3].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.AltEnd, yyDollar[4].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.SemiColon, yylex.(*Parser).GetFreeFloatingToken(yyDollar[4].token))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 194:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:1830
		{
			caseList := stmt.NewCaseList(yyDollar[2].list)
			yyVAL.node = stmt.NewSwitch(nil, caseList)

			// save position
			caseList.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))

			// save comments
			yylex.(*Parser).setFreeFloating(caseList, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(caseList, freefloating.CaseListEnd, yyDollar[3].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 195:
		yyDollar = yyS[yypt-4 : yypt+1]
		// line php7/php7.y:1845
		{
			caseList := stmt.NewCaseList(yyDollar[3].list)
			yyVAL.node = stmt.NewSwitch(nil, caseList)

			// save position
			caseList.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))

			// save comments
			yylex.(*Parser).setFreeFloating(caseList, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(caseList, freefloating.CaseListStart, append(yyDollar[2].token.FreeFloating, yylex.(*Parser).GetFreeFloatingToken(yyDollar[2].token)...))
			yylex.(*Parser).setFreeFloating(caseList, freefloating.CaseListEnd, yyDollar[4].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 196:
		yyDollar = yyS[yypt-4 : yypt+1]
		// line php7/php7.y:1861
		{
			caseList := stmt.NewCaseList(yyDollar[2].list)
			yyVAL.node = stmt.NewAltSwitch(nil, caseList)

			// save position
			caseList.SetPosition(yylex.(*Parser).positionBuilder.NewNodeListPosition(yyDollar[2].list))
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Cond, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(caseList, freefloating.CaseListEnd, yyDollar[3].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.AltEnd, yyDollar[4].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.SemiColon, yylex.(*Parser).GetFreeFloatingToken(yyDollar[4].token))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 197:
		yyDollar = yyS[yypt-5 : yypt+1]
		// line php7/php7.y:1878
		{

			caseList := stmt.NewCaseList(yyDollar[3].list)
			yyVAL.node = stmt.NewAltSwitch(nil, caseList)

			// save position
			caseList.SetPosition(yylex.(*Parser).positionBuilder.NewNodeListPosition(yyDollar[3].list))
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[5].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Cond, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(caseList, freefloating.CaseListStart, append(yyDollar[2].token.FreeFloating, yylex.(*Parser).GetFreeFloatingToken(yyDollar[2].token)...))
			yylex.(*Parser).setFreeFloating(caseList, freefloating.CaseListEnd, yyDollar[4].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.AltEnd, yyDollar[5].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.SemiColon, yylex.(*Parser).GetFreeFloatingToken(yyDollar[5].token))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 198:
		yyDollar = yyS[yypt-0 : yypt+1]
		// line php7/php7.y:1900
		{
			yyVAL.list = []node.Node{}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 199:
		yyDollar = yyS[yypt-5 : yypt+1]
		// line php7/php7.y:1906
		{
			_case := stmt.NewCase(yyDollar[3].node, yyDollar[5].list)
			yyVAL.list = append(yyDollar[1].list, _case)

			// save position
			_case.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodeListPosition(yyDollar[2].token, yyDollar[5].list))

			// save comments
			yylex.(*Parser).setFreeFloating(_case, freefloating.Start, yyDollar[2].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(_case, freefloating.Expr, append(yyDollar[4].token.FreeFloating))
			yylex.(*Parser).setFreeFloating(_case, freefloating.CaseSeparator, yylex.(*Parser).GetFreeFloatingToken(yyDollar[4].token))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 200:
		yyDollar = yyS[yypt-4 : yypt+1]
		// line php7/php7.y:1921
		{
			_default := stmt.NewDefault(yyDollar[4].list)
			yyVAL.list = append(yyDollar[1].list, _default)

			// save position
			_default.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodeListPosition(yyDollar[2].token, yyDollar[4].list))

			// save comments
			yylex.(*Parser).setFreeFloating(_default, freefloating.Start, yyDollar[2].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(_default, freefloating.Default, yyDollar[3].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(_default, freefloating.CaseSeparator, yylex.(*Parser).GetFreeFloatingToken(yyDollar[3].token))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 201:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:1939
		{
			yyVAL.token = yyDollar[1].token
		}
	case 202:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:1943
		{
			yyVAL.token = yyDollar[1].token
		}
	case 203:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:1950
		{
			yyVAL.node = stmt.NewWhile(nil, yyDollar[1].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodePosition(yyDollar[1].node))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 204:
		yyDollar = yyS[yypt-4 : yypt+1]
		// line php7/php7.y:1959
		{
			stmtList := stmt.NewStmtList(yyDollar[2].list)
			yyVAL.node = stmt.NewAltWhile(nil, stmtList)

			// save position
			stmtList.SetPosition(yylex.(*Parser).positionBuilder.NewNodeListPosition(yyDollar[2].list))
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Cond, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Stmts, yyDollar[3].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.AltEnd, yyDollar[4].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.SemiColon, yylex.(*Parser).GetFreeFloatingToken(yyDollar[4].token))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 205:
		yyDollar = yyS[yypt-5 : yypt+1]
		// line php7/php7.y:1979
		{
			yyVAL.node = stmt.NewIf(yyDollar[3].node, yyDollar[5].node, nil, nil)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[5].node))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.If, yyDollar[2].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Expr, yyDollar[4].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 206:
		yyDollar = yyS[yypt-6 : yypt+1]
		// line php7/php7.y:1993
		{
			_elseIf := stmt.NewElseIf(yyDollar[4].node, yyDollar[6].node)
			yyVAL.node = yyDollar[1].node.(*stmt.If).AddElseIf(_elseIf)

			// save position
			_elseIf.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[2].token, yyDollar[6].node))
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[6].node))

			// save comments
			yylex.(*Parser).setFreeFloating(_elseIf, freefloating.Start, yyDollar[2].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(_elseIf, freefloating.ElseIf, yyDollar[3].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(_elseIf, freefloating.Expr, yyDollar[5].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 207:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:2012
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 208:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:2018
		{
			_else := stmt.NewElse(yyDollar[3].node)
			yyVAL.node = yyDollar[1].node.(*stmt.If).SetElse(_else)

			// save position
			_else.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[2].token, yyDollar[3].node))
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).setFreeFloating(_else, freefloating.Start, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 209:
		yyDollar = yyS[yypt-6 : yypt+1]
		// line php7/php7.y:2035
		{
			stmts := stmt.NewStmtList(yyDollar[6].list)
			yyVAL.node = stmt.NewAltIf(yyDollar[3].node, stmts, nil, nil)

			// save position
			stmts.SetPosition(yylex.(*Parser).positionBuilder.NewNodeListPosition(yyDollar[6].list))
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodeListPosition(yyDollar[1].token, yyDollar[6].list))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.If, yyDollar[2].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Expr, yyDollar[4].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Cond, yyDollar[5].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 210:
		yyDollar = yyS[yypt-7 : yypt+1]
		// line php7/php7.y:2052
		{
			stmts := stmt.NewStmtList(yyDollar[7].list)
			_elseIf := stmt.NewAltElseIf(yyDollar[4].node, stmts)
			yyVAL.node = yyDollar[1].node.(*stmt.AltIf).AddElseIf(_elseIf)

			// save position
			stmts.SetPosition(yylex.(*Parser).positionBuilder.NewNodeListPosition(yyDollar[7].list))
			_elseIf.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodeListPosition(yyDollar[2].token, yyDollar[7].list))

			// save comments
			yylex.(*Parser).setFreeFloating(_elseIf, freefloating.Start, yyDollar[2].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(_elseIf, freefloating.ElseIf, yyDollar[3].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(_elseIf, freefloating.Expr, yyDollar[5].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(_elseIf, freefloating.Cond, yyDollar[6].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 211:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:2073
		{
			yyVAL.node = yyDollar[1].node

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodeTokenPosition(yyDollar[1].node, yyDollar[3].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Stmts, yyDollar[2].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.AltEnd, yyDollar[3].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.SemiColon, yylex.(*Parser).GetFreeFloatingToken(yyDollar[3].token))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 212:
		yyDollar = yyS[yypt-6 : yypt+1]
		// line php7/php7.y:2087
		{
			stmts := stmt.NewStmtList(yyDollar[4].list)
			_else := stmt.NewAltElse(stmts)
			yyVAL.node = yyDollar[1].node.(*stmt.AltIf).SetElse(_else)

			// save position
			stmts.SetPosition(yylex.(*Parser).positionBuilder.NewNodeListPosition(yyDollar[4].list))
			_else.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodeListPosition(yyDollar[2].token, yyDollar[4].list))
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodeTokenPosition(yyDollar[1].node, yyDollar[6].token))

			// save comments
			yylex.(*Parser).setFreeFloating(_else, freefloating.Start, yyDollar[2].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(_else, freefloating.Else, yyDollar[3].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Stmts, yyDollar[5].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.AltEnd, yyDollar[6].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.SemiColon, yylex.(*Parser).GetFreeFloatingToken(yyDollar[6].token))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 213:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:2110
		{
			yyVAL.list = yyDollar[1].list

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 214:
		yyDollar = yyS[yypt-0 : yypt+1]
		// line php7/php7.y:2116
		{
			yyVAL.list = nil

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 215:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:2125
		{
			yyVAL.list = []node.Node{yyDollar[1].node}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 216:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:2131
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)

			// save comments
			yylex.(*Parser).setFreeFloating(lastNode(yyDollar[1].list), freefloating.End, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 217:
		yyDollar = yyS[yypt-4 : yypt+1]
		// line php7/php7.y:2143
		{
			identifier := node.NewIdentifier(strings.TrimLeftFunc(yyDollar[4].token.Value, isDollar))
			variable := expr.NewVariable(identifier)
			yyVAL.node = node.NewParameter(yyDollar[1].node, variable, nil, yyDollar[2].token != nil, yyDollar[3].token != nil)

			// save position
			identifier.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[4].token))
			variable.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[4].token))
			if yyDollar[1].node != nil {
				yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodeTokenPosition(yyDollar[1].node, yyDollar[4].token))
			} else if yyDollar[2].token != nil {
				yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[2].token, yyDollar[4].token))
			} else if yyDollar[3].token != nil {
				yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[3].token, yyDollar[4].token))
			} else {
				yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[4].token))
			}

			// save comments
			if yyDollar[1].node != nil {
				yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			}
			if yyDollar[2].token != nil {
				yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.OptionalType, yyDollar[2].token.FreeFloating)
			}
			if yyDollar[3].token != nil {
				yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Ampersand, yyDollar[3].token.FreeFloating)
			}
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Variadic, yyDollar[4].token.FreeFloating)
			yylex.(*Parser).addDollarToken(variable)

			// normalize
			if yyDollar[3].token == nil {
				yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Ampersand, (*yyVAL.node.GetFreeFloating())[freefloating.Variadic])
				delete((*yyVAL.node.GetFreeFloating()), freefloating.Variadic)
			}
			if yyDollar[2].token == nil {
				yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.OptionalType, (*yyVAL.node.GetFreeFloating())[freefloating.Ampersand])
				delete((*yyVAL.node.GetFreeFloating()), freefloating.Ampersand)
			}
			if yyDollar[1].node == nil {
				yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, (*yyVAL.node.GetFreeFloating())[freefloating.OptionalType])
				delete((*yyVAL.node.GetFreeFloating()), freefloating.OptionalType)
			}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 218:
		yyDollar = yyS[yypt-6 : yypt+1]
		// line php7/php7.y:2188
		{
			identifier := node.NewIdentifier(strings.TrimLeftFunc(yyDollar[4].token.Value, isDollar))
			variable := expr.NewVariable(identifier)
			yyVAL.node = node.NewParameter(yyDollar[1].node, variable, yyDollar[6].node, yyDollar[2].token != nil, yyDollar[3].token != nil)

			// save position
			identifier.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[4].token))
			variable.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[4].token))
			if yyDollar[1].node != nil {
				yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[6].node))
			} else if yyDollar[2].token != nil {
				yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[2].token, yyDollar[6].node))
			} else if yyDollar[3].token != nil {
				yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[3].token, yyDollar[6].node))
			} else {
				yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[4].token, yyDollar[6].node))
			}

			// save comments
			if yyDollar[1].node != nil {
				yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			}
			if yyDollar[2].token != nil {
				yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.OptionalType, yyDollar[2].token.FreeFloating)
			}
			if yyDollar[3].token != nil {
				yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Ampersand, yyDollar[3].token.FreeFloating)
			}
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Variadic, yyDollar[4].token.FreeFloating)
			yylex.(*Parser).addDollarToken(variable)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Var, yyDollar[5].token.FreeFloating)

			// normalize
			if yyDollar[3].token == nil {
				yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Ampersand, (*yyVAL.node.GetFreeFloating())[freefloating.Variadic])
				delete((*yyVAL.node.GetFreeFloating()), freefloating.Variadic)
			}
			if yyDollar[2].token == nil {
				yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.OptionalType, (*yyVAL.node.GetFreeFloating())[freefloating.Ampersand])
				delete((*yyVAL.node.GetFreeFloating()), freefloating.Ampersand)
			}
			if yyDollar[1].node == nil {
				yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, (*yyVAL.node.GetFreeFloating())[freefloating.OptionalType])
				delete((*yyVAL.node.GetFreeFloating()), freefloating.OptionalType)
			}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 219:
		yyDollar = yyS[yypt-0 : yypt+1]
		// line php7/php7.y:2237
		{
			yyVAL.node = nil

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 220:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:2243
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 221:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:2252
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 222:
		yyDollar = yyS[yypt-2 : yypt+1]
		// line php7/php7.y:2258
		{
			yyVAL.node = node.NewNullable(yyDollar[2].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 223:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:2273
		{
			yyVAL.node = node.NewIdentifier(yyDollar[1].token.Value)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 224:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:2285
		{
			yyVAL.node = node.NewIdentifier(yyDollar[1].token.Value)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 225:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:2297
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 226:
		yyDollar = yyS[yypt-0 : yypt+1]
		// line php7/php7.y:2306
		{
			yyVAL.node = nil

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 227:
		yyDollar = yyS[yypt-2 : yypt+1]
		// line php7/php7.y:2312
		{
			yyVAL.node = yyDollar[2].node

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Colon, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 228:
		yyDollar = yyS[yypt-2 : yypt+1]
		// line php7/php7.y:2324
		{
			yyVAL.node = node.NewArgumentList(nil)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[2].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.ArgumentList, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 229:
		yyDollar = yyS[yypt-4 : yypt+1]
		// line php7/php7.y:2337
		{
			yyVAL.node = node.NewArgumentList(yyDollar[2].list)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			if yyDollar[3].token != nil {
				yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.ArgumentList, append(yyDollar[3].token.FreeFloating, append(yylex.(*Parser).GetFreeFloatingToken(yyDollar[3].token), yyDollar[4].token.FreeFloating...)...))
			} else {
				yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.ArgumentList, yyDollar[4].token.FreeFloating)
			}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 230:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:2357
		{
			yyVAL.list = []node.Node{yyDollar[1].node}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 231:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:2363
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)

			// save comments
			yylex.(*Parser).setFreeFloating(lastNode(yyDollar[1].list), freefloating.End, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 232:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:2375
		{
			yyVAL.node = node.NewArgument(yyDollar[1].node, false, false)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodePosition(yyDollar[1].node))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 233:
		yyDollar = yyS[yypt-2 : yypt+1]
		// line php7/php7.y:2387
		{
			yyVAL.node = node.NewArgument(yyDollar[2].node, true, false)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 234:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:2402
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)

			// save comments
			yylex.(*Parser).setFreeFloating(lastNode(yyDollar[1].list), freefloating.End, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 235:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:2411
		{
			yyVAL.list = []node.Node{yyDollar[1].node}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 236:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:2420
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 237:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:2429
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)

			// save comments
			yylex.(*Parser).setFreeFloating(lastNode(yyDollar[1].list), freefloating.End, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 238:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:2438
		{
			yyVAL.list = []node.Node{yyDollar[1].node}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 239:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:2447
		{
			identifier := node.NewIdentifier(strings.TrimLeftFunc(yyDollar[1].token.Value, isDollar))
			variable := expr.NewVariable(identifier)
			yyVAL.node = stmt.NewStaticVar(variable, nil)

			// save position
			identifier.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			variable.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).addDollarToken(variable)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 240:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:2464
		{
			identifier := node.NewIdentifier(strings.TrimLeftFunc(yyDollar[1].token.Value, isDollar))
			variable := expr.NewVariable(identifier)
			yyVAL.node = stmt.NewStaticVar(variable, yyDollar[3].node)

			// save position
			identifier.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			variable.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[3].node))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).addDollarToken(variable)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Var, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 241:
		yyDollar = yyS[yypt-2 : yypt+1]
		// line php7/php7.y:2485
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[2].node)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 242:
		yyDollar = yyS[yypt-0 : yypt+1]
		// line php7/php7.y:2491
		{
			yyVAL.list = []node.Node{}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 243:
		yyDollar = yyS[yypt-4 : yypt+1]
		// line php7/php7.y:2500
		{
			yyVAL.node = stmt.NewPropertyList(yyDollar[1].list, yyDollar[2].node, yyDollar[3].list)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodeListTokenPosition(yyDollar[1].list, yyDollar[4].token))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].list[0], yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.PropertyList, yyDollar[4].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.SemiColon, yylex.(*Parser).GetFreeFloatingToken(yyDollar[4].token))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 244:
		yyDollar = yyS[yypt-4 : yypt+1]
		// line php7/php7.y:2514
		{
			yyVAL.node = stmt.NewClassConstList(yyDollar[1].list, yyDollar[3].list)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewOptionalListTokensPosition(yyDollar[1].list, yyDollar[2].token, yyDollar[4].token))

			// save comments
			if len(yyDollar[1].list) > 0 {
				yylex.(*Parser).MoveFreeFloating(yyDollar[1].list[0], yyVAL.node)
				yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.ModifierList, yyDollar[2].token.FreeFloating)
			} else {
				yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[2].token.FreeFloating)
			}
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.ConstList, yyDollar[4].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.SemiColon, yylex.(*Parser).GetFreeFloatingToken(yyDollar[4].token))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 245:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:2533
		{
			yyVAL.node = stmt.NewTraitUse(yyDollar[2].list, yyDollar[3].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[3].node))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 246:
		yyDollar = yyS[yypt-10 : yypt+1]
		// line php7/php7.y:2545
		{
			name := node.NewIdentifier(yyDollar[4].token.Value)
			yyVAL.node = stmt.NewClassMethod(name, yyDollar[1].list, yyDollar[3].token != nil, yyDollar[7].list, yyDollar[9].node, yyDollar[10].node, yyDollar[5].str)

			// save position
			name.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[4].token))
			if yyDollar[1].list == nil {
				yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[2].token, yyDollar[10].node))
			} else {
				yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodeListNodePosition(yyDollar[1].list, yyDollar[10].node))
			}

			// save comments
			if len(yyDollar[1].list) > 0 {
				yylex.(*Parser).MoveFreeFloating(yyDollar[1].list[0], yyVAL.node)
				yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.ModifierList, yyDollar[2].token.FreeFloating)
			} else {
				yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[2].token.FreeFloating)
			}
			if yyDollar[3].token == nil {
				yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Function, yyDollar[4].token.FreeFloating)
			} else {
				yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Function, yyDollar[3].token.FreeFloating)
				yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Ampersand, yyDollar[4].token.FreeFloating)
			}
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Name, yyDollar[6].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.ParameterList, yyDollar[8].token.FreeFloating)
			if yyDollar[9].node != nil {
				yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Params, (*yyDollar[9].node.GetFreeFloating())[freefloating.Colon])
				delete((*yyDollar[9].node.GetFreeFloating()), freefloating.Colon)
			}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 247:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:2582
		{
			yyVAL.list = []node.Node{yyDollar[1].node}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 248:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:2588
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)

			// save comments
			yylex.(*Parser).setFreeFloating(lastNode(yyDollar[1].list), freefloating.End, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 249:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:2600
		{
			yyVAL.node = stmt.NewNop()

			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.SemiColon, yylex.(*Parser).GetFreeFloatingToken(yyDollar[1].token))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 250:
		yyDollar = yyS[yypt-2 : yypt+1]
		// line php7/php7.y:2613
		{
			yyVAL.node = stmt.NewTraitAdaptationList(nil)

			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[2].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.AdaptationList, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 251:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:2625
		{
			yyVAL.node = stmt.NewTraitAdaptationList(yyDollar[2].list)

			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.AdaptationList, yyDollar[3].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 252:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:2640
		{
			yyVAL.list = []node.Node{yyDollar[1].node}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 253:
		yyDollar = yyS[yypt-2 : yypt+1]
		// line php7/php7.y:2646
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[2].node)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 254:
		yyDollar = yyS[yypt-2 : yypt+1]
		// line php7/php7.y:2655
		{
			yyVAL.node = yyDollar[1].node

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.NameList, yyDollar[2].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.SemiColon, yylex.(*Parser).GetFreeFloatingToken(yyDollar[2].token))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 255:
		yyDollar = yyS[yypt-2 : yypt+1]
		// line php7/php7.y:2665
		{
			yyVAL.node = yyDollar[1].node

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Alias, yyDollar[2].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.SemiColon, yylex.(*Parser).GetFreeFloatingToken(yyDollar[2].token))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 256:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:2678
		{
			yyVAL.node = stmt.NewTraitUsePrecedence(yyDollar[1].node, yyDollar[3].list)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodeNodeListPosition(yyDollar[1].node, yyDollar[3].list))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Ref, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 257:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:2694
		{
			alias := node.NewIdentifier(yyDollar[3].token.Value)
			yyVAL.node = stmt.NewTraitUseAlias(yyDollar[1].node, nil, alias)

			// save position
			alias.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[3].token))
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodeTokenPosition(yyDollar[1].node, yyDollar[3].token))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Ref, yyDollar[2].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(alias, freefloating.Start, yyDollar[3].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 258:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:2710
		{
			alias := node.NewIdentifier(yyDollar[3].token.Value)
			yyVAL.node = stmt.NewTraitUseAlias(yyDollar[1].node, nil, alias)

			// save position
			alias.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[3].token))
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodeTokenPosition(yyDollar[1].node, yyDollar[3].token))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Ref, yyDollar[2].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(alias, freefloating.Start, yyDollar[3].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 259:
		yyDollar = yyS[yypt-4 : yypt+1]
		// line php7/php7.y:2726
		{
			alias := node.NewIdentifier(yyDollar[4].token.Value)
			yyVAL.node = stmt.NewTraitUseAlias(yyDollar[1].node, yyDollar[3].node, alias)

			// save position
			alias.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[4].token))
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodeTokenPosition(yyDollar[1].node, yyDollar[4].token))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Ref, yyDollar[2].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(alias, freefloating.Start, yyDollar[4].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 260:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:2742
		{
			yyVAL.node = stmt.NewTraitUseAlias(yyDollar[1].node, yyDollar[3].node, nil)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Ref, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 261:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:2758
		{
			name := node.NewIdentifier(yyDollar[1].token.Value)
			yyVAL.node = stmt.NewTraitMethodRef(nil, name)

			// save position
			name.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 262:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:2772
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 263:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:2781
		{
			target := node.NewIdentifier(yyDollar[3].token.Value)
			yyVAL.node = stmt.NewTraitMethodRef(yyDollar[1].node, target)

			// save position
			target.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[3].token))
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodeTokenPosition(yyDollar[1].node, yyDollar[3].token))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Name, yyDollar[2].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(target, freefloating.Start, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 264:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:2800
		{
			yyVAL.node = stmt.NewNop()

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.SemiColon, yylex.(*Parser).GetFreeFloatingToken(yyDollar[1].token))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 265:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:2813
		{
			yyVAL.node = stmt.NewStmtList(yyDollar[2].list)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Stmts, yyDollar[3].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 266:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:2829
		{
			yyVAL.list = yyDollar[1].list

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 267:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:2835
		{
			modifier := node.NewIdentifier(yyDollar[1].token.Value)
			yyVAL.list = []node.Node{modifier}

			// save position
			modifier.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).setFreeFloating(modifier, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 268:
		yyDollar = yyS[yypt-0 : yypt+1]
		// line php7/php7.y:2851
		{
			yyVAL.list = nil

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 269:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:2857
		{
			yyVAL.list = yyDollar[1].list

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 270:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:2866
		{
			yyVAL.list = []node.Node{yyDollar[1].node}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 271:
		yyDollar = yyS[yypt-2 : yypt+1]
		// line php7/php7.y:2872
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[2].node)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 272:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:2881
		{
			yyVAL.node = node.NewIdentifier(yyDollar[1].token.Value)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 273:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:2893
		{
			yyVAL.node = node.NewIdentifier(yyDollar[1].token.Value)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 274:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:2905
		{
			yyVAL.node = node.NewIdentifier(yyDollar[1].token.Value)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 275:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:2917
		{
			yyVAL.node = node.NewIdentifier(yyDollar[1].token.Value)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 276:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:2929
		{
			yyVAL.node = node.NewIdentifier(yyDollar[1].token.Value)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 277:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:2941
		{
			yyVAL.node = node.NewIdentifier(yyDollar[1].token.Value)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 278:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:2956
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)

			// save comments
			yylex.(*Parser).setFreeFloating(lastNode(yyDollar[1].list), freefloating.End, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 279:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:2965
		{
			yyVAL.list = []node.Node{yyDollar[1].node}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 280:
		yyDollar = yyS[yypt-2 : yypt+1]
		// line php7/php7.y:2974
		{
			identifier := node.NewIdentifier(strings.TrimLeftFunc(yyDollar[1].token.Value, isDollar))
			variable := expr.NewVariable(identifier)
			yyVAL.node = stmt.NewProperty(variable, nil, yyDollar[2].str)

			// save position
			identifier.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			variable.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).addDollarToken(variable)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 281:
		yyDollar = yyS[yypt-4 : yypt+1]
		// line php7/php7.y:2991
		{
			identifier := node.NewIdentifier(strings.TrimLeftFunc(yyDollar[1].token.Value, isDollar))
			variable := expr.NewVariable(identifier)
			yyVAL.node = stmt.NewProperty(variable, yyDollar[3].node, yyDollar[4].str)

			// save position
			identifier.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			variable.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[3].node))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).addDollarToken(variable)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Var, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 282:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:3012
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)

			// save comments
			yylex.(*Parser).setFreeFloating(lastNode(yyDollar[1].list), freefloating.End, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 283:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:3021
		{
			yyVAL.list = []node.Node{yyDollar[1].node}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 284:
		yyDollar = yyS[yypt-4 : yypt+1]
		// line php7/php7.y:3030
		{
			name := node.NewIdentifier(yyDollar[1].token.Value)
			yyVAL.node = stmt.NewConstant(name, yyDollar[3].node, yyDollar[4].str)

			// save position
			name.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[3].node))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Name, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 285:
		yyDollar = yyS[yypt-4 : yypt+1]
		// line php7/php7.y:3048
		{
			name := node.NewIdentifier(yyDollar[1].token.Value)
			yyVAL.node = stmt.NewConstant(name, yyDollar[3].node, yyDollar[4].str)

			// save position
			name.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[3].node))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Name, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 286:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:3066
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)

			// save comments
			yylex.(*Parser).setFreeFloating(lastNode(yyDollar[1].list), freefloating.End, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 287:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:3075
		{
			yyVAL.list = []node.Node{yyDollar[1].node}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 288:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:3084
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 289:
		yyDollar = yyS[yypt-0 : yypt+1]
		// line php7/php7.y:3093
		{
			yyVAL.list = nil

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 290:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:3099
		{
			yyVAL.list = yyDollar[1].list

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 291:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:3108
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)

			// save comments
			yylex.(*Parser).setFreeFloating(lastNode(yyDollar[1].list), freefloating.End, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 292:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:3117
		{
			yyVAL.list = []node.Node{yyDollar[1].node}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 293:
		yyDollar = yyS[yypt-8 : yypt+1]
		// line php7/php7.y:3126
		{
			if yyDollar[2].node != nil {
				yyVAL.node = stmt.NewClass(nil, nil, yyDollar[2].node.(*node.ArgumentList), yyDollar[3].ClassExtends, yyDollar[4].ClassImplements, yyDollar[7].list, yyDollar[5].str)
			} else {
				yyVAL.node = stmt.NewClass(nil, nil, nil, yyDollar[3].ClassExtends, yyDollar[4].ClassImplements, yyDollar[7].list, yyDollar[5].str)
			}

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[8].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Name, yyDollar[6].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Stmts, yyDollar[8].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 294:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:3147
		{
			if yyDollar[3].node != nil {
				yyVAL.node = expr.NewNew(yyDollar[2].node, yyDollar[3].node.(*node.ArgumentList))
				yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[3].node))
			} else {
				yyVAL.node = expr.NewNew(yyDollar[2].node, nil)
				yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))
			}

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 295:
		yyDollar = yyS[yypt-2 : yypt+1]
		// line php7/php7.y:3162
		{
			yyVAL.node = expr.NewNew(yyDollar[2].node, nil)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 296:
		yyDollar = yyS[yypt-6 : yypt+1]
		// line php7/php7.y:3177
		{
			listNode := expr.NewList(yyDollar[3].list)
			yyVAL.node = assign.NewAssign(listNode, yyDollar[6].node)

			// save position
			listNode.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[6].node))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(listNode, freefloating.List, yyDollar[2].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(listNode, freefloating.ArrayPairList, yyDollar[4].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Var, yyDollar[5].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 297:
		yyDollar = yyS[yypt-5 : yypt+1]
		// line php7/php7.y:3194
		{
			shortList := expr.NewShortList(yyDollar[2].list)
			yyVAL.node = assign.NewAssign(shortList, yyDollar[5].node)

			// save position
			shortList.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[5].node))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(shortList, freefloating.ArrayPairList, yyDollar[3].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Var, yyDollar[4].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 298:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:3210
		{
			yyVAL.node = assign.NewAssign(yyDollar[1].node, yyDollar[3].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Var, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 299:
		yyDollar = yyS[yypt-4 : yypt+1]
		// line php7/php7.y:3223
		{
			yyVAL.node = assign.NewReference(yyDollar[1].node, yyDollar[4].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[4].node))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Var, yyDollar[2].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Equal, yyDollar[3].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 300:
		yyDollar = yyS[yypt-2 : yypt+1]
		// line php7/php7.y:3237
		{
			yyVAL.node = expr.NewClone(yyDollar[2].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 301:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:3249
		{
			yyVAL.node = assign.NewPlus(yyDollar[1].node, yyDollar[3].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Var, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 302:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:3262
		{
			yyVAL.node = assign.NewMinus(yyDollar[1].node, yyDollar[3].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Var, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 303:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:3275
		{
			yyVAL.node = assign.NewMul(yyDollar[1].node, yyDollar[3].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Var, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 304:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:3288
		{
			yyVAL.node = assign.NewPow(yyDollar[1].node, yyDollar[3].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Var, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 305:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:3301
		{
			yyVAL.node = assign.NewDiv(yyDollar[1].node, yyDollar[3].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Var, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 306:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:3314
		{
			yyVAL.node = assign.NewConcat(yyDollar[1].node, yyDollar[3].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Var, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 307:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:3327
		{
			yyVAL.node = assign.NewMod(yyDollar[1].node, yyDollar[3].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Var, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 308:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:3340
		{
			yyVAL.node = assign.NewBitwiseAnd(yyDollar[1].node, yyDollar[3].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Var, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 309:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:3353
		{
			yyVAL.node = assign.NewBitwiseOr(yyDollar[1].node, yyDollar[3].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Var, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 310:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:3366
		{
			yyVAL.node = assign.NewBitwiseXor(yyDollar[1].node, yyDollar[3].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Var, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 311:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:3379
		{
			yyVAL.node = assign.NewShiftLeft(yyDollar[1].node, yyDollar[3].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Var, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 312:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:3392
		{
			yyVAL.node = assign.NewShiftRight(yyDollar[1].node, yyDollar[3].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Var, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 313:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:3405
		{
			yyVAL.node = assign.NewCoalesce(yyDollar[1].node, yyDollar[3].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Var, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 314:
		yyDollar = yyS[yypt-2 : yypt+1]
		// line php7/php7.y:3418
		{
			yyVAL.node = expr.NewPostInc(yyDollar[1].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodeTokenPosition(yyDollar[1].node, yyDollar[2].token))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Var, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 315:
		yyDollar = yyS[yypt-2 : yypt+1]
		// line php7/php7.y:3431
		{
			yyVAL.node = expr.NewPreInc(yyDollar[2].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 316:
		yyDollar = yyS[yypt-2 : yypt+1]
		// line php7/php7.y:3443
		{
			yyVAL.node = expr.NewPostDec(yyDollar[1].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodeTokenPosition(yyDollar[1].node, yyDollar[2].token))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Var, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 317:
		yyDollar = yyS[yypt-2 : yypt+1]
		// line php7/php7.y:3456
		{
			yyVAL.node = expr.NewPreDec(yyDollar[2].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 318:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:3468
		{
			yyVAL.node = binary.NewBooleanOr(yyDollar[1].node, yyDollar[3].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Expr, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 319:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:3481
		{
			yyVAL.node = binary.NewBooleanAnd(yyDollar[1].node, yyDollar[3].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Expr, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 320:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:3494
		{
			yyVAL.node = binary.NewLogicalOr(yyDollar[1].node, yyDollar[3].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Expr, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 321:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:3507
		{
			yyVAL.node = binary.NewLogicalAnd(yyDollar[1].node, yyDollar[3].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Expr, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 322:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:3520
		{
			yyVAL.node = binary.NewLogicalXor(yyDollar[1].node, yyDollar[3].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Expr, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 323:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:3533
		{
			yyVAL.node = binary.NewBitwiseOr(yyDollar[1].node, yyDollar[3].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Expr, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 324:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:3546
		{
			yyVAL.node = binary.NewBitwiseAnd(yyDollar[1].node, yyDollar[3].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Expr, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 325:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:3559
		{
			yyVAL.node = binary.NewBitwiseXor(yyDollar[1].node, yyDollar[3].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Expr, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 326:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:3572
		{
			yyVAL.node = binary.NewConcat(yyDollar[1].node, yyDollar[3].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Expr, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 327:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:3585
		{
			yyVAL.node = binary.NewPlus(yyDollar[1].node, yyDollar[3].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Expr, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 328:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:3598
		{
			yyVAL.node = binary.NewMinus(yyDollar[1].node, yyDollar[3].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Expr, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 329:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:3611
		{
			yyVAL.node = binary.NewMul(yyDollar[1].node, yyDollar[3].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Expr, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 330:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:3624
		{
			yyVAL.node = binary.NewPow(yyDollar[1].node, yyDollar[3].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Expr, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 331:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:3637
		{
			yyVAL.node = binary.NewDiv(yyDollar[1].node, yyDollar[3].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Expr, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 332:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:3650
		{
			yyVAL.node = binary.NewMod(yyDollar[1].node, yyDollar[3].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Expr, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 333:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:3663
		{
			yyVAL.node = binary.NewShiftLeft(yyDollar[1].node, yyDollar[3].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Expr, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 334:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:3676
		{
			yyVAL.node = binary.NewShiftRight(yyDollar[1].node, yyDollar[3].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Expr, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 335:
		yyDollar = yyS[yypt-2 : yypt+1]
		// line php7/php7.y:3689
		{
			yyVAL.node = expr.NewUnaryPlus(yyDollar[2].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 336:
		yyDollar = yyS[yypt-2 : yypt+1]
		// line php7/php7.y:3701
		{
			yyVAL.node = expr.NewUnaryMinus(yyDollar[2].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 337:
		yyDollar = yyS[yypt-2 : yypt+1]
		// line php7/php7.y:3713
		{
			yyVAL.node = expr.NewBooleanNot(yyDollar[2].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 338:
		yyDollar = yyS[yypt-2 : yypt+1]
		// line php7/php7.y:3725
		{
			yyVAL.node = expr.NewBitwiseNot(yyDollar[2].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 339:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:3737
		{
			yyVAL.node = binary.NewIdentical(yyDollar[1].node, yyDollar[3].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Expr, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 340:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:3750
		{
			yyVAL.node = binary.NewNotIdentical(yyDollar[1].node, yyDollar[3].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Expr, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 341:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:3763
		{
			yyVAL.node = binary.NewEqual(yyDollar[1].node, yyDollar[3].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Expr, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 342:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:3776
		{
			yyVAL.node = binary.NewNotEqual(yyDollar[1].node, yyDollar[3].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Expr, yyDollar[2].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Equal, yylex.(*Parser).GetFreeFloatingToken(yyDollar[2].token))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 343:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:3790
		{
			yyVAL.node = binary.NewSmaller(yyDollar[1].node, yyDollar[3].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Expr, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 344:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:3803
		{
			yyVAL.node = binary.NewSmallerOrEqual(yyDollar[1].node, yyDollar[3].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Expr, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 345:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:3816
		{
			yyVAL.node = binary.NewGreater(yyDollar[1].node, yyDollar[3].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Expr, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 346:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:3829
		{
			yyVAL.node = binary.NewGreaterOrEqual(yyDollar[1].node, yyDollar[3].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Expr, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 347:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:3842
		{
			yyVAL.node = binary.NewSpaceship(yyDollar[1].node, yyDollar[3].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Expr, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 348:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:3855
		{
			yyVAL.node = expr.NewInstanceOf(yyDollar[1].node, yyDollar[3].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Expr, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 349:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:3868
		{
			yyVAL.node = yyDollar[2].node

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, append(yyDollar[1].token.FreeFloating, append(yylex.(*Parser).GetFreeFloatingToken(yyDollar[1].token), (*yyVAL.node.GetFreeFloating())[freefloating.Start]...)...))
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.End, append((*yyVAL.node.GetFreeFloating())[freefloating.End], append(yyDollar[3].token.FreeFloating, yylex.(*Parser).GetFreeFloatingToken(yyDollar[3].token)...)...))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 350:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:3878
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 351:
		yyDollar = yyS[yypt-5 : yypt+1]
		// line php7/php7.y:3884
		{
			yyVAL.node = expr.NewTernary(yyDollar[1].node, yyDollar[3].node, yyDollar[5].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[5].node))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Cond, yyDollar[2].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.True, yyDollar[4].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 352:
		yyDollar = yyS[yypt-4 : yypt+1]
		// line php7/php7.y:3898
		{
			yyVAL.node = expr.NewTernary(yyDollar[1].node, nil, yyDollar[4].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[4].node))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Cond, yyDollar[2].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.True, yyDollar[3].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 353:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:3912
		{
			yyVAL.node = binary.NewCoalesce(yyDollar[1].node, yyDollar[3].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Expr, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 354:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:3925
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 355:
		yyDollar = yyS[yypt-2 : yypt+1]
		// line php7/php7.y:3931
		{
			yyVAL.node = cast.NewInt(yyDollar[2].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Cast, yylex.(*Parser).GetFreeFloatingToken(yyDollar[1].token))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 356:
		yyDollar = yyS[yypt-2 : yypt+1]
		// line php7/php7.y:3944
		{
			yyVAL.node = cast.NewDouble(yyDollar[2].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Cast, yylex.(*Parser).GetFreeFloatingToken(yyDollar[1].token))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 357:
		yyDollar = yyS[yypt-2 : yypt+1]
		// line php7/php7.y:3957
		{
			yyVAL.node = cast.NewString(yyDollar[2].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Cast, yylex.(*Parser).GetFreeFloatingToken(yyDollar[1].token))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 358:
		yyDollar = yyS[yypt-2 : yypt+1]
		// line php7/php7.y:3970
		{
			yyVAL.node = cast.NewArray(yyDollar[2].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Cast, yylex.(*Parser).GetFreeFloatingToken(yyDollar[1].token))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 359:
		yyDollar = yyS[yypt-2 : yypt+1]
		// line php7/php7.y:3983
		{
			yyVAL.node = cast.NewObject(yyDollar[2].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Cast, yylex.(*Parser).GetFreeFloatingToken(yyDollar[1].token))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 360:
		yyDollar = yyS[yypt-2 : yypt+1]
		// line php7/php7.y:3996
		{
			yyVAL.node = cast.NewBool(yyDollar[2].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Cast, yylex.(*Parser).GetFreeFloatingToken(yyDollar[1].token))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 361:
		yyDollar = yyS[yypt-2 : yypt+1]
		// line php7/php7.y:4009
		{
			yyVAL.node = cast.NewUnset(yyDollar[2].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Cast, yylex.(*Parser).GetFreeFloatingToken(yyDollar[1].token))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 362:
		yyDollar = yyS[yypt-2 : yypt+1]
		// line php7/php7.y:4022
		{
			var e *expr.Exit
			if yyDollar[2].node != nil {
				e = yyDollar[2].node.(*expr.Exit)
			} else {
				e = expr.NewExit(nil)
			}

			yyVAL.node = e

			if strings.EqualFold(yyDollar[1].token.Value, "die") {
				e.Die = true
			}

			// save position
			if yyDollar[2].node == nil {
				yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			} else {
				yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))
			}

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 363:
		yyDollar = yyS[yypt-2 : yypt+1]
		// line php7/php7.y:4049
		{
			yyVAL.node = expr.NewErrorSuppress(yyDollar[2].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 364:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:4061
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 365:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:4067
		{
			yyVAL.node = expr.NewShellExec(yyDollar[2].list)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 366:
		yyDollar = yyS[yypt-2 : yypt+1]
		// line php7/php7.y:4079
		{
			yyVAL.node = expr.NewPrint(yyDollar[2].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 367:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:4091
		{
			yyVAL.node = expr.NewYield(nil, nil)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 368:
		yyDollar = yyS[yypt-2 : yypt+1]
		// line php7/php7.y:4103
		{
			yyVAL.node = expr.NewYield(nil, yyDollar[2].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 369:
		yyDollar = yyS[yypt-4 : yypt+1]
		// line php7/php7.y:4115
		{
			yyVAL.node = expr.NewYield(yyDollar[2].node, yyDollar[4].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[4].node))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Expr, yyDollar[3].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 370:
		yyDollar = yyS[yypt-2 : yypt+1]
		// line php7/php7.y:4128
		{
			yyVAL.node = expr.NewYieldFrom(yyDollar[2].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 371:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:4140
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 372:
		yyDollar = yyS[yypt-2 : yypt+1]
		// line php7/php7.y:4146
		{
			yyVAL.node = yyDollar[2].node

			switch n := yyVAL.node.(type) {
			case *expr.Closure:
				n.Static = true
			case *expr.ArrowFunction:
				n.Static = true
			}

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Static, (*yyVAL.node.GetFreeFloating())[freefloating.Start])
			delete((*yyVAL.node.GetFreeFloating()), freefloating.Start)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 373:
		yyDollar = yyS[yypt-11 : yypt+1]
		// line php7/php7.y:4169
		{
			yyVAL.node = expr.NewClosure(yyDollar[5].list, yyDollar[7].ClosureUse, yyDollar[8].node, yyDollar[10].list, false, yyDollar[2].token != nil, yyDollar[3].str)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[11].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			if yyDollar[2].token == nil {
				yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Function, yyDollar[4].token.FreeFloating)
			} else {
				yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Function, yyDollar[2].token.FreeFloating)
				yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Ampersand, yyDollar[4].token.FreeFloating)
			}
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.ParameterList, yyDollar[6].token.FreeFloating)
			if yyDollar[8].node != nil {
				yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.LexicalVars, (*yyDollar[8].node.GetFreeFloating())[freefloating.Colon])
				delete((*yyDollar[8].node.GetFreeFloating()), freefloating.Colon)
			}
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.ReturnType, yyDollar[9].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Stmts, yyDollar[11].token.FreeFloating)

			// normalize
			if yyDollar[8].node == nil {
				yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.LexicalVars, (*yyVAL.node.GetFreeFloating())[freefloating.ReturnType])
				delete((*yyVAL.node.GetFreeFloating()), freefloating.ReturnType)
			}
			if yyDollar[7].ClosureUse == nil {
				yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Params, (*yyVAL.node.GetFreeFloating())[freefloating.LexicalVarList])
				delete((*yyVAL.node.GetFreeFloating()), freefloating.LexicalVarList)
			}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 374:
		yyDollar = yyS[yypt-9 : yypt+1]
		// line php7/php7.y:4201
		{
			yyVAL.node = expr.NewArrowFunction(yyDollar[4].list, yyDollar[6].node, yyDollar[9].node, false, yyDollar[2].token != nil, yyDollar[7].str)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[9].node))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			if yyDollar[2].token == nil {
				yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Function, yyDollar[3].token.FreeFloating)
			} else {
				yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Function, yyDollar[2].token.FreeFloating)
				yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Ampersand, yyDollar[3].token.FreeFloating)
			}
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.ParameterList, yyDollar[5].token.FreeFloating)
			if yyDollar[6].node != nil {
				yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Params, (*yyDollar[6].node.GetFreeFloating())[freefloating.Colon])
				delete((*yyDollar[6].node.GetFreeFloating()), freefloating.Colon)
			}
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.ReturnType, yyDollar[8].token.FreeFloating)

			// normalize
			if yyDollar[6].node == nil {
				yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Params, (*yyVAL.node.GetFreeFloating())[freefloating.ReturnType])
				delete((*yyVAL.node.GetFreeFloating()), freefloating.ReturnType)
			}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 375:
		yyDollar = yyS[yypt-0 : yypt+1]
		// line php7/php7.y:4232
		{
			yyVAL.str = yylex.(*Parser).Lexer.GetPhpDocComment()
			yylex.(*Parser).Lexer.SetPhpDocComment("")

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 376:
		yyDollar = yyS[yypt-0 : yypt+1]
		// line php7/php7.y:4242
		{
			yyVAL.token = nil
		}
	case 377:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:4246
		{
			yyVAL.token = yyDollar[1].token
		}
	case 378:
		yyDollar = yyS[yypt-0 : yypt+1]
		// line php7/php7.y:4253
		{
			yyVAL.ClosureUse = nil

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 379:
		yyDollar = yyS[yypt-4 : yypt+1]
		// line php7/php7.y:4259
		{
			yyVAL.ClosureUse = expr.NewClosureUse(yyDollar[3].list)

			// save position
			yyVAL.ClosureUse.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.ClosureUse, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.ClosureUse, freefloating.Use, yyDollar[2].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.ClosureUse, freefloating.LexicalVarList, yyDollar[4].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 380:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:4276
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)

			// save comments
			yylex.(*Parser).setFreeFloating(lastNode(yyDollar[1].list), freefloating.End, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 381:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:4285
		{
			yyVAL.list = []node.Node{yyDollar[1].node}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 382:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:4294
		{
			identifier := node.NewIdentifier(strings.TrimLeftFunc(yyDollar[1].token.Value, isDollar))
			yyVAL.node = expr.NewVariable(identifier)

			// save position
			identifier.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).addDollarToken(yyVAL.node)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 383:
		yyDollar = yyS[yypt-2 : yypt+1]
		// line php7/php7.y:4309
		{
			identifier := node.NewIdentifier(strings.TrimLeftFunc(yyDollar[2].token.Value, isDollar))
			variable := expr.NewVariable(identifier)
			yyVAL.node = expr.NewReference(variable)

			// save position
			identifier.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[2].token))
			variable.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[2].token))
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[2].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(variable, freefloating.Start, yyDollar[2].token.FreeFloating)
			yylex.(*Parser).addDollarToken(variable)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 384:
		yyDollar = yyS[yypt-2 : yypt+1]
		// line php7/php7.y:4330
		{
			yyVAL.node = expr.NewFunctionCall(yyDollar[1].node, yyDollar[2].node.(*node.ArgumentList))

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[2].node))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 385:
		yyDollar = yyS[yypt-4 : yypt+1]
		// line php7/php7.y:4342
		{
			yyVAL.node = expr.NewStaticCall(yyDollar[1].node, yyDollar[3].node, yyDollar[4].node.(*node.ArgumentList))

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[4].node))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Name, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 386:
		yyDollar = yyS[yypt-4 : yypt+1]
		// line php7/php7.y:4355
		{
			yyVAL.node = expr.NewStaticCall(yyDollar[1].node, yyDollar[3].node, yyDollar[4].node.(*node.ArgumentList))

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[4].node))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Name, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 387:
		yyDollar = yyS[yypt-2 : yypt+1]
		// line php7/php7.y:4368
		{
			yyVAL.node = expr.NewFunctionCall(yyDollar[1].node, yyDollar[2].node.(*node.ArgumentList))

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[2].node))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 388:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:4383
		{
			yyVAL.node = node.NewIdentifier(yyDollar[1].token.Value)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 389:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:4395
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 390:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:4404
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 391:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:4410
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 392:
		yyDollar = yyS[yypt-0 : yypt+1]
		// line php7/php7.y:4419
		{
			yyVAL.node = nil

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 393:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:4425
		{
			yyVAL.node = expr.NewExit(yyDollar[2].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Exit, append(yyDollar[1].token.FreeFloating, yylex.(*Parser).GetFreeFloatingToken(yyDollar[1].token)...))
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Expr, append(yyDollar[3].token.FreeFloating, yylex.(*Parser).GetFreeFloatingToken(yyDollar[3].token)...))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 394:
		yyDollar = yyS[yypt-0 : yypt+1]
		// line php7/php7.y:4441
		{
			yyVAL.list = []node.Node{}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 395:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:4447
		{
			part := scalar.NewEncapsedStringPart(yyDollar[1].token.Value)
			yyVAL.list = []node.Node{part}

			// save position
			part.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 396:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:4457
		{
			yyVAL.list = yyDollar[1].list

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 397:
		yyDollar = yyS[yypt-0 : yypt+1]
		// line php7/php7.y:4466
		{
			yyVAL.node = nil

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 398:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:4472
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 399:
		yyDollar = yyS[yypt-4 : yypt+1]
		// line php7/php7.y:4481
		{
			yyVAL.node = expr.NewArray(yyDollar[3].list)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Array, yyDollar[2].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.ArrayPairList, yyDollar[4].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 400:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:4495
		{
			yyVAL.node = expr.NewShortArray(yyDollar[2].list)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.ArrayPairList, yyDollar[3].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 401:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:4508
		{
			yyVAL.node = scalar.NewString(yyDollar[1].token.Value)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 402:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:4523
		{
			yyVAL.node = scalar.NewLnumber(yyDollar[1].token.Value)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 403:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:4535
		{
			yyVAL.node = scalar.NewDnumber(yyDollar[1].token.Value)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 404:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:4547
		{
			yyVAL.node = scalar.NewMagicConstant(yyDollar[1].token.Value)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 405:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:4559
		{
			yyVAL.node = scalar.NewMagicConstant(yyDollar[1].token.Value)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 406:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:4571
		{
			yyVAL.node = scalar.NewMagicConstant(yyDollar[1].token.Value)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 407:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:4583
		{
			yyVAL.node = scalar.NewMagicConstant(yyDollar[1].token.Value)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 408:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:4595
		{
			yyVAL.node = scalar.NewMagicConstant(yyDollar[1].token.Value)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 409:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:4607
		{
			yyVAL.node = scalar.NewMagicConstant(yyDollar[1].token.Value)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 410:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:4619
		{
			yyVAL.node = scalar.NewMagicConstant(yyDollar[1].token.Value)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 411:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:4631
		{
			yyVAL.node = scalar.NewMagicConstant(yyDollar[1].token.Value)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 412:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:4643
		{
			encapsed := scalar.NewEncapsedStringPart(yyDollar[2].token.Value)
			yyVAL.node = scalar.NewHeredoc(yyDollar[1].token.Value, []node.Node{encapsed})

			// save position
			encapsed.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[2].token))
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 413:
		yyDollar = yyS[yypt-2 : yypt+1]
		// line php7/php7.y:4657
		{
			yyVAL.node = scalar.NewHeredoc(yyDollar[1].token.Value, nil)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[2].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 414:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:4669
		{
			yyVAL.node = scalar.NewEncapsed(yyDollar[2].list)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 415:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:4681
		{
			yyVAL.node = scalar.NewHeredoc(yyDollar[1].token.Value, yyDollar[2].list)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 416:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:4693
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 417:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:4699
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 418:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:4708
		{
			yyVAL.node = expr.NewConstFetch(yyDollar[1].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodePosition(yyDollar[1].node))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 419:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:4720
		{
			target := node.NewIdentifier(yyDollar[3].token.Value)
			yyVAL.node = expr.NewClassConstFetch(yyDollar[1].node, target)

			// save position
			target.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[3].token))
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodeTokenPosition(yyDollar[1].node, yyDollar[3].token))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Name, yyDollar[2].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(target, freefloating.Start, yyDollar[3].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 420:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:4736
		{
			target := node.NewIdentifier(yyDollar[3].token.Value)
			yyVAL.node = expr.NewClassConstFetch(yyDollar[1].node, target)

			// save position
			target.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[3].token))
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodeTokenPosition(yyDollar[1].node, yyDollar[3].token))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Name, yyDollar[2].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(target, freefloating.Start, yyDollar[3].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 421:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:4755
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 422:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:4761
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 423:
		yyDollar = yyS[yypt-0 : yypt+1]
		// line php7/php7.y:4770
		{
			yyVAL.node = nil

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 424:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:4776
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 425:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:4785
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 426:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:4794
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 427:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:4800
		{
			yyVAL.node = yyDollar[2].node

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, append(yyDollar[1].token.FreeFloating, append(yylex.(*Parser).GetFreeFloatingToken(yyDollar[1].token), (*yyVAL.node.GetFreeFloating())[freefloating.Start]...)...))
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.End, append((*yyVAL.node.GetFreeFloating())[freefloating.End], append(yyDollar[3].token.FreeFloating, yylex.(*Parser).GetFreeFloatingToken(yyDollar[3].token)...)...))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 428:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:4810
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 429:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:4819
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 430:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:4825
		{
			yyVAL.node = yyDollar[2].node

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, append(yyDollar[1].token.FreeFloating, append(yylex.(*Parser).GetFreeFloatingToken(yyDollar[1].token), (*yyVAL.node.GetFreeFloating())[freefloating.Start]...)...))
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.End, append((*yyVAL.node.GetFreeFloating())[freefloating.End], append(yyDollar[3].token.FreeFloating, yylex.(*Parser).GetFreeFloatingToken(yyDollar[3].token)...)...))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 431:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:4835
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 432:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:4844
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 433:
		yyDollar = yyS[yypt-4 : yypt+1]
		// line php7/php7.y:4850
		{
			yyVAL.node = expr.NewArrayDimFetch(yyDollar[1].node, yyDollar[3].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodeTokenPosition(yyDollar[1].node, yyDollar[4].token))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Var, append(yyDollar[2].token.FreeFloating, yylex.(*Parser).GetFreeFloatingToken(yyDollar[2].token)...))
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Expr, append(yyDollar[4].token.FreeFloating, yylex.(*Parser).GetFreeFloatingToken(yyDollar[4].token)...))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 434:
		yyDollar = yyS[yypt-4 : yypt+1]
		// line php7/php7.y:4864
		{
			yyVAL.node = expr.NewArrayDimFetch(yyDollar[1].node, yyDollar[3].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodeTokenPosition(yyDollar[1].node, yyDollar[4].token))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Var, append(yyDollar[2].token.FreeFloating, yylex.(*Parser).GetFreeFloatingToken(yyDollar[2].token)...))
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Expr, append(yyDollar[4].token.FreeFloating, yylex.(*Parser).GetFreeFloatingToken(yyDollar[4].token)...))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 435:
		yyDollar = yyS[yypt-4 : yypt+1]
		// line php7/php7.y:4878
		{
			yyVAL.node = expr.NewArrayDimFetch(yyDollar[1].node, yyDollar[3].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodeTokenPosition(yyDollar[1].node, yyDollar[4].token))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Var, append(yyDollar[2].token.FreeFloating, yylex.(*Parser).GetFreeFloatingToken(yyDollar[2].token)...))
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Expr, append(yyDollar[4].token.FreeFloating, yylex.(*Parser).GetFreeFloatingToken(yyDollar[4].token)...))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 436:
		yyDollar = yyS[yypt-4 : yypt+1]
		// line php7/php7.y:4892
		{
			yyVAL.node = expr.NewMethodCall(yyDollar[1].node, yyDollar[3].node, yyDollar[4].node.(*node.ArgumentList))

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[4].node))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Var, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 437:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:4905
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 438:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:4914
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 439:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:4920
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 440:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:4926
		{
			yyVAL.node = expr.NewPropertyFetch(yyDollar[1].node, yyDollar[3].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Var, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 441:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:4942
		{
			name := node.NewIdentifier(strings.TrimLeftFunc(yyDollar[1].token.Value, isDollar))
			yyVAL.node = expr.NewVariable(name)

			// save position
			name.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).addDollarToken(yyVAL.node)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 442:
		yyDollar = yyS[yypt-4 : yypt+1]
		// line php7/php7.y:4957
		{
			yyVAL.node = expr.NewVariable(yyDollar[3].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Dollar, yylex.(*Parser).GetFreeFloatingToken(yyDollar[1].token))
			yylex.(*Parser).setFreeFloating(yyDollar[3].node, freefloating.Start, append(yyDollar[2].token.FreeFloating, append(yylex.(*Parser).GetFreeFloatingToken(yyDollar[2].token), (*yyDollar[3].node.GetFreeFloating())[freefloating.Start]...)...))
			yylex.(*Parser).setFreeFloating(yyDollar[3].node, freefloating.End, append((*yyDollar[3].node.GetFreeFloating())[freefloating.End], append(yyDollar[4].token.FreeFloating, yylex.(*Parser).GetFreeFloatingToken(yyDollar[4].token)...)...))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 443:
		yyDollar = yyS[yypt-2 : yypt+1]
		// line php7/php7.y:4972
		{
			yyVAL.node = expr.NewVariable(yyDollar[2].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Dollar, yylex.(*Parser).GetFreeFloatingToken(yyDollar[1].token))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 444:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:4988
		{
			yyVAL.node = expr.NewStaticPropertyFetch(yyDollar[1].node, yyDollar[3].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Name, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 445:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:5001
		{
			yyVAL.node = expr.NewStaticPropertyFetch(yyDollar[1].node, yyDollar[3].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Name, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 446:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:5017
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 447:
		yyDollar = yyS[yypt-4 : yypt+1]
		// line php7/php7.y:5023
		{
			yyVAL.node = expr.NewArrayDimFetch(yyDollar[1].node, yyDollar[3].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodeTokenPosition(yyDollar[1].node, yyDollar[4].token))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Var, append(yyDollar[2].token.FreeFloating, yylex.(*Parser).GetFreeFloatingToken(yyDollar[2].token)...))
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Expr, append(yyDollar[4].token.FreeFloating, yylex.(*Parser).GetFreeFloatingToken(yyDollar[4].token)...))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 448:
		yyDollar = yyS[yypt-4 : yypt+1]
		// line php7/php7.y:5037
		{
			yyVAL.node = expr.NewArrayDimFetch(yyDollar[1].node, yyDollar[3].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodeTokenPosition(yyDollar[1].node, yyDollar[4].token))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Var, append(yyDollar[2].token.FreeFloating, yylex.(*Parser).GetFreeFloatingToken(yyDollar[2].token)...))
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Expr, append(yyDollar[4].token.FreeFloating, yylex.(*Parser).GetFreeFloatingToken(yyDollar[4].token)...))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 449:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:5051
		{
			yyVAL.node = expr.NewPropertyFetch(yyDollar[1].node, yyDollar[3].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Var, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 450:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:5064
		{
			yyVAL.node = expr.NewStaticPropertyFetch(yyDollar[1].node, yyDollar[3].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Var, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 451:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:5077
		{
			yyVAL.node = expr.NewStaticPropertyFetch(yyDollar[1].node, yyDollar[3].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Var, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 452:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:5093
		{
			yyVAL.node = node.NewIdentifier(yyDollar[1].token.Value)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 453:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:5105
		{
			yyVAL.node = yyDollar[2].node

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, append(yyDollar[1].token.FreeFloating, append(yylex.(*Parser).GetFreeFloatingToken(yyDollar[1].token), (*yyVAL.node.GetFreeFloating())[freefloating.Start]...)...))
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.End, append((*yyVAL.node.GetFreeFloating())[freefloating.End], append(yyDollar[3].token.FreeFloating, yylex.(*Parser).GetFreeFloatingToken(yyDollar[3].token)...)...))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 454:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:5115
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 455:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:5124
		{
			yyVAL.node = node.NewIdentifier(yyDollar[1].token.Value)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 456:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:5136
		{
			yyVAL.node = yyDollar[2].node

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, append(yyDollar[1].token.FreeFloating, append(yylex.(*Parser).GetFreeFloatingToken(yyDollar[1].token), (*yyVAL.node.GetFreeFloating())[freefloating.Start]...)...))
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.End, append((*yyVAL.node.GetFreeFloating())[freefloating.End], append(yyDollar[3].token.FreeFloating, yylex.(*Parser).GetFreeFloatingToken(yyDollar[3].token)...)...))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 457:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:5146
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 458:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:5155
		{
			yyVAL.list = yyDollar[1].list

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 459:
		yyDollar = yyS[yypt-0 : yypt+1]
		// line php7/php7.y:5164
		{
			yyVAL.node = expr.NewArrayItem(nil, nil, false)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 460:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:5170
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 461:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:5179
		{
			if len(yyDollar[1].list) == 0 {
				yyDollar[1].list = []node.Node{expr.NewArrayItem(nil, nil, false)}
			}

			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)

			// save comments
			yylex.(*Parser).setFreeFloating(lastNode(yyDollar[1].list), freefloating.End, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 462:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:5192
		{
			if yyDollar[1].node.(*expr.ArrayItem).Key == nil && yyDollar[1].node.(*expr.ArrayItem).Val == nil {
				yyVAL.list = []node.Node{}
			} else {
				yyVAL.list = []node.Node{yyDollar[1].node}
			}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 463:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:5205
		{
			yyVAL.node = expr.NewArrayItem(yyDollar[1].node, yyDollar[3].node, false)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Expr, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 464:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:5218
		{
			yyVAL.node = expr.NewArrayItem(nil, yyDollar[1].node, false)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodePosition(yyDollar[1].node))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 465:
		yyDollar = yyS[yypt-4 : yypt+1]
		// line php7/php7.y:5230
		{
			reference := expr.NewReference(yyDollar[4].node)
			yyVAL.node = expr.NewArrayItem(yyDollar[1].node, reference, false)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[4].node))
			reference.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[3].token, yyDollar[4].node))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Expr, yyDollar[2].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(reference, freefloating.Start, yyDollar[3].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 466:
		yyDollar = yyS[yypt-2 : yypt+1]
		// line php7/php7.y:5246
		{
			reference := expr.NewReference(yyDollar[2].node)
			yyVAL.node = expr.NewArrayItem(nil, reference, false)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))
			reference.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 467:
		yyDollar = yyS[yypt-2 : yypt+1]
		// line php7/php7.y:5260
		{
			yyVAL.node = expr.NewArrayItem(nil, yyDollar[2].node, true)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 468:
		yyDollar = yyS[yypt-6 : yypt+1]
		// line php7/php7.y:5272
		{
			// TODO: Cannot use list() as standalone expression
			listNode := expr.NewList(yyDollar[5].list)
			yyVAL.node = expr.NewArrayItem(yyDollar[1].node, listNode, false)

			// save position
			listNode.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[3].token, yyDollar[6].token))
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewNodeTokenPosition(yyDollar[1].node, yyDollar[6].token))

			// save comments
			yylex.(*Parser).MoveFreeFloating(yyDollar[1].node, yyVAL.node)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Expr, yyDollar[2].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(listNode, freefloating.Start, yyDollar[3].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(listNode, freefloating.List, yyDollar[4].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(listNode, freefloating.ArrayPairList, yyDollar[6].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 469:
		yyDollar = yyS[yypt-4 : yypt+1]
		// line php7/php7.y:5291
		{
			// TODO: Cannot use list() as standalone expression
			listNode := expr.NewList(yyDollar[3].list)
			yyVAL.node = expr.NewArrayItem(nil, listNode, false)

			// save position
			listNode.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(listNode, freefloating.List, yyDollar[2].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(listNode, freefloating.ArrayPairList, yyDollar[4].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 470:
		yyDollar = yyS[yypt-2 : yypt+1]
		// line php7/php7.y:5311
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[2].node)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 471:
		yyDollar = yyS[yypt-2 : yypt+1]
		// line php7/php7.y:5317
		{
			encapsed := scalar.NewEncapsedStringPart(yyDollar[2].token.Value)
			yyVAL.list = append(yyDollar[1].list, encapsed)

			// save position
			encapsed.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[2].token))

			// save comments
			yylex.(*Parser).setFreeFloating(encapsed, freefloating.Start, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 472:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:5330
		{
			yyVAL.list = []node.Node{yyDollar[1].node}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 473:
		yyDollar = yyS[yypt-2 : yypt+1]
		// line php7/php7.y:5336
		{
			encapsed := scalar.NewEncapsedStringPart(yyDollar[1].token.Value)
			yyVAL.list = []node.Node{encapsed, yyDollar[2].node}

			// save position
			encapsed.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).setFreeFloating(encapsed, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 474:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:5352
		{
			name := node.NewIdentifier(strings.TrimLeftFunc(yyDollar[1].token.Value, isDollar))
			yyVAL.node = expr.NewVariable(name)

			// save position
			name.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).addDollarToken(yyVAL.node)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 475:
		yyDollar = yyS[yypt-4 : yypt+1]
		// line php7/php7.y:5367
		{
			identifier := node.NewIdentifier(strings.TrimLeftFunc(yyDollar[1].token.Value, isDollar))
			variable := expr.NewVariable(identifier)
			yyVAL.node = expr.NewArrayDimFetch(variable, yyDollar[3].node)

			// save position
			identifier.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			variable.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))

			// save comments
			yylex.(*Parser).addDollarToken(variable)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Var, append(yyDollar[2].token.FreeFloating, yylex.(*Parser).GetFreeFloatingToken(yyDollar[2].token)...))
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Expr, append(yyDollar[4].token.FreeFloating, yylex.(*Parser).GetFreeFloatingToken(yyDollar[4].token)...))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 476:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:5385
		{
			identifier := node.NewIdentifier(strings.TrimLeftFunc(yyDollar[1].token.Value, isDollar))
			variable := expr.NewVariable(identifier)
			fetch := node.NewIdentifier(yyDollar[3].token.Value)
			yyVAL.node = expr.NewPropertyFetch(variable, fetch)

			// save position
			identifier.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			variable.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			fetch.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[3].token))
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))

			// save comments
			yylex.(*Parser).addDollarToken(variable)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Var, yyDollar[2].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(fetch, freefloating.Start, yyDollar[3].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 477:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:5405
		{
			variable := expr.NewVariable(yyDollar[2].node)

			yyVAL.node = variable

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yylex.(*Parser).GetFreeFloatingToken(yyDollar[1].token))
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.End, append(yyDollar[3].token.FreeFloating, yylex.(*Parser).GetFreeFloatingToken(yyDollar[3].token)...))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 478:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:5420
		{
			name := node.NewIdentifier(yyDollar[2].token.Value)
			variable := expr.NewVariable(name)

			yyVAL.node = variable

			// save position
			name.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[2].token))
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yylex.(*Parser).GetFreeFloatingToken(yyDollar[1].token))
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.End, append(yyDollar[3].token.FreeFloating, yylex.(*Parser).GetFreeFloatingToken(yyDollar[3].token)...))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 479:
		yyDollar = yyS[yypt-6 : yypt+1]
		// line php7/php7.y:5437
		{
			identifier := node.NewIdentifier(yyDollar[2].token.Value)
			variable := expr.NewVariable(identifier)
			yyVAL.node = expr.NewArrayDimFetch(variable, yyDollar[4].node)

			// save position
			identifier.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[2].token))
			variable.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[2].token))
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[6].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yylex.(*Parser).GetFreeFloatingToken(yyDollar[1].token))
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Var, append(yyDollar[3].token.FreeFloating, yylex.(*Parser).GetFreeFloatingToken(yyDollar[3].token)...))
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Expr, append(yyDollar[5].token.FreeFloating, yylex.(*Parser).GetFreeFloatingToken(yyDollar[5].token)...))
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.End, append(yyDollar[6].token.FreeFloating, yylex.(*Parser).GetFreeFloatingToken(yyDollar[6].token)...))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 480:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:5456
		{
			yyVAL.node = yyDollar[2].node

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yylex.(*Parser).GetFreeFloatingToken(yyDollar[1].token))
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.End, append(yyDollar[3].token.FreeFloating, yylex.(*Parser).GetFreeFloatingToken(yyDollar[3].token)...))

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 481:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:5469
		{
			yyVAL.node = scalar.NewString(yyDollar[1].token.Value)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 482:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:5481
		{
			// TODO: add option to handle 64 bit integer
			if _, err := strconv.Atoi(yyDollar[1].token.Value); err == nil {
				yyVAL.node = scalar.NewLnumber(yyDollar[1].token.Value)
			} else {
				yyVAL.node = scalar.NewString(yyDollar[1].token.Value)
			}

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 483:
		yyDollar = yyS[yypt-2 : yypt+1]
		// line php7/php7.y:5498
		{
			var lnumber *scalar.Lnumber
			// TODO: add option to handle 64 bit integer
			_, err := strconv.Atoi(yyDollar[2].token.Value)
			isInt := err == nil

			if isInt {
				lnumber = scalar.NewLnumber(yyDollar[2].token.Value)
				yyVAL.node = expr.NewUnaryMinus(lnumber)
			} else {
				yyDollar[2].token.Value = "-" + yyDollar[2].token.Value
				yyVAL.node = scalar.NewString(yyDollar[2].token.Value)
			}

			// save position
			if isInt {
				lnumber.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[2].token))
			}
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[2].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 484:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:5524
		{
			identifier := node.NewIdentifier(strings.TrimLeftFunc(yyDollar[1].token.Value, isDollar))
			yyVAL.node = expr.NewVariable(identifier)

			// save position
			identifier.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).addDollarToken(yyVAL.node)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 485:
		yyDollar = yyS[yypt-5 : yypt+1]
		// line php7/php7.y:5542
		{
			yyVAL.node = expr.NewIsset(yyDollar[3].list)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[5].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Isset, yyDollar[2].token.FreeFloating)
			if yyDollar[4].token == nil {
				yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.VarList, yyDollar[5].token.FreeFloating)
			} else {
				yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.VarList, append(yyDollar[4].token.FreeFloating, append(yylex.(*Parser).GetFreeFloatingToken(yyDollar[4].token), yyDollar[5].token.FreeFloating...)...))
			}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 486:
		yyDollar = yyS[yypt-4 : yypt+1]
		// line php7/php7.y:5560
		{
			yyVAL.node = expr.NewEmpty(yyDollar[3].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Empty, yyDollar[2].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Expr, yyDollar[4].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 487:
		yyDollar = yyS[yypt-2 : yypt+1]
		// line php7/php7.y:5574
		{
			yyVAL.node = expr.NewInclude(yyDollar[2].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 488:
		yyDollar = yyS[yypt-2 : yypt+1]
		// line php7/php7.y:5586
		{
			yyVAL.node = expr.NewIncludeOnce(yyDollar[2].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 489:
		yyDollar = yyS[yypt-4 : yypt+1]
		// line php7/php7.y:5598
		{
			yyVAL.node = expr.NewEval(yyDollar[3].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Eval, yyDollar[2].token.FreeFloating)
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Expr, yyDollar[4].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 490:
		yyDollar = yyS[yypt-2 : yypt+1]
		// line php7/php7.y:5612
		{
			yyVAL.node = expr.NewRequire(yyDollar[2].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 491:
		yyDollar = yyS[yypt-2 : yypt+1]
		// line php7/php7.y:5624
		{
			yyVAL.node = expr.NewRequireOnce(yyDollar[2].node)

			// save position
			yyVAL.node.SetPosition(yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))

			// save comments
			yylex.(*Parser).setFreeFloating(yyVAL.node, freefloating.Start, yyDollar[1].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 492:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:5639
		{
			yyVAL.list = []node.Node{yyDollar[1].node}

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 493:
		yyDollar = yyS[yypt-3 : yypt+1]
		// line php7/php7.y:5645
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)

			// save comments
			yylex.(*Parser).setFreeFloating(lastNode(yyDollar[1].list), freefloating.End, yyDollar[2].token.FreeFloating)

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	case 494:
		yyDollar = yyS[yypt-1 : yypt+1]
		// line php7/php7.y:5657
		{
			yyVAL.node = yyDollar[1].node

			yylex.(*Parser).returnTokenToPool(yyDollar, &yyVAL)
		}
	}
	goto yystack /* stack new state and value */
}
